@@ -137,7 +137,7 @@ extern int	errno;
 */
 #define GKRELLMD_VERSION_MAJOR   2
 #define GKRELLMD_VERSION_MINOR   1
-#define GKRELLMD_VERSION_REV     10
+#define GKRELLMD_VERSION_REV     16
 #define GKRELLMD_EXTRAVERSION    ""
 
 #define GKRELLMD_CONFIG     "gkrellmd.conf"
@@ -168,6 +168,7 @@ typedef struct
 	gint		fd;
 	gchar		*hostname;
 	gboolean	served,
+				alive,
 				last_client;
 	gboolean	feature_subdisk;
 	gchar		buffer[2048];
@@ -214,16 +215,20 @@ struct GkrellmdConfig
 	gchar		*pidfile;
 	};
 
+
+
 extern struct GkrellmdConfig	_GK;
 extern GkrellmdTicks			GK;
 
-extern GList		*gkrellmd_client_list;
+extern GList	*gkrellmd_client_list;
 
-void	gkrellm_client_read(gint client_fd, gint nbytes);
-gint	gkrellm_get_timer_ticks(void);
+void			gkrellmd_client_read(gint client_fd, gint nbytes);
 
-void	load_monitors(void);
-gint	update_monitors(void);
-void	gkrellmd_serve_setup(GkrellmdClient *client);
-gint	send_to_client(gint fd, gchar *buf);
-void	free_glist_and_data(GList **);
+GkrellmdMonitor *gkrellmd_init_mail_monitor(void);
+void			gkrellmd_load_monitors(void);
+gint			gkrellmd_update_monitors(void);
+void			gkrellmd_serve_setup(GkrellmdClient *client);
+gint			gkrellmd_send_to_client(GkrellmdClient *client, gchar *buf);
+void			gkrellmd_free_glist_and_data(GList **);
+void			gkrellmd_add_mailbox(gchar *);
+void			gkrellmd_add_serveflag_done(gboolean *);
