@@ -21,7 +21,8 @@
 
 #include "gkrellmd.h"
 
-GList	*gkrellmd_monitor_list;
+
+GList			*gkrellmd_monitor_list;
 
 static GList	*serveflag_done_list;
 
@@ -46,6 +47,11 @@ check_client_version(GkrellmdClient *client, gint major, gint minor, gint rev)
 	return FALSE;
 	}
 
+void
+gkrellmd_add_serveflag_done(gboolean *flag)
+	{
+	serveflag_done_list = g_list_append(serveflag_done_list, flag);
+	}
 
 /* ======================================================= */
 typedef struct
@@ -157,16 +163,16 @@ serve_cpu_setup(GkrellmdClient *client)
 	GList	*list;
 	gchar	buf[64];
 
-	send_to_client(client->fd, "<cpu_setup>\n");
+	gkrellmd_send_to_client(client, "<cpu_setup>\n");
 	for (list = instance_list; list; list = list->next)
 		{
 		snprintf(buf, sizeof(buf), "cpu_instance %d\n",
 				GPOINTER_TO_INT(list->data));
-		send_to_client(client->fd, buf);
+		gkrellmd_send_to_client(client, buf);
 		}
-	send_to_client(client->fd, n_cpus_setup);
+	gkrellmd_send_to_client(client, n_cpus_setup);
 	if (nice_time_unsupported)
-		send_to_client(client->fd, "nice_time_unsupported\n");
+		gkrellmd_send_to_client(client, "nice_time_unsupported\n");
 	}
 
 static GkrellmdMonitor cpu_monitor =
@@ -482,15 +488,8 @@ serve_disk_data(GkrellmdClient *client)
 	{
 	DiskData	*disk;
 	GList		*list;
-	gchar		buf[128], *d, *s, *fmt1, *fmt2;
+	gchar		*d, *s, *line;
 
-#if defined(__solaris__)
-    fmt1 = "%s %lld %lld\n";
-    fmt2 = "%s %s %lld %lld\n";
-#else
-    fmt1 = "%s %Lu %Lu\n";
-    fmt2 = "%s %s %Lu %Lu\n";
-#endif
 	s = g_strdup("");
 	for (list = disk_list; list; list = list->next)
 		{
@@ -498,13 +497,15 @@ serve_disk_data(GkrellmdClient *client)
 		if (!disk->changed && client->served)
 			continue;
 		if (!disk->subdisk_parent)
-			sprintf(buf, fmt1, disk->name, disk->rb, disk->wb);
+			line = g_strdup_printf("%s %llu %llu\n",
+						disk->name, disk->rb, disk->wb);
 		else if (client->feature_subdisk)
-			sprintf(buf, fmt2, disk->name, disk->subdisk_parent,
-						disk->rb, disk->wb);
+			line = g_strdup_printf("%s %s %llu %llu\n",
+						disk->name, disk->subdisk_parent, disk->rb, disk->wb);
 		else
 			continue;
-		d = g_strconcat(s, buf, NULL);
+		d = g_strconcat(s, line, NULL);
+		g_free(line);
 		g_free(s);
 		s = d;
 		}
@@ -521,7 +522,7 @@ static void
 serve_disk_setup(GkrellmdClient *client)
 	{
 	if (units_are_blocks)
-		send_to_client(client->fd, "<disk_setup>\nunits_are_blocks\n");
+		gkrellmd_send_to_client(client, "<disk_setup>\nunits_are_blocks\n");
 	if (check_client_version(client, 2,1,3))
 		client->feature_subdisk = TRUE;
 	}
@@ -608,7 +609,7 @@ gkrellm_inet_log_tcp_port_data(gpointer data)
 	}
 
 void
-free_glist_and_data(GList **list_head)
+gkrellmd_free_glist_and_data(GList **list_head)
 	{
 	GList	*list;
 
@@ -634,7 +635,7 @@ update_inet(gboolean force)
 
 	if (force || check_tcp == 0)
 		{
-		free_glist_and_data(&inet_dead_list);
+		gkrellmd_free_glist_and_data(&inet_dead_list);
 		inet_new = FALSE;
 		for (list = inet_list; list; list = list->next)
 			{
@@ -777,7 +778,7 @@ static void
 serve_inet_setup(GkrellmdClient *client)
 	{
 	if (inet_unsupported)
-		send_to_client(client->fd, "<inet_setup>\ninet_unsupported\n");
+		gkrellmd_send_to_client(client, "<inet_setup>\ninet_unsupported\n");
 	}
 
 static GkrellmdMonitor inet_monitor =
@@ -1024,7 +1025,7 @@ serve_net_setup(GkrellmdClient *client)
 	|  needs to know if server sysdep uses routed for config purposes.
 	*/
 	if (net_use_routed)
-		send_to_client(client->fd, "<net_setup>\nnet_use_routed\n");
+		gkrellmd_send_to_client(client, "<net_setup>\nnet_use_routed\n");
 	}
 
 static GkrellmdMonitor net_monitor =
@@ -1115,18 +1116,11 @@ update_mem(gboolean force)
 static gchar *
 serve_mem_data(GkrellmdClient *client)
 	{
-	gchar		buf[128], *s, *d, *fmt1, *fmt2;
+	gchar	*s, *d, *e;
 
-#if defined(__solaris__)
-    fmt1 = "<mem>\n%lld %lld %lld %lld %lld %lld\n";
-    fmt2 = "%lld %lld %lu %lu\n";
-#else
-    fmt1 = "<mem>\n%Lu %Lu %Lu %Lu %Lu %Lu\n";
-    fmt2 = "%Lu %Lu %lu %lu\n";
-#endif
 	if (mem.mem_changed || !client->served)
 		{
-		s = g_strdup_printf(fmt1,
+		s = g_strdup_printf("<mem>\n%llu %llu %llu %llu %llu %llu\n",
 				mem.total, mem.used, mem.free,
 				mem.shared, mem.buffers, mem.cached);
 		}
@@ -1135,10 +1129,11 @@ serve_mem_data(GkrellmdClient *client)
 
 	if (mem.swap_changed || !client->served)
 		{
-		sprintf(buf, fmt2,
+		e = g_strdup_printf("<swap>\n%llu %llu %lu %lu\n",
 				mem.swap_total, mem.swap_used,
 				mem.swap_in, mem.swap_out);
-		d = g_strconcat(s, "<swap>\n", buf, NULL);
+		d = g_strconcat(s, e, NULL);
+		g_free(e);
 		g_free(s);
 		s = d;
 		}
@@ -1343,10 +1338,15 @@ get_fsusage_thread(void *data)
 	Mount	*m = (Mount *) data;
 
 	gkrellm_sys_fs_get_fsusage(m, m->directory);
+
 	if (m->deleted)
 		g_free(m);
 	else
+		{
+		if (m->changed)
+			fs_need_serve = TRUE;
 		m->busy = FALSE;
+		}
 	return NULL;
 	}
 #endif
@@ -1361,7 +1361,11 @@ update_fs(gboolean force)
 	if (GK.second_tick)
 		++check_tick;
 	fs_check = !(check_tick % _GK.fs_interval);
-	nfs_check = !(check_tick % _GK.nfs_interval);
+
+	if (_GK.nfs_interval > 0)
+		nfs_check = !(check_tick % _GK.nfs_interval);
+	else
+		nfs_check = 0;
 
 	if (!force && (!GK.second_tick || (!fs_check && !nfs_check)))
 		return;
@@ -1373,8 +1377,8 @@ update_fs(gboolean force)
 			gkrellm_sys_fs_get_fsusage(m, m->directory);
 		else if (nfs_check && m->is_nfs && !m->busy)
 			{
-			m->busy = TRUE;
 #if GLIB_CHECK_VERSION(2,0,0)
+			m->busy = TRUE;
 			g_thread_create(get_fsusage_thread, m, FALSE, NULL);
 #else
 			gkrellm_sys_fs_get_fsusage(m, m->directory);
@@ -1454,7 +1458,7 @@ static void
 serve_fs_setup(GkrellmdClient *client)
 	{
 	if (mounting_unsupported)
-		send_to_client(client->fd, "<fs_setup>\nmounting_unsupported\n");
+		gkrellmd_send_to_client(client, "<fs_setup>\nmounting_unsupported\n");
 	}
 
 static GkrellmdMonitor fs_monitor =
@@ -1481,22 +1485,7 @@ init_fs_monitor(void)
 
 /* ======================================================= */
 
-void
-gkrellm_mail_local_unsupported(void)
-	{
-	}
 
-#if GLIB_CHECK_VERSION(2,0,0)
-GThread *
-#else
-gpointer
-#endif
-gkrellm_mail_get_active_thread(void)
-	{
-	return NULL;
-	}
-
-/* ======================================================= */
 typedef struct
 	{
 	gboolean	changed,
@@ -1511,7 +1500,9 @@ typedef struct
 	}
 	Battery;
 
-GList	*battery_list;
+static GList	*battery_list;
+
+static Battery	*composite_battery;
 
 
 static Battery *
@@ -1520,8 +1511,25 @@ battery_nth(gint n)
 	Battery		*bat;
 	static gint	n_batteries;
 
-	if (n < 0 || n > 10)
+	if (n > 10)
 		return NULL;
+	if (n < 0)
+		{
+		if (!composite_battery)
+			{
+			bat = g_new0(Battery, 1);
+			battery_list = g_list_prepend(battery_list, bat);
+			bat->id = GKRELLM_BATTERY_COMPOSITE_ID;
+			composite_battery = bat;
+			serveflag_done_list = g_list_append(serveflag_done_list,
+								&composite_battery->changed);
+			}
+		return composite_battery;
+		}
+
+	if (composite_battery)
+		++n;
+
 	while ((bat = (Battery *)g_list_nth_data(battery_list, n)) == NULL)
 		{
 		bat = g_new0(Battery, 1);
@@ -1622,7 +1630,7 @@ serve_battery_setup(GkrellmdClient *client)
 	gkrellm_sys_battery_read_data();
 	if (battery_list)
 		{
-		send_to_client(client->fd, "<battery_setup>\nbattery_available\n");
+		gkrellmd_send_to_client(client, "<battery_setup>\nbattery_available\n");
 		}
 	}
 
@@ -1816,7 +1824,7 @@ serve_sensors_setup(GkrellmdClient *client)
 	Sensor	*s;
 	gchar	buf[256];
 
-	send_to_client(client->fd, "<sensors_setup>\n");
+	gkrellmd_send_to_client(client, "<sensors_setup>\n");
 	for (list = sensors_list; list; list = list->next)
 		{
 		s = (Sensor *) list->data;
@@ -1824,7 +1832,7 @@ serve_sensors_setup(GkrellmdClient *client)
 				s->type, s->basename,
 				s->id, s->iodev, s->inter,
 				s->factor, s->offset, s->vref, s->default_label);
-		send_to_client(client->fd, buf);
+		gkrellmd_send_to_client(client, buf);
 		}
 	}
 
@@ -1908,7 +1916,7 @@ init_uptime_monitor(void)
 
 /* ======================================================= */
 static void
-send_time(gint fd)
+send_time(GkrellmdClient *client)
 	{
 	struct tm	*t;
 	gchar		buf[128];
@@ -1918,7 +1926,7 @@ send_time(gint fd)
 		t->tm_sec, t->tm_min, t->tm_hour,
 		t->tm_mday, t->tm_mon, t->tm_year,
 		t->tm_wday, t->tm_yday, t->tm_isdst);
-	send_to_client(fd, buf);
+	gkrellmd_send_to_client(client, buf);
 	}
 
 /* ======================================================= */
@@ -1932,7 +1940,7 @@ add_monitor(GkrellmdMonitor *mon)
 	}
 
 void
-load_monitors(void)
+gkrellmd_load_monitors(void)
 	{
 	add_monitor(init_sensors_monitor());
 	add_monitor(init_cpu_monitor());
@@ -1942,12 +1950,13 @@ load_monitors(void)
 	add_monitor(init_inet_monitor());
 	add_monitor(init_mem_monitor());
 	add_monitor(init_fs_monitor());
+	add_monitor(gkrellmd_init_mail_monitor());
 	add_monitor(init_battery_monitor());
 	add_monitor(init_uptime_monitor());
 	}
 
 gint
-update_monitors(void)
+gkrellmd_update_monitors(void)
 	{
 	GList			*list, *c_list;
 	GkrellmdMonitor	*mon;
@@ -1990,7 +1999,7 @@ update_monitors(void)
 		client = (GkrellmdClient *) c_list->data;
 		client->last_client = !c_list->next;
 		if (!client->served)
-			send_to_client(client->fd, "<initial_update>\n");
+			gkrellmd_send_to_client(client, "<initial_update>\n");
 		send_string = g_strdup("");
 		for (list = gkrellmd_monitor_list; list; list = list->next)
 			{
@@ -2007,19 +2016,19 @@ update_monitors(void)
 				g_free(d);
 				}
 			}
-		send_to_client(client->fd, send_string);
+		gkrellmd_send_to_client(client, send_string);
 		g_free(send_string);
 
 		if (GK.minute_tick || !client->served)
-			send_time(client->fd);
+			send_time(client);
 		else if (GK.second_tick)
 			{
 			sprintf(buf, "<.%d>\n", gkrellmd_current_tm.tm_sec);
-			send_to_client(client->fd, buf);
+			gkrellmd_send_to_client(client, buf);
 			}
 
 		if (!client->served)
-			send_to_client(client->fd, "</initial_update>\n");
+			gkrellmd_send_to_client(client, "</initial_update>\n");
 		client->served = TRUE;
 		}
 
@@ -2038,12 +2047,12 @@ gkrellmd_serve_setup(GkrellmdClient *client)
 	struct lconv	*lc;
 	gchar			buf[32], *s, *name;
 
-	send_to_client(client->fd, "<gkrellmd_setup>\n");
+	gkrellmd_send_to_client(client, "<gkrellmd_setup>\n");
 
 	lc = localeconv();
 	sprintf(buf, "%c\n", *lc->decimal_point);
 	s = g_strconcat("<decimal_point>\n", buf, "\n", NULL);
-	send_to_client(client->fd, s);
+	gkrellmd_send_to_client(client, s);
 	g_free(s);
 
 	for (list = gkrellmd_monitor_list; list; list = list->next)
@@ -2054,50 +2063,63 @@ gkrellmd_serve_setup(GkrellmdClient *client)
 		}
 	name = gkrellm_sys_get_host_name();
 	s = g_strconcat("<hostname>\n", name, "\n", NULL);
-	send_to_client(client->fd, s);
+	gkrellmd_send_to_client(client, s);
 	g_free(s);
 
 	name = gkrellm_sys_get_system_name();
 	s = g_strconcat("<sysname>\n", name, "\n", NULL);
-	send_to_client(client->fd, s);
+	gkrellmd_send_to_client(client, s);
 	g_free(s);
 
-	send_time(client->fd);
+	send_time(client);
 
-	send_to_client(client->fd, "<monitors>\n");
+	gkrellmd_send_to_client(client, "<monitors>\n");
 	for (list = gkrellmd_monitor_list; list; list = list->next)
 		{
 		mon = (GkrellmdMonitor *) list->data;
 		sprintf(buf, "%s\n", mon->name);
-		send_to_client(client->fd, buf);
+		gkrellmd_send_to_client(client, buf);
 		}
 
 	sprintf(buf, "%d\n", _GK.io_timeout);
 	s = g_strconcat("<io_timeout>\n", buf, "\n", NULL);
-	send_to_client(client->fd, s);
+	gkrellmd_send_to_client(client, s);
 	g_free(s);
 
 	sprintf(buf, "%d\n", _GK.reconnect_timeout);
 	s = g_strconcat("<reconnect_timeout>\n", buf, "\n", NULL);
-	send_to_client(client->fd, s);
+	gkrellmd_send_to_client(client, s);
 	g_free(s);
 
-	send_to_client(client->fd, "</gkrellmd_setup>\n");
+	gkrellmd_send_to_client(client, "</gkrellmd_setup>\n");
 	}
 
 void
-gkrellm_client_read(gint fd, gint nbytes)
+gkrellmd_client_read(gint fd, gint nbytes)
 	{
 	GList			*list;
 	GkrellmdClient	*client;
 	gchar			buf[128];
+	gint			n, buflen;
 
 	for (list = gkrellmd_client_list; list; list = list->next)
 		{
 		client = (GkrellmdClient *) list->data;
 		if (client->fd != fd)
 			continue;
-		recv(fd, buf, nbytes, 0);
-		printf("received %d bytes: %s\n", nbytes, buf);
+
+		/* Just drain it.  Don't use client sent data yet.
+		*/
+		buflen = sizeof(buf);
+		while (nbytes > 0)
+			{
+			n = (nbytes > buflen) ? buflen : nbytes;
+			n = recv(fd, buf, n, 0);
+			if (n <= 0)
+				break;
+			nbytes -= n;
+			printf("received unexpected %d bytes from %s\n",
+						n, client->hostname);
+			}
 		}
 	}
