@@ -77,8 +77,10 @@ gkrellm_sys_get_system_name(void)
 	static gchar	*sname;
 	struct utsname	utsn;
 
-	if (!sname && !uname(&utsn))
+	if (!sname && uname(&utsn) > -1)
 		sname = g_strdup_printf("%s %s", utsn.sysname, utsn.release);
+	if (!sname)
+		sname = g_strdup("unknown name");
 	return sname;
 	}
 
