@@ -25,39 +25,9 @@
 
 #include <math.h>
 
-static GList	*gkrellm_alert_list;
-static gint		voice_fifo = -1;
+static GList	*gkrellm_alert_list,
+				*alert_plugin_list;
 
-static gboolean
-gvoice_fifo_open(void)
-	{
-	gchar	buf[32];
-
-	if (voice_fifo < 0)
-		{
-		sprintf(buf, "/tmp/gvoice_%08x", getpid());
-		if (!access(buf, W_OK))
-			voice_fifo = open(buf, O_NONBLOCK|O_WRONLY);
-		}
-	if (voice_fifo >= 0)
-		return TRUE;
-	return FALSE;
-	}
-	
-static void
-send_gvoice(GkrellmAlert *alert, gchar *string)
-	{
-	gchar	str[1024];
-
-	if (!string || !*string || !gvoice_fifo_open())
-		return;
-	if (alert->cb_command_process)
-		(*alert->cb_command_process)(alert, string, str, sizeof(str),
-				alert->cb_command_process_data);
-	else
-		snprintf(str, sizeof(str), "%s", string);
-	write(voice_fifo, str, strlen(str));
-	}
 
 static void
 run_command(GkrellmAlert *alert, gchar *command)
@@ -165,6 +135,38 @@ destroy_alert_objects(GkrellmAlert *alert)
 	alert->ak.krell = NULL;
 	}
 
+static void
+plugin_warn(GkrellmAlert *alert, gboolean state)
+	{
+	GList					*list;
+	GkrellmAlertPlugin		*gap;
+	GkrellmAlertPluginLink	*apl;
+
+	for (list = alert->plugin_list; list; list = list->next)
+		{
+		apl = (GkrellmAlertPluginLink *) list->data;
+		gap = apl->alert_plugin;
+		if (MONITOR_ENABLED(gap->mon) && gap->warn_func)
+			(*gap->warn_func)(alert, apl->data, state);
+		}
+	}
+
+static void
+plugin_alarm(GkrellmAlert *alert, gboolean state)
+	{
+	GList					*list;
+	GkrellmAlertPlugin		*gap;
+	GkrellmAlertPluginLink	*apl;
+
+	for (list = alert->plugin_list; list; list = list->next)
+		{
+		apl = (GkrellmAlertPluginLink *) list->data;
+		gap = apl->alert_plugin;
+		if (MONITOR_ENABLED(gap->mon) && gap->alarm_func)
+			(*gap->alarm_func)(alert, apl->data, state);
+		}
+	}
+
 static void
 trigger_warn(GkrellmAlert *alert, GkrellmTrigger *trigger)
 	{
@@ -174,11 +176,12 @@ trigger_warn(GkrellmAlert *alert, GkrellmTrigger *trigger)
 		create_alert_objects(alert);
 		gkrellm_alert_list = g_list_append(gkrellm_alert_list, alert);
 		if (!alert->suppress_command)
+			{
 			run_command(alert, alert->warn_command);
+			plugin_warn(alert, ON);
+			}
 		alert->suppress_command = FALSE;
 		alert->warn_repeat = alert->warn_repeat_set;
-		send_gvoice(alert, alert->voice_string);
-		alert->voice_repeat = alert->voice_repeat_set;
 		}
 	}
 
@@ -187,6 +190,7 @@ stop_warn(GkrellmAlert *alert, GkrellmTrigger *trigger)
 	{
 	if (trigger->warn_on)
 		{
+		plugin_warn(alert, OFF);
 		trigger->warn_on = FALSE;
 		destroy_alert_objects(alert);
 		gkrellm_alert_list = g_list_remove(gkrellm_alert_list, alert);
@@ -205,11 +209,12 @@ trigger_alarm(GkrellmAlert *alert, GkrellmTrigger *trigger)
 		create_alert_objects(alert);
 		gkrellm_alert_list = g_list_append(gkrellm_alert_list, alert);
 		if (!alert->suppress_command)
+			{
 			run_command(alert, alert->alarm_command);
+			plugin_alarm(alert, ON);
+			}
 		alert->suppress_command = FALSE;
 		alert->alarm_repeat = alert->alarm_repeat_set;
-		send_gvoice(alert, alert->voice_string);
-		alert->voice_repeat = alert->voice_repeat_set;
 		}
 	}
 
@@ -218,6 +223,7 @@ stop_alarm(GkrellmAlert *alert, GkrellmTrigger *trigger)
 	{
 	if (trigger->alarm_on)
 		{
+		plugin_alarm(alert, OFF);
 		trigger->alarm_on = FALSE;
 		destroy_alert_objects(alert);
 		gkrellm_alert_list = g_list_remove(gkrellm_alert_list, alert);
@@ -388,6 +394,11 @@ gkrellm_alert_stop_connect(GkrellmAlert *alert, void (*func)(), gpointer data)
 static void
 destroy_alert(GkrellmAlert *alert)
 	{
+	GList					*plist, *list;
+	GkrellmAlertPlugin		*gap;
+	GkrellmAlertPluginLink	*apl;
+	gpointer				data;
+
 	if (!alert)
 		return;
 	if (g_list_find(gkrellm_alert_list, alert))
@@ -395,22 +406,40 @@ destroy_alert(GkrellmAlert *alert)
 		gkrellm_alert_list = g_list_remove(gkrellm_alert_list, alert);
 		destroy_alert_objects(alert);
 		}
+
+	for (plist = alert_plugin_list; plist; plist = plist->next)
+		{
+		data = NULL;
+		gap = (GkrellmAlertPlugin *) plist->data;
+		for (list = alert->plugin_list; list; list = list->next)
+			{
+			apl = (GkrellmAlertPluginLink *) list->data;
+			if (apl->alert_plugin == gap)
+				{
+				data = apl->data;
+				break;
+				}
+			}
+		if (MONITOR_ENABLED(gap->mon) && gap->destroy_func)
+			(*gap->destroy_func)(alert, data);
+		}
+	free_glist_and_data(&alert->plugin_list);	/* should all be detached */
 	g_free(alert->name);
 	g_free(alert->unit_string);
 	g_free(alert->alarm_command);
 	g_free(alert->warn_command);
-	g_free(alert->voice_string);
+	g_free(alert->id_string);
 	g_free(alert);
 	}
 
 void
-gkrellm_alert_destroy(GkrellmAlert **alert)
+gkrellm_alert_destroy(GkrellmAlert **ap)
 	{
-	if (!alert || !*alert)
+	if (!ap || !*ap)
 		return;
-	gkrellm_alert_window_destroy(alert);
-	destroy_alert(*alert);
-	*alert = NULL;
+	gkrellm_alert_window_destroy(ap);
+	destroy_alert(*ap);
+	*ap = NULL;
 	}
 
 GkrellmAlert *
@@ -436,7 +465,6 @@ gkrellm_alert_create(GkrellmPanel *p, gchar *name, gchar *unit_string,
 
 	alert->alarm_command = g_strdup("");
 	alert->warn_command = g_strdup("");
-	alert->voice_string = g_strdup("");
 	alert->do_alarm_command = alert->do_warn_command = TRUE;
 
 	gkrellm_alert_set_triggers(alert, min_low, min_low, min_low, min_low);
@@ -507,6 +535,8 @@ gkrellm_alert_dup(GkrellmAlert **a_dst, GkrellmAlert *a_src)
 
 	if (!a_dst || !a_src)
 		return;
+	if (*a_dst)
+		(*a_dst)->plugin_list = NULL;	/* XXX fixme */
 	gkrellm_alert_destroy(a_dst);
 	alert = gkrellm_alert_create(a_src->panel, a_src->name,
 				a_src->unit_string, a_src->check_high, a_src->check_low,
@@ -514,13 +544,12 @@ gkrellm_alert_dup(GkrellmAlert **a_dst, GkrellmAlert *a_src)
 				a_src->step0, a_src->step1, a_src->digits);
 	gkrellm_dup_string(&alert->alarm_command, a_src->alarm_command);
 	gkrellm_dup_string(&alert->warn_command, a_src->warn_command);
-	gkrellm_dup_string(&alert->voice_string, a_src->voice_string);
 	alert->warn_repeat_set = a_src->warn_repeat_set;
 	alert->alarm_repeat_set = a_src->alarm_repeat_set;
-	alert->voice_repeat_set = a_src->voice_repeat_set;
 	alert->low = a_src->low;
 	alert->high = a_src->high;
 	gkrellm_alert_set_delay(alert, a_src->delay);
+	alert->plugin_list = a_src->plugin_list;	/* XXX fixme */
 
 	alert->activated = a_src->activated;
 	*a_dst = alert;
@@ -614,10 +643,12 @@ reset_all_alerts(void)
 void
 update_alerts(void)
 	{
-	GList			*list;
-	GkrellmAlert	*alert;
-	GkrellmAlertdecal *ad;
-	GkrellmAlertkrell *ak;
+	GList					*list, *plist;
+	GkrellmAlert			*alert;
+	GkrellmAlertdecal		*ad;
+	GkrellmAlertkrell		*ak;
+	GkrellmAlertPlugin		*gap;
+	GkrellmAlertPluginLink	*apl;
 
 	for (list = gkrellm_alert_list; list; list = list->next)
 		{
@@ -652,10 +683,12 @@ update_alerts(void)
 				run_command(alert, alert->warn_command);
 				alert->warn_repeat = alert->warn_repeat_set;
 				}
-			if (alert->voice_repeat > 0 && --alert->voice_repeat == 0)
+			for (plist = alert->plugin_list; plist; plist = plist->next)
 				{
-				send_gvoice(alert, alert->voice_string);
-				alert->voice_repeat = alert->voice_repeat_set;
+				apl = (GkrellmAlertPluginLink *) plist->data;
+				gap = apl->alert_plugin;
+				if (MONITOR_ENABLED(gap->mon) && gap->update_func)
+					(*gap->update_func)(alert, apl->data);
 				}
 			}
 		}
@@ -669,6 +702,131 @@ gkrellm_alert_is_activated(GkrellmAlert *alert)
 	return FALSE;
 	}
 
+void
+gkrellm_alert_get_alert_state(GkrellmAlert *alert, gboolean *alarm_state,
+				gboolean *warn_state)
+	{
+	*warn_state = (alert->high.warn_on || alert->low.warn_on);
+	*alarm_state = (alert->high.alarm_on || alert->low.alarm_on);
+	}
+
+
+/* ------------------------------------------------------------ */
+/* Plugin interface to alerts. */
+
+GkrellmAlertPlugin *
+gkrellm_alert_plugin_add(GkrellmMonitor *mon, gchar *name)
+	{
+	GkrellmAlertPlugin	*gap;
+
+	if (!mon || !name || !*name)
+		return NULL;
+	gap = g_new0(GkrellmAlertPlugin, 1);
+	gap->mon = mon;
+	gap->name = g_strdup(name);
+	alert_plugin_list = g_list_append(alert_plugin_list, gap);
+	return gap;
+	}
+
+void
+gkrellm_alert_plugin_connect(GkrellmAlertPlugin *gap,
+			void (*alarm_func)(), void (*warn_func)(),
+			void (*update_func)(), void (*destroy_func)())
+	{
+	gap->alarm_func = alarm_func;
+	gap->warn_func = warn_func;
+	gap->update_func = update_func;
+	gap->destroy_func = destroy_func;
+	}
+
+void
+gkrellm_alert_plugin_config_connect(GkrellmAlertPlugin *gap, gchar *tab_name,
+			void (*config_create_func)(), void (*config_apply_func),
+			void (*config_save_func)(), void (*config_load_func)())
+	{
+	if (!gap)
+		return;
+	g_free(gap->tab_name);
+	gap->tab_name = g_strdup(tab_name);
+	gap->config_create_func = config_create_func;
+	gap->config_apply_func = config_apply_func;
+	gap->config_save_func = config_save_func;
+	gap->config_load_func = config_load_func;
+	}
+
+  /* The id_string is so a plugin can get a unique config name for an alert.
+  */
+gchar *
+gkrellm_alert_plugin_config_get_id_string(GkrellmAlert *alert)
+	{
+	if (alert)
+		return alert->id_string;
+	return NULL;
+	}
+
+void
+gkrellm_alert_plugin_alert_attach(GkrellmAlertPlugin *gap,
+			GkrellmAlert *alert, gpointer data)
+	{
+	GkrellmAlertPluginLink	*apl;
+
+	if (!gap || !alert)
+		return;
+	apl = g_new0(GkrellmAlertPluginLink, 1);
+	apl->alert_plugin = gap;
+	apl->data = data;
+	alert->plugin_list = g_list_append(alert->plugin_list, apl);
+	}
+
+void
+gkrellm_alert_plugin_alert_detach(GkrellmAlertPlugin *gap, GkrellmAlert *alert)
+	{
+	GList					*list;
+	GkrellmAlertPluginLink	*apl;
+
+	if (!gap || !alert)
+		return;
+	for (list = alert->plugin_list; list; list = list->next)
+		{
+		apl = (GkrellmAlertPluginLink *) list->data;
+		if (apl->alert_plugin == gap)
+			{
+			alert->plugin_list = g_list_remove(alert->plugin_list, apl);
+			g_free(apl);
+			break;
+			}
+		}
+	}
+
+gpointer
+gkrellm_alert_plugin_get_data(GkrellmAlertPlugin *gap,GkrellmAlert *alert)
+	{
+	GList					*list;
+	GkrellmAlertPluginLink	*apl;
+
+	if (!gap || !alert)
+		return NULL;
+	for (list = alert->plugin_list; list; list = list->next)
+		{
+		apl = (GkrellmAlertPluginLink *) list->data;
+		if (apl->alert_plugin == gap)
+			return apl->data;
+		}
+	return NULL;
+	}
+
+void
+gkrellm_alert_plugin_command_process(GkrellmAlert *alert, gchar *src,
+			gchar *dst, gint dst_size)
+	{
+	if (alert && alert->cb_command_process)
+		(*alert->cb_command_process)(alert, src, dst, dst_size,
+				alert->cb_command_process_data);
+	else
+		snprintf(dst, dst_size, "%s", src);
+	}
+
+
 /* ------------------------------------------------------------ */
 void
 gkrellm_alert_command_process_connect(GkrellmAlert *alert, void (*func)(),
@@ -702,12 +860,13 @@ gkrellm_alert_config_create_connect(GkrellmAlert *alert,
 static void
 alert_delete(GtkWidget *widget, GkrellmAlert **ap)
 	{
-	GkrellmAlert	*alert;
+	GkrellmAlert			*alert;
 
 	if (!ap)
 		return;
 	alert = *ap;
 	alert->activated = FALSE;
+	alert->config_closing = TRUE;
 	if (alert->cb_config)
 		(*alert->cb_config)(alert, alert->cb_config_data);
 	if (alert->config_window)
@@ -717,47 +876,52 @@ alert_delete(GtkWidget *widget, GkrellmAlert **ap)
 	}
 
 static void
-alert_close(GtkWidget *widget, GkrellmAlert **alert)
+alert_close(GtkWidget *widget, GkrellmAlert **ap)
 	{
-	GkrellmAlert	*ap;
+	GkrellmAlert	*alert;
 
-	if (!alert)
+	if (!ap)
 		return;
-	ap = *alert;
-	if (!ap->activated)
-		alert_delete(NULL, alert);
-	else if (ap->config_window)
+	alert = *ap;
+	if (!alert->activated)
+		alert_delete(NULL, ap);
+	else if (alert->config_window)
 		{
-		gtk_widget_destroy(ap->config_window);
-		ap->config_window = NULL;
-		ap->delete_button = NULL;
+		gtk_widget_destroy(alert->config_window);
+		alert->config_window = NULL;
+		alert->delete_button = NULL;
 		}
 	}
 
 void
-gkrellm_alert_window_destroy(GkrellmAlert **alert)
+gkrellm_alert_window_destroy(GkrellmAlert **ap)
 	{
-	alert_close(NULL, alert);
+	alert_close(NULL, ap);
 	}
 
 static gint
 alert_config_window_delete_event(GtkWidget *widget, GdkEvent *ev,
-		GkrellmAlert **alert)
+		GkrellmAlert **ap)
 	{
-	alert_close(widget, alert);
+	alert_close(widget, ap);
 	return FALSE;
 	}
 
 static void
 alert_apply(GtkWidget *widget, GkrellmAlert **ap)
 	{
-	GkrellmAlert	*alert;
-	gchar			*s;
-	GtkSpinButton	*spin;
-	gint			n;
+	GList					*plist, *list;
+	GkrellmAlert			*alert;
+	GkrellmAlertPlugin		*gap;
+	GkrellmAlertPluginLink	*apl;
+	gpointer				data;
+	gchar					*s;
+	GtkSpinButton			*spin;
+	gint					n;
 
 	alert = *ap;
-
+	if (!alert->activated && !alert->config_modified)
+		return;
 	if (alert->high.alarm_limit_spin_button)
 		{
 		spin = GTK_SPIN_BUTTON(alert->high.alarm_limit_spin_button);
@@ -774,6 +938,8 @@ alert_apply(GtkWidget *widget, GkrellmAlert **ap)
 		alert->low.warn_limit = gtk_spin_button_get_value(spin);
 		}
 
+	if (alert->high.alarm_limit == 0.0 && alert->low.alarm_limit == 0.0)
+		return;
 	if (alert->alarm_command_entry)
 		{
 		spin = GTK_SPIN_BUTTON(alert->alarm_repeat_spin_button);
@@ -786,7 +952,6 @@ alert_apply(GtkWidget *widget, GkrellmAlert **ap)
 		if (alert->high.alarm_on || alert->low.alarm_on)
 			alert->alarm_repeat = alert->alarm_repeat_set;
 		}
-
 	if (alert->warn_command_entry)
 		{
 		spin = GTK_SPIN_BUTTON(alert->warn_repeat_spin_button);
@@ -799,38 +964,44 @@ alert_apply(GtkWidget *widget, GkrellmAlert **ap)
 		if (alert->high.warn_on || alert->low.warn_on)
 			alert->warn_repeat = alert->warn_repeat_set;
 		}
-
-	if (alert->voice_string_entry)
-		{
-		spin = GTK_SPIN_BUTTON(alert->voice_repeat_spin_button);
-		alert->voice_repeat_set = gtk_spin_button_get_value_as_int(spin);
-
-		s = gkrellm_gtk_entry_get_text(&alert->voice_string_entry);
-		gkrellm_dup_string(&alert->voice_string, s);
-		if (!*s)
-			alert->voice_repeat_set = 0;
-		alert->voice_repeat = alert->voice_repeat_set;
-		}
-
 	if (alert->delay_spin_button)
 		{
 		spin = GTK_SPIN_BUTTON(alert->delay_spin_button);
 		n = gtk_spin_button_get_value_as_int(spin);
 		gkrellm_alert_set_delay(alert, n / alert->delay_step);
 		}
-
 	alert->activated = TRUE;
+
+	for (plist = alert_plugin_list; plist; plist = plist->next)
+		{
+		data = NULL;
+		gap = (GkrellmAlertPlugin *) plist->data;
+		for (list = alert->plugin_list; list; list = list->next)
+			{
+			apl = (GkrellmAlertPluginLink *) list->data;
+			if (apl->alert_plugin == gap)
+				{
+				data = apl->data;
+				break;
+				}
+			}
+		if (MONITOR_ENABLED(gap->mon) && gap->config_apply_func)
+			(*gap->config_apply_func)(alert, data, alert->config_closing);
+		}
+
 	gtk_widget_set_sensitive(alert->delete_button, alert->activated);
+	gtk_widget_set_sensitive(alert->icon_box, alert->activated);
 	if (alert->cb_config)
 		(*alert->cb_config)(alert, alert->cb_config_data);
 	gkrellm_config_modified();
 	}
 
 static void
-alert_ok(GtkWidget *widget, GkrellmAlert **alert)
+alert_ok(GtkWidget *widget, GkrellmAlert **ap)
 	{
-	alert_apply(NULL, alert);
-	alert_close(NULL, alert);
+	(*ap)->config_closing = TRUE;
+	alert_apply(NULL, ap);
+	alert_close(NULL, ap);
 	}
 
 static void
@@ -846,6 +1017,8 @@ cb_delay_spin_changed(GtkAdjustment *adjustment, GkrellmAlert *alert)
 		delay = delay / alert->delay_step * alert->delay_step;
 		gtk_spin_button_set_value(spin, delay);
 		}
+	alert->config_modified = TRUE;
+	gtk_widget_set_sensitive(alert->icon_box, TRUE);
 	}
 
 static void
@@ -860,6 +1033,8 @@ cb_high_alarm_spin_changed(GtkAdjustment *adjustment, GkrellmAlert *alert)
 	warn = gtk_spin_button_get_value(spin);
 	if (alarm < warn)
 		gtk_spin_button_set_value(spin, alarm);
+	alert->config_modified = TRUE;
+	gtk_widget_set_sensitive(alert->icon_box, TRUE);
 	}
 
 static void
@@ -881,6 +1056,8 @@ cb_high_warn_spin_changed(GtkWidget *adjustment, GkrellmAlert *alert)
 		if (low_warn > warn)
 			gtk_spin_button_set_value(spin, warn);
 		}
+	alert->config_modified = TRUE;
+	gtk_widget_set_sensitive(alert->icon_box, TRUE);
 	}
 
 static void
@@ -902,6 +1079,8 @@ cb_low_warn_spin_changed(GtkWidget *adjustment, GkrellmAlert *alert)
 		if (high_warn < warn)
 			gtk_spin_button_set_value(spin, warn);
 		}
+	alert->config_modified = TRUE;
+	gtk_widget_set_sensitive(alert->icon_box, TRUE);
 	}
 
 static void
@@ -916,105 +1095,123 @@ cb_low_alarm_spin_changed(GtkWidget *adjustment, GkrellmAlert *alert)
 	warn = gtk_spin_button_get_value(spin);
 	if (alarm > warn)
 		gtk_spin_button_set_value(spin, alarm);
+	alert->config_modified = TRUE;
+	gtk_widget_set_sensitive(alert->icon_box, TRUE);
 	}
 
 void
-gkrellm_alert_config_window(GkrellmAlert **alert)
+gkrellm_alert_config_window(GkrellmAlert **ap)
 	{
-	GtkWidget	*main_vbox, *vbox, *vbox1 = NULL, *hbox;
-	GtkWidget	*table = NULL;
-	GtkWidget	*button;
-	GtkWidget	*label;
-	GtkWidget	*separator;
-	GkrellmAlert *ap;
-	gchar		*title;
-	gint		w;
+	GtkWidget				*tabs;
+	GtkWidget				*main_vbox, *tab_vbox, *vbox, *vbox1 = NULL;
+	GtkWidget				*hbox, *hbox1, *image;
+	GtkWidget				*table = NULL;
+	GtkWidget				*button;
+	GtkWidget				*label;
+	GtkWidget				*separator;
+	GList					*plist, *list;
+	GkrellmAlert			*alert;
+	GkrellmAlertPlugin		*gap;
+	GkrellmAlertPluginLink	*apl;
+	gpointer				data;
+	gchar					*title;
+	gint					w, n_tabs = 1;
 
-	if (!alert || !*alert)
+	if (!ap || !*ap)
 		return;
-	ap = *alert;
-	if (!ap->config_window)
+	alert = *ap;
+	if (!alert->config_window)
 		{
-		ap->config_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-		g_signal_connect(G_OBJECT(ap->config_window), "delete_event",
-				G_CALLBACK(alert_config_window_delete_event), alert);
-		gtk_window_set_title(GTK_WINDOW(ap->config_window),
+		alert->config_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+		g_signal_connect(G_OBJECT(alert->config_window), "delete_event",
+				G_CALLBACK(alert_config_window_delete_event), ap);
+		gtk_window_set_title(GTK_WINDOW(alert->config_window),
 				_("GKrellM Set Alerts"));
-		gtk_window_set_wmclass(GTK_WINDOW(ap->config_window),
+		gtk_window_set_wmclass(GTK_WINDOW(alert->config_window),
 				"Gkrellm_conf", "Gkrellm");
 
-		gtk_container_set_border_width(GTK_CONTAINER(ap->config_window), 4);
+		gtk_container_set_border_width(GTK_CONTAINER(alert->config_window), 4);
 		main_vbox = gtk_vbox_new(FALSE, 0);
-		gtk_container_add(GTK_CONTAINER(ap->config_window), main_vbox);
+		gtk_container_add(GTK_CONTAINER(alert->config_window), main_vbox);
 
-		if (ap->name && ap->unit_string)
-			title = g_strdup_printf("%s - %s", ap->name, ap->unit_string);
-		else if (ap->name)
-			title = g_strdup_printf("%s", ap->name);
+		tabs = gtk_notebook_new();
+		gtk_box_pack_start(GTK_BOX(main_vbox), tabs, TRUE, TRUE, 0);
+		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
+
+		tab_vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Alerts"));
+
+		if (alert->name && alert->unit_string)
+			title = g_strdup_printf("%s - %s",
+						alert->name, alert->unit_string);
+		else if (alert->name)
+			title = g_strdup_printf("%s", alert->name);
 		else
-			title = g_strdup_printf("%s", ap->unit_string);
+			title = g_strdup_printf("%s", alert->unit_string);
 
-		vbox = gkrellm_gtk_framed_vbox(main_vbox, title, 4, FALSE, 4, 3);
+		vbox = gkrellm_gtk_framed_vbox(tab_vbox, title, 4, FALSE, 4, 3);
 		g_free(title);
 
 		hbox = gtk_hbox_new(FALSE, 0);
 		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 		w = 70;
-		if (ap->max_high > 100000)
-			w += log(ap->max_high / 100000) * 5;
+		if (alert->max_high > 100000)
+			w += log(alert->max_high / 100000) * 5;
 
-		ap->high.alarm_limit_spin_button = NULL;
-		ap->high.warn_limit_spin_button = NULL;
-		if (ap->check_high  && !ap->check_hardwired)
+		alert->high.alarm_limit_spin_button = NULL;
+		alert->high.warn_limit_spin_button = NULL;
+		if (alert->check_high  && !alert->check_hardwired)
 			{
 			vbox1 = gkrellm_gtk_framed_vbox(hbox, _("High Limits"),
 						2, FALSE, 2,2);
-			gkrellm_gtk_spin_button(vbox1, &ap->high.alarm_limit_spin_button,
-					ap->high.alarm_limit, ap->min_low, ap->max_high,
-					ap->step0, ap->step1,
-					ap->digits, w, cb_high_alarm_spin_changed, ap, FALSE,
+			gkrellm_gtk_spin_button(vbox1,
+					&alert->high.alarm_limit_spin_button,
+					alert->high.alarm_limit, alert->min_low, alert->max_high,
+					alert->step0, alert->step1,
+					alert->digits, w, cb_high_alarm_spin_changed, alert, FALSE,
 					_("High alarm limit"));
 		
-			gkrellm_gtk_spin_button(vbox1, &ap->high.warn_limit_spin_button,
-					ap->high.warn_limit, ap->min_low, ap->max_high,
-					ap->step0, ap->step1,
-					ap->digits, w, cb_high_warn_spin_changed, ap, FALSE,
+			gkrellm_gtk_spin_button(vbox1, &alert->high.warn_limit_spin_button,
+					alert->high.warn_limit, alert->min_low, alert->max_high,
+					alert->step0, alert->step1,
+					alert->digits, w, cb_high_warn_spin_changed, alert, FALSE,
 					_("High warn limit"));
 			}
 
-		ap->low.alarm_limit_spin_button = NULL;
-		ap->low.warn_limit_spin_button = NULL;
-		if (ap->check_low && !ap->check_hardwired)
+		alert->low.alarm_limit_spin_button = NULL;
+		alert->low.warn_limit_spin_button = NULL;
+		if (alert->check_low && !alert->check_hardwired)
 			{
 			vbox1 = gkrellm_gtk_framed_vbox_end(hbox, _("Low Limits"),
 					2, FALSE, 2, 2);
-			gkrellm_gtk_spin_button(vbox1, &ap->low.warn_limit_spin_button,
-					ap->low.warn_limit, ap->min_low, ap->max_high,
-					ap->step0, ap->step1,
-					ap->digits, w, cb_low_warn_spin_changed, ap, FALSE,
+			gkrellm_gtk_spin_button(vbox1, &alert->low.warn_limit_spin_button,
+					alert->low.warn_limit, alert->min_low, alert->max_high,
+					alert->step0, alert->step1,
+					alert->digits, w, cb_low_warn_spin_changed, alert, FALSE,
 					_("Low warn limit"));
-			gkrellm_gtk_spin_button(vbox1, &ap->low.alarm_limit_spin_button,
-					ap->low.alarm_limit, ap->min_low, ap->max_high,
-					ap->step0, ap->step1,
-					ap->digits, w, cb_low_alarm_spin_changed, ap, FALSE,
+			gkrellm_gtk_spin_button(vbox1, &alert->low.alarm_limit_spin_button,
+					alert->low.alarm_limit, alert->min_low, alert->max_high,
+					alert->step0, alert->step1,
+					alert->digits, w, cb_low_alarm_spin_changed, alert, FALSE,
 					_("Low alarm limit"));
 			}
-		if (ap->delay_step > 0)
+		if (alert->delay_step > 0)
 			{
 			vbox1 = gkrellm_gtk_framed_vbox(vbox, NULL, 2, FALSE, 2, 2);
-			gkrellm_gtk_spin_button(vbox1, &ap->delay_spin_button,
-					ap->delay * ap->delay_step, ap->delay_low, ap->delay_high,
-					ap->delay_step, ap->delay_step,
-					0, 70, cb_delay_spin_changed, ap, FALSE,
+			gkrellm_gtk_spin_button(vbox1, &alert->delay_spin_button,
+					alert->delay * alert->delay_step,
+					alert->delay_low, alert->delay_high,
+					alert->delay_step, alert->delay_step,
+					0, 70, cb_delay_spin_changed, alert, FALSE,
 					_("Seconds limit conditions must exist to have an alert"));
 			}
-		if (ap->cb_config_create)
+		if (alert->cb_config_create)
 			{
 			vbox1 = gkrellm_gtk_framed_vbox(vbox, NULL, 2, FALSE, 2, 2);
-			(*ap->cb_config_create)(alert, vbox1, ap->cb_config_create_data);
+			(*alert->cb_config_create)(alert, vbox1,
+						alert->cb_config_create_data);
 			}
 
-		if (ap->do_alarm_command || ap->do_warn_command)
+		if (alert->do_alarm_command || alert->do_warn_command)
 			{
 			vbox1 = gkrellm_gtk_framed_vbox(vbox,
 					_("Commands - with repeat intervals in seconds"),
@@ -1024,69 +1221,48 @@ gkrellm_alert_config_window(GkrellmAlert **alert)
 			gtk_box_pack_start(GTK_BOX(vbox1), table, FALSE, FALSE, 0);
 			}
 
-		ap->alarm_command_entry = ap->alarm_repeat_spin_button = NULL;
-		if (ap->do_alarm_command)
+		alert->alarm_command_entry = alert->alarm_repeat_spin_button = NULL;
+		if (alert->do_alarm_command)
 			{
 			label = gtk_label_new(_("Alarm command:"));
 			gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
 			gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
-			ap->alarm_command_entry = gtk_entry_new();
-			gtk_entry_set_max_length(GTK_ENTRY(ap->alarm_command_entry), 255);
+			alert->alarm_command_entry = gtk_entry_new();
+			gtk_entry_set_max_length(GTK_ENTRY(alert->alarm_command_entry),
+					255);
 			gtk_table_attach_defaults(GTK_TABLE(table),
-					ap->alarm_command_entry, 1, 2, 0, 1);
-			gtk_entry_set_text(GTK_ENTRY(ap->alarm_command_entry),
-					ap->alarm_command);
-			gkrellm_gtk_spin_button(NULL, &ap->alarm_repeat_spin_button,
-					ap->alarm_repeat_set, 0, 1000,
+					alert->alarm_command_entry, 1, 2, 0, 1);
+			gtk_entry_set_text(GTK_ENTRY(alert->alarm_command_entry),
+					alert->alarm_command);
+			gkrellm_gtk_spin_button(NULL, &alert->alarm_repeat_spin_button,
+					alert->alarm_repeat_set, 0, 1000,
 					1, 10, 0, 60, NULL, NULL, FALSE, NULL);
 			gtk_table_attach_defaults(GTK_TABLE(table),
-					ap->alarm_repeat_spin_button, 2, 3, 0, 1);
+					alert->alarm_repeat_spin_button, 2, 3, 0, 1);
 			}
 
-		ap->warn_command_entry = ap->warn_repeat_spin_button = NULL;
-		if (ap->do_warn_command)
+		alert->warn_command_entry = alert->warn_repeat_spin_button = NULL;
+		if (alert->do_warn_command)
 			{
 			label = gtk_label_new(_("Warn command:"));
 			gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
 			gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
-			ap->warn_command_entry = gtk_entry_new();
-			gtk_entry_set_max_length(GTK_ENTRY(ap->warn_command_entry), 255);
-			gtk_table_attach_defaults(GTK_TABLE(table), ap->warn_command_entry,
-					1, 2, 1, 2);
-			gtk_entry_set_text(GTK_ENTRY(ap->warn_command_entry),
-					ap->warn_command);
-			gtk_widget_set_size_request(ap->warn_command_entry, 300, -1);
-			gkrellm_gtk_spin_button(NULL, &ap->warn_repeat_spin_button,
-					ap->warn_repeat_set, 0, 1000,
+			alert->warn_command_entry = gtk_entry_new();
+			gtk_entry_set_max_length(GTK_ENTRY(alert->warn_command_entry),
+					255);
+			gtk_table_attach_defaults(GTK_TABLE(table),
+					alert->warn_command_entry, 1, 2, 1, 2);
+			gtk_entry_set_text(GTK_ENTRY(alert->warn_command_entry),
+					alert->warn_command);
+			gtk_widget_set_size_request(alert->warn_command_entry, 300, -1);
+			gkrellm_gtk_spin_button(NULL, &alert->warn_repeat_spin_button,
+					alert->warn_repeat_set, 0, 1000,
 					1, 10, 0, 60, NULL, NULL, FALSE, NULL);
 			gtk_table_attach_defaults(GTK_TABLE(table),
-					ap->warn_repeat_spin_button, 2, 3, 1, 2);
+					alert->warn_repeat_spin_button, 2, 3, 1, 2);
 			}
 
-		ap->voice_string_entry = NULL;
-		ap->voice_repeat_spin_button = NULL;
-		if (   (ap->do_alarm_command || ap->do_warn_command)
-			&& gvoice_fifo_open()
-		   )
-			{
-			label = gtk_label_new(_("Voice string:"));
-			gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-			gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
-			ap->voice_string_entry = gtk_entry_new();
-			gtk_entry_set_max_length(GTK_ENTRY(ap->voice_string_entry), 255);
-			gtk_table_attach_defaults(GTK_TABLE(table), ap->voice_string_entry,
-					1, 2, 2, 3);
-			gtk_entry_set_text(GTK_ENTRY(ap->voice_string_entry),
-					ap->voice_string);
-			gtk_widget_set_size_request(ap->voice_string_entry, 300, -1);
-			gkrellm_gtk_spin_button(NULL, &ap->voice_repeat_spin_button,
-					ap->voice_repeat_set, 0, 1000,
-					1, 10, 0, 60, NULL, NULL, FALSE, NULL);
-			gtk_table_attach_defaults(GTK_TABLE(table),
-					ap->voice_repeat_spin_button, 2, 3, 2, 3);
-			}
-
-		if (ap->do_alarm_command || ap->do_warn_command)
+		if (alert->do_alarm_command || alert->do_warn_command)
 			{
 			separator = gtk_hseparator_new();
 			gtk_box_pack_start(GTK_BOX(vbox1), separator, FALSE, FALSE, 4);
@@ -1098,48 +1274,85 @@ gkrellm_alert_config_window(GkrellmAlert **alert)
 			gtk_box_pack_start(GTK_BOX(vbox1), label, FALSE, FALSE, 0);
 			}
 
+		for (plist = alert_plugin_list; plist; plist = plist->next)
+			{
+			data = NULL;
+			gap = (GkrellmAlertPlugin *) plist->data;
+			for (list = alert->plugin_list; list; list = list->next)
+				{
+				apl = (GkrellmAlertPluginLink *) list->data;
+				if (apl->alert_plugin == gap)
+					{
+					data = apl->data;
+					break;
+					}
+				}
+			if (MONITOR_ENABLED(gap->mon) && gap->config_create_func)
+				{
+				tab_vbox = gkrellm_gtk_framed_notebook_page(tabs,
+							gap->tab_name);
+				(*gap->config_create_func)(tab_vbox, alert, data);
+				++n_tabs;
+				}
+			}
+
+		if (n_tabs == 1)
+			gtk_notebook_set_show_tabs(GTK_NOTEBOOK(tabs), FALSE);
+
+		alert->icon_box = gtk_event_box_new();
+		hbox1 = gtk_hbox_new(FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(main_vbox), hbox1, FALSE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(hbox1), alert->icon_box, TRUE, FALSE, 0);
+		image = gtk_image_new_from_pixbuf(gkrellm_alert_pixbuf());
+		gtk_container_add(GTK_CONTAINER(alert->icon_box), image);
+		gtk_widget_set_sensitive(alert->icon_box, alert->activated);
+
 		hbox = gtk_hbutton_box_new();
 		gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
 		gtk_box_set_spacing(GTK_BOX(hbox), 5);
-		gtk_box_pack_start(GTK_BOX(main_vbox), hbox, FALSE, FALSE, 0);
+		gtk_box_pack_end(GTK_BOX(hbox1), hbox, FALSE, FALSE, 0);
 
-		ap->delete_button = gtk_button_new_from_stock(GTK_STOCK_DELETE);
-		GTK_WIDGET_SET_FLAGS(ap->delete_button, GTK_CAN_DEFAULT);
-		g_signal_connect(G_OBJECT(ap->delete_button), "clicked",
-				G_CALLBACK(alert_delete), alert);
-		gtk_box_pack_start(GTK_BOX(hbox), ap->delete_button, TRUE, TRUE, 0);
-		gtk_widget_set_sensitive(ap->delete_button, ap->activated);
+		alert->delete_button = gtk_button_new_from_stock(GTK_STOCK_DELETE);
+		GTK_WIDGET_SET_FLAGS(alert->delete_button, GTK_CAN_DEFAULT);
+		g_signal_connect(G_OBJECT(alert->delete_button), "clicked",
+				G_CALLBACK(alert_delete), ap);
+		gtk_box_pack_start(GTK_BOX(hbox), alert->delete_button, TRUE, TRUE, 0);
+		gtk_widget_set_sensitive(alert->delete_button, alert->activated);
 
 		button = gtk_button_new_from_stock(GTK_STOCK_APPLY);
 		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 		g_signal_connect(G_OBJECT(button), "clicked",
-				G_CALLBACK(alert_apply), alert);
+				G_CALLBACK(alert_apply), ap);
 		gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
 
 		button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
 		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 		g_signal_connect(G_OBJECT(button), "clicked",
-				G_CALLBACK(alert_close), alert);
+				G_CALLBACK(alert_close), ap);
 		gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
 
 		button = gtk_button_new_from_stock(GTK_STOCK_OK);
 		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 		g_signal_connect(G_OBJECT(button), "clicked",
-				G_CALLBACK(alert_ok), alert);
+				G_CALLBACK(alert_ok), ap);
 		gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
 		gtk_widget_grab_default(button);
 
-		gtk_widget_show_all(ap->config_window);
+		gtk_widget_show_all(alert->config_window);
 		}
 	else
-		gtk_window_present(GTK_WINDOW(ap->config_window));
+		gtk_window_present(GTK_WINDOW(alert->config_window));
+	alert->config_closing = FALSE;
 	}
 
 void
 gkrellm_save_alertconfig(FILE *f, GkrellmAlert *alert,
 			gchar *mon_keyword, gchar *name)
 	{
-	gchar		*s;
+	GList					*list;
+	GkrellmAlertPlugin		*gap;
+	GkrellmAlertPluginLink	*apl;
+	gchar					*s, *p;
 
 	if (!f || !alert || !mon_keyword)
 		return;
@@ -1153,28 +1366,63 @@ gkrellm_save_alertconfig(FILE *f, GkrellmAlert *alert,
 		fprintf(f, "%s alarm_command %s\n", s, alert->alarm_command);
 	if (alert->warn_command && *alert->warn_command)
 		fprintf(f, "%s warn_command %s\n", s, alert->warn_command);
-	if (alert->voice_string && *alert->voice_string)
-		fprintf(f, "%s voice_string %s\n", s, alert->voice_string);
 	fprintf(f, "%s values %d %d %d %d\n", s, alert->do_panel_updates,
 			alert->check_high, alert->check_low, alert->check_hardwired);
-	fprintf(f, "%s repeat %d %d %d\n", s, alert->alarm_repeat_set,
-			alert->warn_repeat_set, alert->voice_repeat_set);
-	fprintf(f, "%s limits %.2f %.2f %.2f %.2f\n", s,
-			alert->high.alarm_limit, alert->high.warn_limit,
-			alert->low.warn_limit, alert->low.alarm_limit);
+	fprintf(f, "%s repeat %d %d\n", s, alert->alarm_repeat_set,
+			alert->warn_repeat_set);
+
+	/* 2.1.15: scale saved float values to avoid decimal points in the config
+	|  because of locale breakage if decimal point changes '.' <-> ',"
+	*/
+	fprintf(f, "%s limits %.0f %.0f %.0f %.0f\n", s,
+				alert->high.alarm_limit * GKRELLM_FLOAT_FACTOR,
+				alert->high.warn_limit  * GKRELLM_FLOAT_FACTOR,
+				alert->low.warn_limit   * GKRELLM_FLOAT_FACTOR,
+				alert->low.alarm_limit  * GKRELLM_FLOAT_FACTOR);
 	if (alert->delay_step > 0)
 		fprintf(f, "%s delay %d %d %d %d\n", s, alert->delay,
 					alert->delay_high, alert->delay_low, alert->delay_step);
 
+	/* name can be quoted, but the id_string should not have embedded quotes.
+	|  id_string is so a plugin can get a unique config name for an alert
+	*/
+	if (!alert->id_string)
+		{
+		p = name;
+		if (p && *p == '"')
+			++p;
+		alert->id_string = g_strconcat(mon_keyword, p ? "-" : NULL, p, NULL);
+		if ((p = strrchr(alert->id_string, '"')) != NULL)
+			*p = '\0';
+		for (p = alert->id_string; *p; ++p)
+			if (*p == '/' || *p == ' ')
+				*p = '-';
+		}
+	fprintf(f, "%s id_string %s\n", s, alert->id_string);
+
+	for (list = alert->plugin_list; list; list = list->next)
+		{
+		apl = (GkrellmAlertPluginLink *) list->data;
+		gap = apl->alert_plugin;
+		if (MONITOR_ENABLED(gap->mon) && gap->config_save_func)
+			{
+			p = g_strconcat(s, "plugin ", gap->name, NULL);
+			(*gap->config_save_func)(alert, apl->data, f, p, alert->id_string);
+			g_free(p);
+			}
+		}
 	g_free(s);
 	}
 
 void
 gkrellm_load_alertconfig(GkrellmAlert **ap, gchar *config_line)
 	{
-	GkrellmAlert *alert;
-	gchar		config[32], item[CFG_BUFSIZE];
-	gint		n;
+	GList					*list;
+	GkrellmAlert 			*alert;
+	GkrellmAlertPlugin		*gap;
+	gchar					config[32], item[CFG_BUFSIZE];
+	gchar					name[64], item1[CFG_BUFSIZE];
+	gint					n;
 
 	if (!ap || !config_line)
 		return;
@@ -1190,8 +1438,6 @@ gkrellm_load_alertconfig(GkrellmAlert **ap, gchar *config_line)
 		gkrellm_dup_string(&alert->alarm_command, item);
 	else if (!strcmp(config, "warn_command"))
 		gkrellm_dup_string(&alert->warn_command, item);
-	else if (!strcmp(config, "voice_string"))
-		gkrellm_dup_string(&alert->voice_string, item);
 	else if (!strcmp(config, "values"))
 		sscanf(item, "%d %d %d %d", &alert->do_panel_updates,
 			&alert->check_high, &alert->check_low, &alert->check_hardwired);
@@ -1202,13 +1448,46 @@ gkrellm_load_alertconfig(GkrellmAlert **ap, gchar *config_line)
 		gkrellm_alert_set_delay(alert, n);
 		}
 	else if (!strcmp(config, "repeat"))
-		sscanf(item, "%d %d %d", &alert->alarm_repeat_set,
-				&alert->warn_repeat_set, &alert->voice_repeat_set);
+		sscanf(item, "%d %d", &alert->alarm_repeat_set,
+				&alert->warn_repeat_set);
 	else if (!strcmp(config, "limits"))
+		{
 		sscanf(item, "%f %f %f %f",
 				&alert->high.alarm_limit, &alert->high.warn_limit,
 				&alert->low.warn_limit, &alert->low.alarm_limit);
-
+		
+		alert->high.alarm_limit /= _GK.float_factor;
+		alert->high.warn_limit  /= _GK.float_factor;
+		alert->low.warn_limit   /= _GK.float_factor;
+		alert->low.alarm_limit  /= _GK.float_factor;
+		}
+	else if (!strcmp(config, "id_string"))
+		{
+		gkrellm_dup_string(&alert->id_string, item);
+		for (list = alert_plugin_list; list; list = list->next)
+			{
+			gap = (GkrellmAlertPlugin *) list->data;
+			if (MONITOR_ENABLED(gap->mon) && gap->config_load_func)
+				(*gap->config_load_func)(alert, "id_string", alert->id_string);
+			}
+		}
+	else if (!strcmp(config, "plugin"))
+		{
+		if (sscanf(item, "%63s %[^\n]", name, item1) == 2)
+			{
+			for (list = alert_plugin_list; list; list = list->next)
+				{
+				gap = (GkrellmAlertPlugin *) list->data;
+				if (!strcmp(name, gap->name))
+					{
+					if (MONITOR_ENABLED(gap->mon) && gap->config_load_func)
+						(*gap->config_load_func)(alert,
+									item1, alert->id_string);
+					break;
+					}
+				}
+			}
+		}
 	alert->activated = TRUE;
 	}
 
