@@ -532,7 +532,7 @@ set_auto_grid_resolution(GkrellmChart *cp, gint maxval)
 		{
 		cf->grid_resolution = grid_res;
 		set_grid_resolution_spin_button(cp, grid_res);
-		if (cf->cb_grid_resolution)
+		if (cf->cb_grid_resolution && !cf->cb_block)
 			(*cf->cb_grid_resolution)(cf, cf->cb_grid_resolution_data);
 		cp->redraw_all = TRUE;
 		}
@@ -1888,7 +1888,7 @@ cb_chart_grid_resolution(GtkWidget *adjustment, GkrellmChart *cp)
 						cf->low, cf->high, FALSE, FALSE);
 			cf->grid_resolution = res;
 			gtk_spin_button_set_value(spin, (gfloat) res);
-			if (cf->cb_grid_resolution)
+			if (cf->cb_grid_resolution && !cf->cb_block)
 				(*cf->cb_grid_resolution)(cf, cf->cb_grid_resolution_data);
 			gkrellm_refresh_chart(cp);
 			}
@@ -1901,7 +1901,7 @@ cb_chart_grid_resolution(GtkWidget *adjustment, GkrellmChart *cp)
 		cf->grid_resolution = (gint) fres;
 		if (cf->grid_resolution < 1)
 			cf->grid_resolution = 1;
-		if (cf->cb_grid_resolution)
+		if (cf->cb_grid_resolution && !cf->cb_block)
 			(*cf->cb_grid_resolution)(cf, cf->cb_grid_resolution_data);
 		gkrellm_refresh_chart(cp);
 		}
@@ -1909,6 +1909,14 @@ cb_chart_grid_resolution(GtkWidget *adjustment, GkrellmChart *cp)
 
 
 /* ---- GkrellmChartconfig functions ---- */
+void
+gkrellm_chartconfig_callback_block(GkrellmChartconfig *cf, gboolean block)
+	{
+	if (!cf)
+		return;
+	cf->cb_block = block;
+	}
+
 void
 gkrellm_set_chartconfig_grid_resolution(GkrellmChartconfig *cf, gint res)
 	{
@@ -2055,11 +2063,12 @@ gkrellm_chartconfig_height_connect(GkrellmChartconfig *cf,
 void
 gkrellm_set_chart_height(GkrellmChart *cp, gint h)
 	{
-	GtkWidget		*top_win = gkrellm_get_top_window();
-	GList			*list;
-	GkrellmChartdata   *cd;
-	GkrellmChartconfig *cf;
-	gint			dy;
+	GtkWidget			*top_win = gkrellm_get_top_window();
+	GtkSpinButton		*spin;
+	GList				*list;
+	GkrellmChartdata	*cd;
+	GkrellmChartconfig	*cf;
+	gint				dy;
 
 	if (!cp || cp->h == h)
 		return;
@@ -2078,7 +2087,7 @@ gkrellm_set_chart_height(GkrellmChart *cp, gint h)
 	render_chart_pixmaps(cp);
 
 	cf = cp->config;
-	if (cf->cb_height)
+	if (cf->cb_height && !cf->cb_block)
 		(*cf->cb_height)(cf, cf->cb_height_data);
 	gtk_widget_set_size_request(cp->drawing_area, cp->w, cp->h);
 	set_chartdata_split_heights(cp);
@@ -2088,6 +2097,12 @@ gkrellm_set_chart_height(GkrellmChart *cp, gint h)
 		gkrellm_pack_side_frames();
 		gkrellm_refresh_chart(cp);
 		}
+	if (cp->config_window)
+		{
+		spin = GTK_SPIN_BUTTON(cf->height_spin_button);
+		if (h != gtk_spin_button_get_value_as_int(spin))
+			gtk_spin_button_set_value(spin, (gfloat) h);
+		}
 	}
 
 gboolean
@@ -2166,7 +2181,7 @@ cb_chart_fixed_grids(GtkWidget *adjustment, GkrellmChart *cp)
 	cf->fixed_grids = gtk_spin_button_get_value_as_int(spin);
 	if (cf->auto_grid_resolution)
 		set_auto_grid_resolution(cp, cp->maxval_auto);
-	if (cf->cb_fixed_grids)
+	if (cf->cb_fixed_grids && !cf->cb_block)
 		(*cf->cb_fixed_grids)(cf, cf->cb_fixed_grids_data);
 
 	set_resolution_menubar_items_sensitivity(cf);
@@ -2506,9 +2521,9 @@ gkrellm_save_chartconfig(FILE *f, GkrellmChartconfig *cf, gchar *mon_keyword,
 	for (list = cf->cd_list; list; list = list->next)
 		{
 		cd = (GkrellmChartdata *) list->data;
-		fprintf(f, " : %d %d %d %d %.2f",
+		fprintf(f, " : %d %d %d %d %.0f",
 				cd->draw_style, cd->inverted, cd->hide,
-				cd->split_chart, cd->split_fraction);
+				cd->split_chart, cd->split_fraction * GKRELLM_FLOAT_FACTOR);
 		}
 	fprintf(f, "\n");
 	}
@@ -2549,6 +2564,7 @@ gkrellm_load_chartconfig(GkrellmChartconfig **config, gchar *string,
 		sscanf(s, "%d %d %d %d %f",
 				&cd->draw_style, &cd->inverted, &cd->hide,
 				&cd->split_chart, &cd->split_fraction);
+		cd->split_fraction /= _GK.float_factor;
 		if (cd->split_fraction <= 0.01 || cd->split_fraction >= 0.99)
 			cd->split_fraction = 0.5;
 
