@@ -264,7 +264,7 @@ client_disk_line_from_server(gchar *line)
 	guint64		rblk, wblk;
 	gint		n;
 
-	n = sscanf(line, "%16s %32s %32s %32s", name, s1, s2, s3);
+	n = sscanf(line, "%15s %31s %31s %31s", name, s1, s2, s3);
 	if (n == 4)
 		{
 		rblk = strtoull(s2, NULL, 0);
@@ -365,7 +365,7 @@ client_net_line_from_server(gchar *line)
 	gchar		name[32];
 	gulong		rx, tx;
 
-	sscanf(line, "%32s %lu %lu", name, &rx, &tx);
+	sscanf(line, "%31s %lu %lu", name, &rx, &tx);
 	for (list = net_list; list; list = list->next)
 		{
 		net = (NetData *) list->data;
@@ -394,7 +394,7 @@ client_net_routed_line_from_server(gchar *line)
 	gchar		name[32];
 	gboolean	routed;
 
-	sscanf(line, "%32s %d", name, &routed);
+	sscanf(line, "%31s %d", name, &routed);
 	for (list = net_list; list; list = list->next)
 		{
 		net = (NetData *) list->data;
@@ -488,7 +488,7 @@ client_inet_line_from_server(gchar *line)
 
 	if (*(line + 1) == '0')
 		{
-		n = sscanf(line + 3, "%x %128[^:]:%x", &tcp.local_port,
+		n = sscanf(line + 3, "%x %127[^:]:%x", &tcp.local_port,
 			   buf, &tcp.remote_port);
 		if (n != 3 || inet_aton(buf, &tcp.remote_addr) == 0)
 			return;
@@ -609,17 +609,7 @@ struct
 static void
 client_mem_line_from_server(gchar *line)
 	{
-	gchar	*fmt;
-
-/* Solaris 8 doesn't grok %L and I've seen a bug report about a FreeBSD
-|  %ll bug: http://www.freebsd.org/cgi/query-pr.cgi?pr=34682
-*/
-#if defined(__solaris__)
-	fmt = "%lld %lld %lld %lld %lld %lld";
-#else
-	fmt = "%Lu %Lu %Lu %Lu %Lu %Lu";
-#endif
-	sscanf(line, fmt,
+	sscanf(line, "%llu %llu %llu %llu %llu %llu",
 			&mem.total, &mem.used, &mem.free,
 			&mem.shared, &mem.buffers, &mem.cached);
 	}
@@ -627,14 +617,7 @@ client_mem_line_from_server(gchar *line)
 static void
 client_swap_line_from_server(gchar *line)
 	{
-	gchar	*fmt;
-
-#if defined(__solaris__)
-	fmt = "%lld %lld %lu %lu";
-#else
-	fmt = "%Lu %Lu %lu %lu";
-#endif
-	sscanf(line, "%Lu %Lu %lu %lu",
+	sscanf(line, "%llu %llu %lu %lu",
 			&mem.swap_total, &mem.swap_used,
 			&mem.swap_in, &mem.swap_out);
 	}
@@ -702,7 +685,7 @@ client_fstab_line_from_server(gchar *line)
 	else
 		{
 		m = g_new0(Mount, 1);
-		sscanf(line, "%128s %64s %64s", dir, dev, type);
+		sscanf(line, "%127s %63s %63s", dir, dev, type);
 		m->directory = g_strdup(dir);
 		m->device = g_strdup(dev);
 		m->type = g_strdup(type);
@@ -733,7 +716,7 @@ client_mounts_line_from_server(gchar *line)
 	else
 		{
 		m = g_new0(Mount, 1);
-		sscanf(line, "%128s %64s %64s %lu %lu %lu %lu", dir, dev, type,
+		sscanf(line, "%127s %63s %63s %lu %lu %lu %lu", dir, dev, type,
 				&m->blocks, &m->bavail, &m->bfree, &m->bsize);
 		m->directory = g_strdup(dir);
 		m->device = g_strdup(dev);
@@ -750,7 +733,7 @@ client_fs_line_from_server(gchar *line)
 	gchar	dir[128], dev[64];
 	gulong	blocks, bavail, bfree, bsize;
 
-	sscanf(line, "%128s %64s %lu %lu %lu %lu", dir, dev,
+	sscanf(line, "%127s %63s %lu %lu %lu %lu", dir, dev,
 				&blocks, &bavail, &bfree, &bsize);
 	for (list = mounts_list; list; list = list->next)
 		{
@@ -834,6 +817,72 @@ client_fs_setup(gchar *line)
 		mounting_unsupported = TRUE;
 	}
 
+/* ================================================================= */
+typedef struct
+	{
+	gchar		*path;
+	gint		total,
+				new;
+	gpointer	mbox_ptr;
+	}
+	Mailbox;
+
+static GList	*mailbox_list;
+
+static gboolean
+check_mail(Mailbox *mbox)
+	{
+	gkrellm_set_external_mbox_counts(mbox->mbox_ptr, mbox->total, mbox->new);
+	return TRUE;
+	}
+
+static void
+client_mail_line_from_server(gchar *line)
+	{
+	Mailbox		*mbox = NULL;
+	GList		*list;
+	gchar		path[256];
+//	gchar		*s;
+	gint		total, new;
+
+	if (sscanf(line, "%255s %d %d", path, &total, &new) < 3)
+		return;
+	for (list = mailbox_list; list; list = list->next)
+		{
+		mbox = (Mailbox *) list->data;
+		if (!strcmp(path, mbox->path))
+			break;
+		}
+	if (!list)
+		{
+		mbox = g_new0(Mailbox, 1);
+		mbox->path = g_strdup(path);
+		mailbox_list = g_list_append(mailbox_list, mbox);
+		mbox->mbox_ptr =
+				gkrellm_add_external_mbox(check_mail, FALSE, mbox);
+		gkrellm_set_external_mbox_tooltip(mbox->mbox_ptr, mbox->path);
+
+//		s = g_strdup_printf("%s:%s", _GK.server_hostname, mbox->path);
+//		gkrellm_set_external_mbox_tooltip(mbox->mbox_ptr, s);
+//		g_free(s);
+		}
+	if (mbox)
+		{
+		mbox->total = total;
+		mbox->new = new;
+		}
+	}
+
+static void
+client_sys_mail_init(void)
+	{
+	}
+
+static void
+client_mail_setup(gchar *line)
+	{
+	}
+
 /* ================================================================= */
 GList		*battery_list;
 
@@ -850,6 +899,7 @@ typedef struct
 	}
 	Battery;
 
+static Battery	*composite_battery;
 
 static Battery *
 battery_nth(gint n)
@@ -857,8 +907,23 @@ battery_nth(gint n)
 	Battery		*bat;
 	static gint	n_batteries;
 
-	if (n < 0 || n > 10)
+	if (n > 10)
 		return NULL;
+	if (n < 0)
+		{
+		if (!composite_battery)
+			{
+			bat = g_new0(Battery, 1);
+			battery_list = g_list_prepend(battery_list, bat);
+			bat->id = GKRELLM_BATTERY_COMPOSITE_ID;
+			composite_battery = bat;
+			}
+		return composite_battery;
+		}
+
+	if (composite_battery)
+		++n;
+
 	while ((bat = (Battery *)g_list_nth_data(battery_list, n)) == NULL)
 		{
 		bat = g_new0(Battery, 1);
@@ -954,7 +1019,7 @@ client_sensors_line_from_server(gchar *line)
 
 	if (need_locale_fix)
 		locale_fix(line);
-	sscanf(line, "%d \"%128[^\"]\" %d %d %d %f",
+	sscanf(line, "%d \"%127[^\"]\" %d %d %d %f",
 			&s.type, basename, &s.id, &s.iodev, &s.inter, &s.raw_value);
 	for (list = sensors_list; list; list = list->next)
 		{
@@ -1065,7 +1130,7 @@ client_sensors_setup(gchar *line)
 
 	s = g_new0(Sensor, 1);
 	if (sscanf(line,
-			"%d \"%128[^\"]\" %d %d %d %f %f \"%32[^\"]\" \"%32[^\"]\"",
+			"%d \"%127[^\"]\" %d %d %d %f %f \"%31[^\"]\" \"%31[^\"]\"",
 			&s->type, basename, &s->id, &s->iodev, &s->inter,
 			&s->factor, &s->offset, vref, default_label) == 9)
 		{
@@ -1153,6 +1218,7 @@ KeyTable	monitor_table[] =
 	{"inet",		client_sys_inet_init },
 	{"mem",			client_sys_mem_init },
 	{"fs",			client_sys_fs_init },
+	{"mail",		client_sys_mail_init },
 	{"apm",			client_sys_battery_init },
 	{"battery",		client_sys_battery_init },
 	{"uptime",		client_sys_uptime_init },
@@ -1232,6 +1298,7 @@ KeyTable	setup_table[] =
 	{"<inet_setup>",	client_inet_setup },
 	{"<net_setup>",		client_net_setup },
 	{"<fs_setup>",		client_fs_setup },
+	{"<mail_setup>",	client_mail_setup },
 	{"<apm_setup>",		client_battery_setup },
 	{"<battery_setup>",	client_battery_setup },
 	{"<time>",			client_time_line_from_server},
@@ -1255,6 +1322,7 @@ KeyTable	update_table[] =
 	{"<fs_fstab>",		client_fstab_line_from_server},
 	{"<fs_mounts>",		client_mounts_line_from_server},
 	{"<inet>",			client_inet_line_from_server},
+	{"<mail>",			client_mail_line_from_server},
 	{"<apm>",			client_battery_line_from_server},
 	{"<battery>",		client_battery_line_from_server},
 	{"<sensors>",		client_sensors_line_from_server},
@@ -1507,7 +1575,7 @@ gkrellm_client_mode_connect(void)
 	if ((client_fd = connect_to_server()) < 0)
 		return FALSE;
 
-	snprintf(buf, sizeof(buf), "gkrellm %d.%d.%d %s\n",
+	snprintf(buf, sizeof(buf), "gkrellm %d.%d.%d%s\n",
 			GKRELLM_VERSION_MAJOR, GKRELLM_VERSION_MINOR,
 			GKRELLM_VERSION_REV, GKRELLM_EXTRAVERSION);
 	send(client_fd, buf, strlen(buf), 0);
