@@ -1136,7 +1136,7 @@ assign_gkrellmrc_spacer(gchar *source_line, gchar *area, gchar *string)
 	gchar			style_name[32], arg[32], *s;
 	gint			n;
 
-	if ((n = sscanf(string, "%32s %32[^\n]", style_name, arg)) < 1)
+	if ((n = sscanf(string, "%31s %31[^\n]", style_name, arg)) < 1)
 		return;
 	if (n == 1)
 		strcpy(arg, style_name);
@@ -1531,7 +1531,7 @@ gkrellm_set_gkrellmrc_piximage_border(gchar *image_name,
 		s = list->data;
 		if ((r = strchr(s, '=')) != NULL)
 			*r = ' ';
-		sscanf(s, "%64s %32s", name, border_string);
+		sscanf(s, "%63s %31s", name, border_string);
 		if (!strcmp(name, image_name))
 			{
 			set_border(&b, border_string);
@@ -1560,7 +1560,7 @@ gkrellm_get_gkrellmrc_piximage_border(gchar *image_name, GkrellmPiximage *image,
 		s = list->data;
 		if ((r = strchr(s, '=')) != NULL)
 			*r = ' ';
-		sscanf(s, "%64s %32s", name, border_string);
+		sscanf(s, "%63s %31s", name, border_string);
 		if (!strcmp(name, image_name))
 			{
 			set_border(&b, border_string);
@@ -1588,7 +1588,7 @@ gkrellm_get_gkrellmrc_integer(gchar *int_name, gint *result)
 		s = list->data;
 		if ((r = strchr(s, '=')) != NULL)
 			*r = ' ';
-		sscanf(s, "%64s %64s", name, string);
+		sscanf(s, "%63s %63s", name, string);
 		if (!strcmp(name, int_name) && sscanf(string, "%d", result) == 1)
 			found = TRUE;
 		}
@@ -1609,7 +1609,7 @@ gkrellm_get_gkrellmrc_string(gchar *string_name)
 		s = list->data;
 		if ((r = strchr(s, '=')) != NULL)
 			*r = ' ';
-		sscanf(s, "%64s %[^\n]", name, string);
+		sscanf(s, "%63s %[^\n]", name, string);
 		if (!strcmp(name, string_name))
 			{
 			if ((s = cut_quoted_string(string, NULL)) != NULL)
@@ -1708,7 +1708,7 @@ parse_config_line(gchar *line, struct _config *cf, gint size)
 	{
 	struct _config	*conf;
 	gchar			*s, *ss;
-	gint			i, val;
+	gint			i, val, len;
 
 	s = line;
 	while (*s && *s != ' ' && *s != '\t' && *s != '=')
@@ -1718,7 +1718,8 @@ parse_config_line(gchar *line, struct _config *cf, gint size)
 	for (i = 0; i < size; ++i)
 		{
 		conf = cf + i;
-		if (strncmp(conf->option, line, s - line))
+		len = strlen(conf->option);
+		if (strncmp(conf->option, line, len) || len != s - line)
 			continue;
 		while (*s == ' ' || *s == '\t' || *s == '=')
 			++s;
@@ -2252,7 +2253,8 @@ struct	_config	user_config[] =
 	{"on_top",			&_GK.on_top,		NULL,		0  },
 #endif
 
-	{"track_gtk_rcfiles", &_GK.track_gtk_rcfiles, NULL,	0  },
+	{"track_gtk_theme_name", &_GK.track_gtk_theme_name, NULL,	0  },
+	{"default_track_theme", NULL, &_GK.default_track_theme,	0  },
 	{"save_position",	&_GK.save_position,		NULL,	0  },
 
 	{"chart_width",		&_GK.chart_width,		NULL,	0  },
@@ -2269,7 +2271,7 @@ void
 load_user_config(gboolean monitor_values)
 	{
 	FILE	*f;
-	gchar	*s, *ss, *config;
+	gchar	*s, *ss, *config, *config_2_1_14 = NULL;
 	gchar	buf[CFG_BUFSIZE];
 
 	if (!monitor_values)
@@ -2280,17 +2282,26 @@ load_user_config(gboolean monitor_values)
 		_GK.chart_width = 60;
 		_GK.update_HZ = 10;
 		_GK.theme_scale = 100;
+		_GK.float_factor = 1.0;
+		_GK.default_track_theme = g_strdup("Default");
 		}
 	config = gkrellm_make_config_file_name(gkrellm_homedir(),
 				GKRELLM_USER_CONFIG);
 	f = fopen(config, "r");
-	g_free(config);
+
+	if (!f)		/* >= 2.1.15 configs are not backward compatible so there is */
+		{		/* a new config file name.  < 2.1.15 is forward compatible   */
+		config_2_1_14 = gkrellm_make_config_file_name(gkrellm_homedir(),
+					GKRELLM_2_1_14_CONFIG);
+		f = fopen(config_2_1_14, "r");
+		}
 	if (!f)		/* Upgrade from 1.2 to 2.0 ? */
 		{
 		config = gkrellm_make_config_file_name(gkrellm_homedir(),
 					".gkrellm/user_config");
 		f = fopen(config, "r");
-		g_free(config);
+		g_free(config_2_1_14);
+		config_2_1_14 = NULL;
 		}
 	if (f)
 		{
@@ -2304,7 +2315,9 @@ load_user_config(gboolean monitor_values)
 			ss = strchr(s, '\n');
 			if (ss)
 				*ss = '\0';
-			if (monitor_values)
+			if (!strncmp(s, "float_factor ", 13))
+				sscanf(s + 13, "%f", &_GK.float_factor);
+			else if (monitor_values)
 				parse_monitor_config_keyword(s);
 			else
 				parse_config_line(s, &user_config[0],
@@ -2316,6 +2329,10 @@ load_user_config(gboolean monitor_values)
 		_GK.chart_width = CHART_WIDTH_MIN;
 	if (_GK.chart_width > CHART_WIDTH_MAX)
 		_GK.chart_width = CHART_WIDTH_MAX;
+//	if (config_2_1_14)
+//		rename(config_2_1_14, config);
+	g_free(config);
+	g_free(config_2_1_14);
 	}
 
 void
@@ -2426,6 +2443,7 @@ save_user_config(void)
 			fprintf(f, "%s \"%s\"\n",user_config[i].option,
 						*(user_config[i].arg));
 		}
+	fprintf(f, "float_factor %.0f\n", GKRELLM_FLOAT_FACTOR);
 	for (list = gkrellm_monitor_list; list; list = list->next)
 		{
 		mon = (GkrellmMonitor *) list->data;
