@@ -92,7 +92,7 @@ static gboolean	omit_nice_mode,
 				config_tracking,
 				sys_handles_composite_data,
 				nice_time_unsupported;
-static gint		ascent;
+
 static gint		style_id;
 static gint		sensor_separate_mode;
 
@@ -448,7 +448,11 @@ update_cpu(void)
 			if (alert_includes_nice)
 				alert_value += cpu->nice;
 			alert_total_diff = total - cpu->previous_alert_total;
-			if (alert_total_diff > 0)
+			if (   alert_total_diff > 0
+				&& (   ( cpu->is_composite && smp_mode == SMP_COMPOSITE_MODE)
+					|| (!cpu->is_composite && smp_mode != SMP_COMPOSITE_MODE)
+				   )
+			   )
 				gkrellm_check_alert(cpu->alert,
 					100.0 * (gfloat) (alert_value - cpu->previous_alert_value)
 					/ (gfloat) alert_total_diff);
@@ -535,24 +539,6 @@ cb_cpu_extra(GtkWidget *widget, GdkEventButton *ev)
 	return FALSE;
 	}
 
-static void
-cb_cpu_height(GkrellmChartconfig *cf, gpointer data)
-	{
-	GList			*list;
-	GkrellmChart	*cp;
-	gint			h;
-
-	if (!config_tracking)
-		return;
-	h = gkrellm_get_chartconfig_height(cf);
-	for (list = cpu_mon_list; list; list = list->next)
-		{
-		cp = ((CpuMon *) list->data)->chart;
-		if (cp->h != h)
-			gkrellm_set_chart_height(cp, h);
-		}
-	}
-
 static void
 setup_cpu_scaling(GkrellmChartconfig *cf)
 	{
@@ -735,6 +721,30 @@ static GdkColor			nice_color,
 static gchar			*nice_color_string,
 						*nice_grid_color_string;
 
+static void
+cb_height(GkrellmChartconfig *cf, CpuMon *cpu)
+	{
+	GList			*list;
+	GkrellmChart	*cp;
+	gint			h, h_max = 0;
+
+	h = gkrellm_get_chartconfig_height(cf);
+	for (list = cpu_mon_list; list; list = list->next)
+		{
+		cp = ((CpuMon *) list->data)->chart;
+		if (cp->h != h && config_tracking)
+			{
+			gkrellm_chartconfig_callback_block(cp->config, TRUE);
+			gkrellm_set_chart_height(cp, h);
+			gkrellm_chartconfig_callback_block(cp->config, FALSE);
+			}
+		if (cp->h > h_max)
+			h_max = cp->h;
+		}
+	gkrellm_render_data_pixmap(nice_data_piximage,
+				&nice_data_pixmap, &nice_color, h_max);
+	}
+
 static void
 load_nice_data_piximages(void)
 	{
@@ -793,7 +803,6 @@ create_cpu(GtkWidget *vbox, gint first_create)
 	GList			*list;
 
 	load_nice_data_piximages();
-	ascent = 0;
 	for (list = cpu_mon_list; list; list = list->next)
 		{
 		cpu = (CpuMon *) list->data;
@@ -846,7 +855,7 @@ create_cpu(GtkWidget *vbox, gint first_create)
 		gkrellm_set_chartconfig_auto_grid_resolution(cp->config, FALSE);
 		gkrellm_chartconfig_fixed_grids_connect(cp->config,
 					setup_cpu_scaling, NULL);
-		gkrellm_chartconfig_height_connect(cp->config, cb_cpu_height, NULL);
+		gkrellm_chartconfig_height_connect(cp->config, cb_height, cpu);
 		setup_cpu_scaling(cp->config);
 
 		if (! (!strcmp(cpu->name, "cpu") && cpu->is_composite))
@@ -920,7 +929,7 @@ dup_cpu_alert(void)
 	GList	*list;
 	CpuMon	*cpu;
 
-	for (list = cpu_mon_list->next; list; list = list->next)
+	for (list = cpu_mon_list; list; list = list->next)
 		{
 		cpu = (CpuMon *) list->data;
 		gkrellm_alert_dup(&cpu->alert, cpu_alert);
@@ -1175,6 +1184,7 @@ cb_enable(GtkWidget *button, gpointer data)
 			gkrellm_reset_and_draw_chart(cpu->chart);
 		gkrellm_apply_launcher(&cpu->launch_entry, &cpu->tooltip_entry,
 					cpu->chart->panel, &cpu->launch, gkrellm_launch_button_cb);
+		gkrellm_reset_alert_soft(cpu->alert);
 		}
 	if (cpu_enabled)
 		gkrellm_spacers_show(mon_cpu);
