@@ -52,6 +52,7 @@ typedef struct
 	GkrellmDecal *mount_decal,
 				*eject_decal,
 				*text_decal;
+	GkrellmKrell *krell;
 	gchar		*label,			/* Actual utf8 label */
 				*label_shadow;	/* Shadow label for gdk_draw functions */
 	gboolean	label_is_data,
@@ -738,8 +739,7 @@ printf("fs update %d nfs %d force %d\n", fs_check, nfs_check, force_fs_check);
 		{
 		fs = (FSmon *) list->data;
 		p  = fs->panel;
-		k = KRELL(p);
-		k->previous = 0;
+		k = fs->krell;
 		mounting_enabled = FS_MOUNTING_ENABLED(fs);
 		if (fs_is_mounted(fs))
 			{
@@ -824,7 +824,16 @@ printf("fs update %d nfs %d force %d\n", fs_check, nfs_check, force_fs_check);
 		if (!fs->label_is_data && fs != fs_in_motion)
 			draw_fs_text_decal(fs, 0);
 
+		if (_GK.client_mode)
+			{
+			if (fs->blocks == 0)
+				gkrellm_remove_krell(p, k);
+			else
+				gkrellm_insert_krell(p, k, FALSE);
+			}
+
 		gkrellm_draw_panel_layers(p);
+
 		if (   fs->ejectable
 			&& fs->eject_pending && fs->eject_pending < GK.timer_ticks
 		   )
@@ -1111,7 +1120,9 @@ create_fs_monitor(GtkWidget *vbox, FSmon *fs, gint index, gint first_create)
 	*/
 	text_decal_width = fs->mount_decal->x - fs->text_decal->x;
 
-	gkrellm_create_krell(p, gkrellm_krell_meter_piximage(style_id), style);
+	fs->krell = gkrellm_create_krell(p,
+						gkrellm_krell_meter_piximage(style_id), style);
+	gkrellm_monotonic_krell_values(fs->krell, FALSE);
 
 	gkrellm_panel_configure(p, NULL, style);
 	gkrellm_panel_create(vbox, mon_fs, p);
@@ -1374,7 +1385,7 @@ load_fs_config(gchar *arg)
 		}
 	else if (!strcmp(config, GKRELLM_ALERTCONFIG_KEYWORD))
 		{
-		if (   sscanf(item, "\"%64[^\"]\" %[^\n]", name, item1) == 2
+		if (   sscanf(item, "\"%63[^\"]\" %[^\n]", name, item1) == 2
 			&& (fs = lookup_fs(name)) != NULL
 		   )
 			{
