@@ -30,6 +30,8 @@
 #define	DEBUG_CLIENT	0x1000
 #define DEBUG_GUI		0x2000
 
+#define	GKRELLM_FLOAT_FACTOR	1000.0
+
 
 /* Private global extern declarations and function prototypes.
 */
@@ -66,6 +68,9 @@ struct	GkrellmConfig
 	gchar		*command_line_theme;
 	gchar		*command_line_plugin;
 
+	gchar		*gtk_theme_name;
+	GtkSettings	*gtk_settings;
+
 	gchar		*server;
 	gint		server_port;
 	gchar		*server_hostname;
@@ -103,10 +108,12 @@ struct	GkrellmConfig
 	gboolean	config_modified;
 	gboolean	any_transparency;
 
+	gboolean	track_gtk_theme_name;
+	gchar		*default_track_theme;
+
 	gint		update_HZ;
 	gint		chart_width;
-	gboolean	save_position;
-	gboolean	track_gtk_rcfiles,
+	gboolean	save_position,
 				withdrawn,
 				on_top;
 	gboolean	sticky_state,
@@ -123,6 +130,8 @@ struct	GkrellmConfig
 				hostname_short,
 				enable_system_name;
 
+	gfloat		float_factor;			/* avoid config locale breakage */
+
 	gint		rx_led_x;				/* Move these to net monitor */
 	gint		rx_led_y;
 	gint		tx_led_x;
@@ -291,7 +300,6 @@ void load_plugin_monitors(void);
 void	create_plugin_config(GtkWidget *);
 void	close_plugins_config(void);
 
-/* hostname?? */
 GkrellmMonitor *init_host_monitor(void);
 GkrellmMonitor *init_cal_monitor(void);
 GkrellmMonitor *init_clock_monitor(void);
@@ -354,7 +362,7 @@ void	save_user_config(void);
 void	save_theme_config(void);
 void	load_theme_config(void);
 void	load_theme_piximages(void);
-void	read_theme_event(gint);
+void	gkrellm_read_theme_event(GtkSettings *settings);
 void	make_themes_list(void);
 gchar	*get_large_font_override(void);
 gchar	*get_normal_font_override(void);
@@ -391,6 +399,7 @@ void	start_timer(gint);
 GtkItemFactory *create_item_factory_popup(void);
 
 void	gkrellm_apply_hostname_config(void);
+gboolean gkrellm_hostname_can_shorten(void);
 
 gchar	*gkrellm_proc_get_sensor_panel_label(void);
 gboolean gkrellm_proc_set_sensor(gpointer sr, gint type);
