@@ -54,6 +54,8 @@ void	gkrellm_sys_main_cleanup(void);
 gchar	*gkrellm_sys_get_host_name(void);
 gchar	*gkrellm_sys_get_system_name(void);
 
+gint	gkrellm_get_timer_ticks(void);
+
   /* If local mailboxes are not applicable, call this from
   |  gkrellm_sys_main_init().
   */
@@ -247,6 +249,8 @@ void	gkrellm_fs_mounting_unsupported(void);
 /* Battery monitor interface
 */
 
+#define	GKRELLM_BATTERY_COMPOSITE_ID	-1
+
 gboolean gkrellm_sys_battery_init(void);
 void	gkrellm_sys_battery_read_data(void);
 
@@ -283,4 +287,3 @@ void	gkrellm_sensors_client_divert(gboolean (*get_temp_func)(),
 void	gkrellm_sensors_add_sensor(gint type, gchar *dir, gchar *basename,
         	gint id, gint iodev, gint inter,
         	gfloat factor, gfloat offset, gchar *vref, gchar *default_label);
-
