@@ -89,7 +89,7 @@
 */
 #define	GKRELLM_VERSION_MAJOR	2
 #define	GKRELLM_VERSION_MINOR	1
-#define	GKRELLM_VERSION_REV		10
+#define	GKRELLM_VERSION_REV		16
 #define	GKRELLM_EXTRAVERSION	""
 
 #define GKRELLM_CHECK_VERSION(major,minor,rev)    \
@@ -99,7 +99,8 @@
 	GKRELLM_VERSION_REV >= (rev)))
 
 #define	GKRELLM_DIR				".gkrellm2"
-#define	GKRELLM_USER_CONFIG		".gkrellm2/user_config"
+#define	GKRELLM_USER_CONFIG		".gkrellm2/user-config"
+#define	GKRELLM_2_1_14_CONFIG	".gkrellm2/user_config"
 #define	GKRELLM_THEME_CONFIG	".gkrellm2/theme_config"
 #define GKRELLM_THEMES_DIR		".gkrellm2/themes"
 #define	GKRELLM_DATA_DIR		".gkrellm2/data"
@@ -143,7 +144,8 @@
 
   /* Features
   */
-#define	GKRELLM_HAVE_THEME_SCALE	1
+#define	GKRELLM_HAVE_THEME_SCALE		1
+#define	GKRELLM_HAVE_DECAL_TEXT_INSERT	1
 
 
   /* Label midpoints are positioned as a percent of chart_width.
@@ -524,6 +526,8 @@ typedef struct
 
 	GList		*cd_list;			/* Same list as in parent GkrellmChart */
 	GList		**chart_cd_list;
+
+	gboolean	cb_block;
 	}
 	GkrellmChartconfig;
 
@@ -692,12 +696,37 @@ typedef struct
 	gchar		*tooltip_comment;
 	GkrellmDecal *decal;		/* Used if DECAL_LAUNCHER type	*/
 	GkrellmMargin margin;
+
+	GtkWidget	*widget;
 	}
 	GkrellmLauncher;
 
 
 /* ------- Alerts ------- */
-#define	GKRELLM_ALERTCONFIG_KEYWORD	"alert_config"
+#define	GKRELLM_ALERTCONFIG_KEYWORD			"alert_config"
+
+typedef struct
+	{
+	struct _GkrellmMonitor	*mon;
+	gchar		*name,
+				*tab_name;
+	void		(*warn_func)(),
+				(*alarm_func)(),
+				(*update_func)(),
+				(*destroy_func)();
+	void		(*config_create_func)(),
+				(*config_apply_func)(),
+				(*config_save_func)(),
+				(*config_load_func)();
+	}
+	GkrellmAlertPlugin;
+
+typedef struct
+	{
+	GkrellmAlertPlugin	*alert_plugin;
+	gpointer			data;
+	}
+	GkrellmAlertPluginLink;
 
 typedef struct
 	{
@@ -747,14 +776,11 @@ typedef struct
 				check_high,
 				check_hardwired;
 	gchar		*warn_command,
-				*alarm_command,
-				*voice_string;
+				*alarm_command;
 	gint		warn_repeat_set,
 				warn_repeat,
 				alarm_repeat_set,
-				alarm_repeat,
-				voice_repeat_set,
-				voice_repeat;
+				alarm_repeat;
 	gint		delay;
 
 	void		(*cb_trigger)();
@@ -771,12 +797,11 @@ typedef struct
 	GtkWidget	*config_window,
 				*warn_command_entry,
 				*alarm_command_entry,
-				*voice_string_entry,
 				*warn_repeat_spin_button,
 				*alarm_repeat_spin_button,
-				*voice_repeat_spin_button,
 				*delay_spin_button,
-				*delete_button;
+				*delete_button,
+				*icon_box;
 	gboolean	do_alarm_command,
 				do_warn_command;
 
@@ -792,6 +817,12 @@ typedef struct
 
 	GkrellmTrigger low,
 				high;
+
+	gboolean	config_closing,
+				config_modified;
+
+	gchar		*id_string;		/* For unique alert names for alert plugins */
+	GList		*plugin_list;
 	}
 	GkrellmAlert;
 
@@ -899,6 +930,7 @@ extern GkrellmTicks	GK;
 #define	MONITOR_ID(m)			((m)->id & MON_ID_MASK)
 #define	MONITOR_CONFIG(m,flag)	(((m)->id & MON_CONFIG_MASK) & flag)
 
+#define	MONITOR_ENABLED(mon)	((mon)->privat->enabled)
 
 typedef struct
 	{
@@ -935,7 +967,7 @@ typedef struct
 
 
 
-typedef struct
+typedef struct _GkrellmMonitor
 	{
 	gchar		*name;
 	gint		id;
