@@ -152,24 +152,66 @@ text_view_append(GtkWidget *view, gchar *s)
 void
 gkrellm_gtk_text_view_append(GtkWidget *view, gchar *string)
 	{
-	text_view_append(view, string);
+	static gchar	*tag;
+	gchar			*s;
+
+	s = string;
+	if (   *s == '<'
+		&& (   (*(s + 2) == '>' && !*(s + 3))
+			|| (*(s + 3) == '>' && !*(s + 4))
+		   )
+	   )
+		{
+		tag = g_strdup(s);
+		return;
+		}
+	if (tag)
+		{
+		s = g_strconcat(tag, string, NULL);
+		text_view_append(view, s);
+		g_free(s);
+		g_free(tag);
+		tag = NULL;
+		}
+	else
+		text_view_append(view, string);
 	}
 
 void
 gkrellm_gtk_text_view_append_strings(GtkWidget *view, gchar **string,
 			gint n_strings)
 	{
-	gchar	*s;
+	gchar	*tag = NULL;
+	gchar	*s, *t;
 	gint	i;
 
 	for (i = 0; i < n_strings; ++i)
 		{
+		s = string[i];
+		if (   *s == '<'
+			&& (   (*(s + 2) == '>' && !*(s + 3))
+				|| (*(s + 3) == '>' && !*(s + 4))
+			   )
+		   )
+			{
+			tag = g_strdup(s);
+			continue;
+			}
 #if defined(ENABLE_NLS)
 		s = gettext(string[i]);
 #else
 		s = string[i];
 #endif
-		text_view_append(view, s);
+		if (tag)
+			{
+			t = g_strconcat(tag, s, NULL);
+			text_view_append(view, t);
+			g_free(t);
+			g_free(tag);
+			tag = NULL;
+			}
+		else
+			text_view_append(view, s);
 		}
 	}
 
@@ -215,9 +257,20 @@ gkrellm_launch_button_cb(GkrellmDecalbutton *button)
 	g_spawn_command_line_async(launch->command, NULL /* GError */);
 	}
 
+void
+gkrellm_remove_launcher(GkrellmLauncher *launch)
+	{
+	if (launch->button)
+		gkrellm_destroy_button(launch->button);
+	launch->button = NULL;
+	if (launch->tooltip)
+		gtk_tooltips_set_tip(launch->tooltip, launch->widget, NULL, NULL);
+	}
+
 void
 gkrellm_configure_tooltip(GkrellmPanel *p, GkrellmLauncher *launch)
 	{
+	launch->widget = p->drawing_area;
 	if (!launch->tooltip)
 		{
 		if (*launch->tooltip_comment && *launch->command)
@@ -249,11 +302,12 @@ gkrellm_apply_launcher(GtkWidget **launch_entry, GtkWidget **tooltip_entry,
 	command = gkrellm_gtk_entry_get_text(launch_entry);
 	tip_comment = tooltip_entry ?
 						gkrellm_gtk_entry_get_text(tooltip_entry) : "";
-	if (   !strcmp(command, launch->command)
+	if (   launch->command && !strcmp(command, launch->command)
+		&& launch->tooltip_comment
 		&& !strcmp(tip_comment, launch->tooltip_comment)
 	   )
 		return;
-	if (*command)		
+	if (*command)
 		{
 		if (!launch->button)
 			{
@@ -605,6 +659,41 @@ gkrellm_gtk_framed_vbox_end(GtkWidget *box, gchar *label,
 	return vbox;
 	}
 
+GtkWidget *
+gkrellm_gtk_category_vbox(GtkWidget *box, gchar *category_header,
+		gint header_pad, gint box_pad, gboolean pack_start)
+	{
+	GtkWidget	*vbox, *vbox1, *hbox, *label;
+	gchar		*s;
+
+	vbox = gtk_vbox_new(FALSE, 0);
+	if (pack_start)
+		gtk_box_pack_start(GTK_BOX(box), vbox, FALSE, FALSE, 0);
+	else
+		gtk_box_pack_end(GTK_BOX(box), vbox, FALSE, FALSE, 0);
+
+	if (category_header)
+		{
+		label = gtk_label_new(NULL);
+		s = g_strconcat("<span weight=\"bold\">", category_header,
+					"</span>",NULL);
+		gtk_label_set_markup(GTK_LABEL(label), s);
+		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+		gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, header_pad);
+		g_free(s);
+		}
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
+	label = gtk_label_new("    ");
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+	vbox1 = gtk_vbox_new(FALSE, box_pad);
+	gtk_box_pack_start(GTK_BOX(hbox), vbox1, FALSE, FALSE, 0);
+
+	return vbox1;
+	}
+
 GtkWidget *
 gkrellm_gtk_notebook_page(GtkWidget *tabs, char *name)
 	{
@@ -626,7 +715,7 @@ gkrellm_gtk_framed_notebook_page(GtkWidget *tabs, char *name)
 	GtkWidget	*vbox;
 
 	vbox = gkrellm_gtk_notebook_page(tabs, name);
-	vbox = gkrellm_gtk_framed_vbox(vbox, NULL, 2, TRUE, 0, 2);
+	vbox = gkrellm_gtk_framed_vbox(vbox, NULL, 2, TRUE, 4, 4);
 	return vbox;
 	}
 
@@ -640,11 +729,11 @@ create_about_tab(GtkWidget *vbox)
 	label = gtk_label_new("");
 	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
 
-	buf = g_strdup_printf(_("GKrellM %d.%d.%d %s\nGNU Krell Monitors\n\n"
+	buf = g_strdup_printf(_("GKrellM %d.%d.%d%s\nGNU Krell Monitors\n\n"
 				"Copyright (c) 1999-2003 by Bill Wilson\n"
 				"bill@gkrellm.net\n"
 				"http://gkrellm.net\n\n"
-				"Released under the GNU Public License"),
+				"Released under the GNU General Public License"),
 				GKRELLM_VERSION_MAJOR, GKRELLM_VERSION_MINOR,
 				GKRELLM_VERSION_REV, GKRELLM_EXTRAVERSION);
 	label = gtk_label_new(buf);
@@ -742,7 +831,8 @@ static void
 cb_hostname_sysname(GtkWidget *widget, gpointer data)
 	{
 	_GK.enable_hostname = GTK_TOGGLE_BUTTON(enable_hst_button)->active;
-	_GK.hostname_short = GTK_TOGGLE_BUTTON(hostname_short_button)->active;
+	if (hostname_short_button)
+		_GK.hostname_short = GTK_TOGGLE_BUTTON(hostname_short_button)->active;
 	_GK.enable_system_name = GTK_TOGGLE_BUTTON(enable_sysname_button)->active;
 	gkrellm_apply_hostname_config();
 	}
@@ -754,8 +844,6 @@ cb_general(void)
 	gint		n;
 	gboolean	new_state;
 
-	if (track_gtk_button)
-		_GK.track_gtk_rcfiles = GTK_TOGGLE_BUTTON(track_gtk_button)->active;
 	if (allow_multiple_button)
 		_GK.allow_multiple_instances =
 					GTK_TOGGLE_BUTTON(allow_multiple_button)->active;
@@ -922,10 +1010,11 @@ create_general_tab(GtkWidget *tab_vbox)
 			cb_hostname_sysname, NULL,
 			_("Hostname display"));
 
-	gkrellm_gtk_check_button_connected(hbox, &hostname_short_button,
-			_GK.hostname_short, FALSE, FALSE, 10,
-			cb_hostname_sysname, NULL,
-			_("Short hostname"));
+	if (gkrellm_hostname_can_shorten())
+		gkrellm_gtk_check_button_connected(hbox, &hostname_short_button,
+				_GK.hostname_short, FALSE, FALSE, 10,
+				cb_hostname_sysname, NULL,
+				_("Short hostname"));
 
 	gkrellm_gtk_check_button_connected(vbox, &enable_sysname_button,
 			_GK.enable_system_name, FALSE, FALSE, 0,
@@ -938,10 +1027,6 @@ create_general_tab(GtkWidget *tab_vbox)
 		_("Remember screen location at exit and move to it at next startup"));
 
 #if !defined(WIN32)
-	gkrellm_gtk_check_button_connected(vbox, &track_gtk_button,
-			_GK.track_gtk_rcfiles, FALSE, FALSE, 0,
-			cb_general, NULL,
-			_("Track Gtk theme changes for similarly named themes"));
 	gkrellm_gtk_check_button_connected(vbox, &allow_multiple_button,
 			_GK.allow_multiple_instances, FALSE, FALSE, 0,
 			cb_general, NULL,
@@ -1050,6 +1135,13 @@ enum
 	N_THEME_COLUMNS
 	};
 
+typedef struct
+	{
+	gchar	*path;
+	gchar	*name;
+	}
+	Theme;
+
 static GtkTreeView      *theme_treeview;
 static GtkTreeSelection *theme_selection;
 
@@ -1188,10 +1280,29 @@ get_theme_author(gchar *path)
 	return buf;
 	}
 
+static Theme *
+find_theme_in_list(gchar *name)
+	{
+	GList	*list;
+	Theme	*theme;
+
+	if (!name || !*name)
+		return NULL;
+
+	for (list = themes_list ; list; list = list->next)
+		{
+		theme = (Theme *) list->data;
+		if (!strcmp(theme->name, name))
+			return theme;
+		}
+	return NULL;
+	}
+
 static void
-add_themes_to_list(GList **list, gchar *theme_dir)
+add_themes_to_list(gchar *theme_dir, gboolean in_gkrellm2)
 	{
 	GDir	*dir;
+	Theme	*theme;
 	gchar	*name;
 	gchar	*path;
 
@@ -1199,11 +1310,20 @@ add_themes_to_list(GList **list, gchar *theme_dir)
 		return;
 	while ((name = (gchar *) g_dir_read_name(dir)) != NULL)
 		{
-		if (basename_in_list(*list, name))
-			continue;		/* Duplicate theme */
-		path = g_strconcat(theme_dir, G_DIR_SEPARATOR_S, name, NULL);
+		if (find_theme_in_list(name))
+			continue;
+		if (in_gkrellm2)
+			path = g_build_filename(theme_dir, name, "gkrellm2", NULL);
+		else
+			path = g_build_filename(theme_dir, name, NULL);
+
 		if (g_file_test(path, G_FILE_TEST_IS_DIR))
-			*list = g_list_append(*list, path);
+			{
+			theme = g_new0(Theme, 1);
+			theme->path = path;
+			theme->name = g_strdup(name);
+			themes_list = g_list_append(themes_list, theme);
+			}
 		else
 			g_free(path);
 		}
@@ -1213,37 +1333,73 @@ add_themes_to_list(GList **list, gchar *theme_dir)
 static void
 find_theme_position_in_list(void)
 	{
-	theme_position_in_list = string_in_list(themes_list,
-			(*(_GK.theme_path)) ? _GK.theme_path : "Default");
-	if (! theme_position_in_list)	/* Shouldn't happen */
-		theme_position_in_list = themes_list;
+	GList	*list;
+	Theme	*theme;
+	gchar	*name;
+
+	name = *(_GK.theme_path) ? _GK.theme_path : "Default";
+	for (list = themes_list; list; list = list->next)
+		{
+		theme = (Theme *) list->data;
+		if (!strcmp(name, theme->path))
+			break;
+		}
+	theme_position_in_list = list ? list : themes_list;
+	}
+
+gint
+theme_compare(Theme *th1, Theme *th2)
+	{
+	return strcmp(th1->name, th2->name);
 	}
 
 void
 make_themes_list(void)
 	{
+	GList	*list;
+	Theme	*theme;
 	gchar	*theme_dir;
 
+	for (list = themes_list; list; list = list->next)
+		{
+		theme = (Theme *) list->data;
+		g_free(theme->path);
+		g_free(theme->name);
+		}
 	free_glist_and_data(&themes_list);
-	themes_list = g_list_append(themes_list, g_strdup("Default"));
-	theme_dir = g_strdup_printf("%s/%s", gkrellm_homedir(),
-					GKRELLM_THEMES_DIR);
-	add_themes_to_list(&themes_list, theme_dir);
+
+	theme = g_new0(Theme, 1);
+	theme->path = g_strdup("Default");
+	theme->name = g_strdup(theme->path);
+	themes_list = g_list_append(themes_list, theme);
+
+	theme_dir = g_build_filename(gkrellm_homedir(), GKRELLM_THEMES_DIR, NULL);
+	add_themes_to_list(theme_dir, FALSE);
 	g_free(theme_dir);
 
+	theme_dir = g_build_filename(gkrellm_homedir(), ".themes", NULL);
+	add_themes_to_list(theme_dir, TRUE);
+	g_free(theme_dir);
+
+	theme_dir = gtk_rc_get_theme_dir();
+	add_themes_to_list(theme_dir, TRUE);
+
 #if defined(LOCAL_THEMES_DIR)
-	add_themes_to_list(&themes_list, LOCAL_THEMES_DIR);
+	add_themes_to_list(LOCAL_THEMES_DIR, FALSE);
 #endif
 #if defined(SYSTEM_THEMES_DIR)
-	add_themes_to_list(&themes_list, SYSTEM_THEMES_DIR);
+	add_themes_to_list(SYSTEM_THEMES_DIR, FALSE);
 #endif
 
-	themes_list = g_list_sort(themes_list, (GCompareFunc) strcmp_basename);
+	themes_list = g_list_sort(themes_list, (GCompareFunc) theme_compare);
 
 	if (_GK.command_line_theme)
-		themes_list = g_list_prepend(themes_list,
-							g_strdup(_GK.command_line_theme));
-
+		{
+		theme = g_new0(Theme, 1);
+		theme->path = g_strdup(_GK.command_line_theme);
+		theme->name = g_strdup(theme->path);
+		themes_list = g_list_append(themes_list, theme);
+		}
 	find_theme_position_in_list();
 	}
 
@@ -1253,24 +1409,17 @@ theme_create_model(void)
 	GtkListStore	*store;
 	GtkTreeIter		iter;
 	GList			*list;
-	gchar			*path, *name;
+	Theme			*theme;
 
 	make_themes_list();
 	store = gtk_list_store_new(N_THEME_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);
 	for (list = themes_list; list; list = list->next)
 		{
-		path = (gchar *) list->data;
-		if ((name = strrchr(path, '/')) == NULL)
-			name = path;
-		else
-			++name;
-		if (!strcmp(name, "Default"))
-			path = "";
-
+		theme = (Theme *) list->data;
 		gtk_list_store_append(store, &iter);
 		gtk_list_store_set(store, &iter,
-				THEME_COLUMN, name,
-				PATH_COLUMN, path,
+				THEME_COLUMN, theme->name,
+				PATH_COLUMN, theme->path,
 				-1);
 		}
 	return GTK_TREE_MODEL(store);
@@ -1443,71 +1592,56 @@ load_theme_config(void)
 				"-adobe-helvetica-medium-r-*-*-*-100-*-*-*-*-iso8859-*");
 	}
 
-  /* Some Gtk themes have a suffix */
-static gchar *gtk_suffix[]	=
-	{
-	"-gtk", "_gtk", "-GTK", "_GTK"
-	};
 
-  /* Called from cb_client_event.  GkrellmTriggered by a Gtk theme switch or if
-  |  an app sends _GKRELLM_READ_THEME event.
-  */
 void
-read_theme_event(gint gkrellm_read_theme)
+gkrellm_read_theme_event(GtkSettings  *settings)
 	{
-	if (gkrellm_read_theme)
-		{
-		g_free(_GK.theme_path);
-		_GK.theme_path = NULL;	/* Force reread of GKRELLM_THEME_CONFIG */
-		}
-	else if (_GK.track_gtk_rcfiles)
-		{
-		FILE	*f;
-		gchar	*gtkrc_path, *s, *g;
-		gchar	buf[512];
-		gint	i;
-		GList	*list;
+	Theme	*theme;
+	gchar	*s, *theme_name = NULL;
+	gint	alt = 0;
 
-		gtkrc_path = g_strdup_printf("%s/.gtkrc", gkrellm_homedir());
-		if ((f = fopen(gtkrc_path, "r")) != NULL)
+	if (settings)	/* called via "notify::gtk-theme-name" signal connect, */
+					/* so get the current gtk theme name and switch to it  */
+		{
+		g_object_get(_GK.gtk_settings, "gtk-theme-name", &theme_name, NULL);
+		if ((_GK.debug_level & DEBUG_GUI) && theme_name)
+			printf("notify::gtk-theme-name: %s\n", theme_name);
+
+		if (   gkrellm_dup_string(&_GK.gtk_theme_name, theme_name)
+			&& _GK.track_gtk_theme_name
+		   )
 			{
-				/* # -- THEME AUTO-WRITTEN DO NOT EDIT 	*/
-			fgets(buf, sizeof(buf), f);
-				/* include "/home/bill/.themes/XX/gtk/gtkrc	*/
-			fgets(buf, sizeof(buf), f);
-			fclose(f);
-			if ((s = strstr(buf, "/gtk/gtkrc")) != NULL)
+			theme = find_theme_in_list(theme_name);
+			if (!theme)
 				{
-				*s = '\0';
-				if ((s = strrchr(buf, (int) '/')) != NULL)
+				theme_name = g_strdup(_GK.default_track_theme);
+				if ((s = strrchr(theme_name, ':')) != NULL)
 					{
-					++s;
-					list = basename_in_list(themes_list, s);
-					if (!list)
-						{
-						for (i = 0; i < sizeof(gtk_suffix) / sizeof(gchar *);
-									++i)
-							if ((g = strstr(s, gtk_suffix[i])) != NULL)
-								{
-								*g = '\0';
-								list = basename_in_list(themes_list, s);
-								break;
-								}
-						}
-					if (list)
-						{
-						s = (gchar *) list->data;
-						gkrellm_dup_string(&_GK.theme_path,
-									strcmp(s, "Default") ? s : "");
-						theme_modified = TRUE;
-						save_theme_config();
-						}
+					*s++ = '\0';
+					alt = atoi(s);
 					}
+				theme = find_theme_in_list(theme_name);
+				g_free(theme_name);
+				}
+			if (   theme && theme->path
+				&& gkrellm_dup_string(&_GK.theme_path,
+							strcmp(theme->path, "Default") ? theme->path : "")
+			   )
+				{
+				_GK.theme_alternative = alt;
+				theme_modified = TRUE;
+				save_theme_config();
+				build_gkrellm();
 				}
 			}
-		g_free(gtkrc_path);
 		}
-	build_gkrellm();
+	else		/* Called from cb_client_event() because we were sent the    */
+				/* _GKRELLM_READ_THEME client event, so reread theme config. */
+		{
+		g_free(_GK.theme_path);
+		_GK.theme_path = NULL;	/* Forces reread of GKRELLM_THEME_CONFIG */
+		build_gkrellm();
+		}
 	}
 
 static void
@@ -1516,7 +1650,7 @@ cb_load_theme(gpointer data, guint action, GtkWidget *widget)
 	GtkTreeIter		iter;
 	GtkTreeModel	*model;
 	GtkTreePath		*path;
-	gchar			*theme_path;
+	Theme			*theme;
 	gint			row;
 
 	++_GK.theme_reload_count;
@@ -1565,9 +1699,9 @@ cb_load_theme(gpointer data, guint action, GtkWidget *widget)
 			gtk_tree_view_set_cursor(theme_treeview, path, NULL, FALSE);
 			return;		/* cb_theme_tree_selection_changed -> build_gkrellm()*/
 			}
-		theme_path = (gchar *) theme_position_in_list->data;
+		theme = (Theme *) theme_position_in_list->data;
 		gkrellm_dup_string(&_GK.theme_path,
-					strcmp(theme_path, "Default") ? theme_path : "");
+					strcmp(theme->path, "Default") ? theme->path : "");
 		theme_modified = TRUE;
 		}
 	if (action == THEME_SCALE_UP && _GK.theme_scale < 380)
@@ -1651,12 +1785,29 @@ gkrellm_get_theme_scale(void)
 	return (gfloat) (_GK.theme_scale) / 100.0;
 	}
 
+static void
+cb_track_gtk(GtkToggleButton *button, GtkWidget *box)
+	{
+	_GK.track_gtk_theme_name = button->active;
+	gtk_widget_set_sensitive(box, _GK.track_gtk_theme_name);
+	}
+
+static void
+cb_track_entry_changed(GtkWidget *widget, gpointer *data)
+	{
+	gchar	*s;
+
+	s = gkrellm_gtk_entry_get_text(&widget);
+	gkrellm_dup_string(&_GK.default_track_theme, s);
+	gkrellm_config_modified();
+	}
+
 static void
 create_theme_tab(GtkWidget *tabs_vbox)
 	{
 	GtkWidget		*tabs;
 	GtkWidget		*vbox, *vbox1, *vbox2, *hbox;
-	GtkWidget		*label;
+	GtkWidget		*label, *entry;
 	GtkWidget		*scrolled;
 	GtkTreeModel	*model;
 	GtkCellRenderer	*renderer;
@@ -1717,8 +1868,35 @@ create_theme_tab(GtkWidget *tabs_vbox)
 	gtk_misc_set_alignment(GTK_MISC(theme_alt_label), 0, 0.5);
 	set_theme_alternatives_label();
 
+
+/* -- Options tab */
+	vbox = gkrellm_gtk_notebook_page(tabs, _("Options"));
+	vbox = gkrellm_gtk_framed_vbox(vbox, NULL, 2, TRUE, 10, 4);
+
+#if !defined(WIN32)
+	vbox1 = gtk_vbox_new(FALSE, 0);
+	gkrellm_gtk_check_button_connected(vbox, &track_gtk_button,
+			_GK.track_gtk_theme_name, FALSE, FALSE, 0,
+			cb_track_gtk, vbox1,
+			_("Track Gtk theme changes for similarly named themes"));
+	gtk_widget_set_sensitive(vbox1, _GK.track_gtk_theme_name);
+	gtk_box_pack_start(GTK_BOX(vbox), vbox1, FALSE, FALSE, 0);
+
+	vbox1 = gkrellm_gtk_category_vbox(vbox1, NULL, 0, 0, TRUE);
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
+	label = gtk_label_new(_("Default"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+	entry = gtk_entry_new();
+	gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+	gtk_entry_set_text(GTK_ENTRY(entry), _GK.default_track_theme);
+	g_signal_connect(G_OBJECT(entry), "changed",
+				G_CALLBACK(cb_track_entry_changed), NULL);
+
+#endif
+
 	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
+	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
 	gkrellm_gtk_spin_button(hbox, NULL,
 			(gfloat)_GK.theme_scale, 40.0, 400.0, 10.0, 20.0, 0, 60,
 			cb_theme_scale_changed, NULL, FALSE, NULL);
@@ -1726,6 +1904,7 @@ create_theme_tab(GtkWidget *tabs_vbox)
 	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 4);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
+
 /* --Fonts tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Fonts"));
 	vbox1 = gkrellm_gtk_framed_vbox(vbox, NULL, 2, FALSE, 0, 2);
