@@ -442,16 +442,18 @@ draw_inet_chart(InetMon *in)
 			if (tm.tm_min == 0)		/* Draw hour mark	*/
 				{
 				pixel0 = gdk_image_get_pixel(grid, cp->x + n, 0);
-				pixel1 = gdk_image_get_pixel(grid, cp->x + n, 1);
-
 				tmp_color.pixel = pixel0;
 				gdk_gc_set_foreground(gc3, &tmp_color);
 				gdk_draw_line(cp->pixmap, gc3,
 						cp->x + n - 1, y0 - 3, cp->x + n - 1, y0 - h4);
-				tmp_color.pixel = pixel1;
-				gdk_gc_set_foreground(gc3, &tmp_color);
-				gdk_draw_line(cp->pixmap, gc3,
-						cp->x + n, y0 - 3, cp->x + n, y0 - h4);
+				if (h > 1)
+					{
+					pixel1 = gdk_image_get_pixel(grid, cp->x + n, 1);
+					tmp_color.pixel = pixel1;
+					gdk_gc_set_foreground(gc3, &tmp_color);
+					gdk_draw_line(cp->pixmap, gc3,
+							cp->x + n, y0 - 3, cp->x + n, y0 - h4);
+					}
 				}
 			if (--tm.tm_min < 0)
 				tm.tm_min = 60;		/* extra minute for skipped slot */
