@@ -222,6 +222,32 @@ gkrellm_draw_decal_text(GkrellmPanel *p, GkrellmDecal *d, gchar *s, gint value)
 	d->value = value;
 	}
 
+void
+gkrellm_decal_text_clear(GkrellmDecal *d)
+	{
+	gdk_draw_rectangle(d->stencil, _GK.bit0_GC, TRUE, 0, 0, d->w, d->h);
+	}
+
+void
+gkrellm_decal_text_insert(GkrellmDecal *d, gchar *s, GkrellmTextstyle *ts,
+		gint x_off, gint y_off)
+	{
+	gint				x, y;
+
+	if (!s || !d || d->state == DS_INVISIBLE)
+		return;
+	if (!ts)
+		ts = &d->text_style;
+	x = x_off;
+	y = d->y_baseline + y_off;
+	gkrellm_draw_string(d->pixmap, ts, x, y, s);
+	gkrellm_gdk_draw_string(d->stencil, ts->font, _GK.bit1_GC, x, y, s);
+	if (ts->effect)
+		gkrellm_gdk_draw_string(d->stencil, ts->font, _GK.bit1_GC,
+					x + 1, y + 1, s);
+	d->modified = TRUE;
+	}
+
 void
 gkrellm_move_krell_yoff(GkrellmPanel *p, GkrellmKrell *k, gint y)
 	{
@@ -258,7 +284,7 @@ gkrellm_remove_krell(GkrellmPanel *p, GkrellmKrell *k)
 	{
 	GkrellmDrawrec	*dr;
 
-	if (!p || !k || !g_list_find(p->krell_list, k))
+	if (!k || !k->panel || !p || !g_list_find(p->krell_list, k))
 		return;
 	p->krell_list = g_list_remove(p->krell_list, k);
 	k->panel = NULL;
@@ -278,7 +304,7 @@ gkrellm_remove_krell(GkrellmPanel *p, GkrellmKrell *k)
 void
 gkrellm_insert_krell(GkrellmPanel *p, GkrellmKrell *k, gboolean append)
 	{
-	if (!p || !k || g_list_find(p->krell_list, k))
+	if (!k || k->panel || !p || g_list_find(p->krell_list, k))
 		return;
 	if (append)
 		p->krell_list = g_list_append(p->krell_list, k);
@@ -668,18 +694,16 @@ gkrellm_create_decal_pixmap(GkrellmPanel *p,
 	return d;
 	}
 
-GkrellmDecal *
-gkrellm_create_decal_text(GkrellmPanel *p, gchar *string,
+static GkrellmDecal *
+_create_decal_text(GkrellmPanel *p, gchar *string,
 				GkrellmTextstyle *ts, GkrellmStyle *style,
-				gint x, gint y, gint w)
+				gint x, gint y, gint w, gint h, gint y_base)
 	{
 	GtkWidget		*top_win;
 	GkrellmMargin	*m;
 	GkrellmDecal	*d;
 	gint			ll, rr, ww, aa, dd;
 
-	if (!ts || !string)
-		return NULL;
 	top_win = gkrellm_get_top_window();
 	d = (GkrellmDecal *) g_new0(GkrellmDecal, 1);
 	if (p)
@@ -687,16 +711,26 @@ gkrellm_create_decal_text(GkrellmPanel *p, gchar *string,
 	d->panel = (gpointer) p;
 
 	d->text_style = *ts;
-	gkrellm_winop_text_extents(d->text_style.font, string, strlen(string),
+
+	if (string && *string)
+		{
+		gkrellm_winop_text_extents(d->text_style.font, string, strlen(string),
 				&ll, &rr, &ww, &aa, &dd);
+		d->h = aa + dd + d->text_style.effect;
+		d->y_baseline = aa;
+		}
+	else
+		{
+		ww = 2;
+		d->h = h;
+		d->y_baseline = y_base;
+		}
+	if (d->h <= 0)
+		d->h = 2;
 
-	d->y_baseline = aa;
-
-	/* Setup size and position defaults.  Height is a function of the
-	|  reference string passed in, but width defaults to full chart width
+	/* Width defaults to full chart width
 	|  minus borders unless w > 0, or w == 0 to use string width.
 	*/
-	d->h = aa + dd + d->text_style.effect;
 	if (style)
 		{
 		m = gkrellm_get_style_margins(style);
@@ -737,6 +771,26 @@ gkrellm_create_decal_text(GkrellmPanel *p, gchar *string,
 	return d;
 	}
 
+GkrellmDecal *
+gkrellm_create_decal_text(GkrellmPanel *p, gchar *string,
+				GkrellmTextstyle *ts, GkrellmStyle *style,
+				gint x, gint y, gint w)
+	{
+	if (!ts || !string)
+		return NULL;
+	return _create_decal_text(p, string, ts, style, x, y, w, 0, 0);
+	}
+
+GkrellmDecal *
+gkrellm_create_decal_text_with_height(GkrellmPanel *p,
+				GkrellmTextstyle *ts, GkrellmStyle *style,
+				gint x, gint y, gint w, gint h, gint y_baseline)
+	{
+	if (!ts)
+		return NULL;
+	return _create_decal_text(p, NULL, ts, style, x, y, w, h, y_baseline);
+	}
+
 void
 gkrellm_make_decal_invisible(GkrellmPanel *p, GkrellmDecal *d)
 	{
