@@ -271,6 +271,7 @@ set_or_save_position(gint save)
 			x = y = 0;
 			fscanf(f, "%d %d", &x, &y);
 			fclose(f);
+
 			if (   x >= 0 && x < _GK.w_display - 10
 				&& y >= 0 && y < _GK.h_display - 25
 			   )
@@ -509,14 +510,16 @@ x_label_position(gint position, gint w_field, gint w_label, gint margin)
 	}
 
 /* ---------------------------------------------------------------------- */
+#define	RESISTANCE_PIXELS	35
 
 static gint		moving_gkrellm	= FALSE;
 static gint		x_press_event, y_press_event;
+static gint		gkrellm_width, gkrellm_height;
 
 void
 gkrellm_motion(GtkWidget *widget, GdkEventMotion *ev, gpointer data)
 	{
-	gint			x_pointer, y_pointer;
+	gint			x_pointer, y_pointer, x, y, right_zone, bottom_zone;
 	GdkModifierType	m;
 
 	if (moving_gkrellm)
@@ -532,12 +535,32 @@ gkrellm_motion(GtkWidget *widget, GdkEventMotion *ev, gpointer data)
 		gkrellm_winop_flush_motion_events();
 		gdk_window_get_pointer(NULL, &x_pointer, &y_pointer, &m);
 
-		/* Moves to x,y position relative to root.  Subtract the press
-		|  event coordinates to account for pointer offset into top_window.
+		/* Subtract press event coordinates to account for pointer offset
+		|  into top_window.  Have edge resistance to the move.
 		*/
-		_GK.y_position = y_pointer - y_press_event;
-		gdk_window_move(top_window->window,
-					x_pointer - x_press_event, _GK.y_position);
+		x = x_pointer - x_press_event;
+		y = y_pointer - y_press_event;
+		right_zone = _GK.w_display - gkrellm_width;
+		bottom_zone = _GK.h_display - gkrellm_height;
+
+		if (_GK.x_position >= 0 && x < 0 && x > -RESISTANCE_PIXELS)
+			x = 0;
+		if (   _GK.x_position <= right_zone
+			&& x > right_zone && x < right_zone + RESISTANCE_PIXELS
+		   )
+			x = right_zone;
+		if (_GK.y_position >= 0 && y < 0 && y > -RESISTANCE_PIXELS)
+			y = 0;
+		if (   _GK.y_position <= bottom_zone
+			&& y > bottom_zone && y < bottom_zone + RESISTANCE_PIXELS
+		   )
+			y = bottom_zone;
+
+		/* Moves to x,y position relative to root.
+		*/
+		gdk_window_move(top_window->window, x, y);
+		_GK.y_position = y;
+		_GK.x_position = x;
 		}
 	}
 
@@ -557,6 +580,11 @@ top_frame_button_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 	time_t			time_check;
 
 	gdk_window_get_pointer(NULL, &x_pointer, &y_pointer, &m);
+	gdk_window_get_geometry(NULL, NULL, NULL, &_GK.w_display,
+				&_GK.h_display, NULL);
+	gkrellm_width = _GK.chart_width
+				+ _GK.frame_left_width + _GK.frame_right_width;
+	gkrellm_height = _GK.monitor_height + _GK.total_frame_height;
 
 	/* If system time is changed, gtk_timeout_add() setting gets
 	|  confused, so put some duct tape here ...
@@ -1470,18 +1498,13 @@ create_widget_tree()
 static gint
 cb_client_event(GtkWidget  *widget, GdkEventClient *event, gpointer data)
 	{
-	static GdkAtom	atom_gtk_read_rcfiles = GDK_NONE;
 	static GdkAtom	atom_gkrellm_read_theme = GDK_NONE;
 
-	if (!atom_gtk_read_rcfiles)
-		{
-		atom_gtk_read_rcfiles = gdk_atom_intern("_GTK_READ_RCFILES", FALSE);
+	if (!atom_gkrellm_read_theme)
 		atom_gkrellm_read_theme = gdk_atom_intern("_GKRELLM_READ_THEME",FALSE);
-		}
-	if (event->message_type == atom_gtk_read_rcfiles) 
-		read_theme_event(0);
-	else if (event->message_type == atom_gkrellm_read_theme) 
-		read_theme_event(1);
+
+	if (event->message_type == atom_gkrellm_read_theme) 
+		gkrellm_read_theme_event(NULL);
 	return FALSE;
 	}
 
@@ -1524,6 +1547,8 @@ cb_configure_notify(GtkWidget *widget, GdkEventConfigure *ev, gpointer data)
 	gint        x, y, w, h, w_gkrellm, h_gkrellm;
 	static gint	x_prev, y_prev, w_prev, h_prev;
 
+	gdk_window_get_geometry(NULL, NULL, NULL, &_GK.w_display,
+				&_GK.h_display, NULL);
 	gdk_window_get_position(widget->window, &x, &y);
 	gdk_drawable_get_size(gtree.window->window, &w, &h);
 
@@ -1611,6 +1636,12 @@ check_gkrellm_directories(void)
 							GKRELLM_THEME_CONFIG);
 		u = gkrellm_make_config_file_name(gkrellm_homedir(),
 							GKRELLM_USER_CONFIG);
+		if (!g_file_test(u, G_FILE_TEST_IS_REGULAR))
+			{
+			g_free(u);
+			u = gkrellm_make_config_file_name(gkrellm_homedir(),
+							GKRELLM_2_1_14_CONFIG);
+			}
 		if (   !g_file_test(u, G_FILE_TEST_IS_REGULAR)
 			&& !g_file_test(t, G_FILE_TEST_IS_REGULAR)
 		   )
@@ -1956,7 +1987,7 @@ main(gint argc, gchar **argv)
 			_GK.test += 1;
 		else if (!strcmp(s, "version") || !strcmp(s, "v"))
 			{
-			printf("%s %d.%d.%d %s\n", PACKAGE, GKRELLM_VERSION_MAJOR,
+			printf("%s %d.%d.%d%s\n", PACKAGE, GKRELLM_VERSION_MAJOR,
 					GKRELLM_VERSION_MINOR, GKRELLM_VERSION_REV,
 					GKRELLM_EXTRAVERSION);
 			exit(0);
@@ -1982,6 +2013,7 @@ main(gint argc, gchar **argv)
 	if (_GK.server && !gkrellm_client_mode_connect())
 		exit(0);
 
+	check_gkrellm_directories();
 	load_user_config(FALSE);		
 	decorated = (_GK.command_line_decorated || _GK.decorated);
 	if (   _GK.command_line_plugin || _GK.command_line_theme
@@ -1993,13 +2025,20 @@ main(gint argc, gchar **argv)
 	create_widget_tree();
 	top_window = gtree.window;
 
-	check_gkrellm_directories();
 	load_builtin_monitors();
 	load_plugin_monitors();
 
 	build_gkrellm();
 	make_themes_list();
 
+	if ((_GK.gtk_settings = gtk_settings_get_default()) != NULL)
+		{
+		g_object_get(_GK.gtk_settings,
+					"gtk-theme-name", &_GK.gtk_theme_name, NULL);
+		g_signal_connect(_GK.gtk_settings, "notify::gtk-theme-name",
+					G_CALLBACK(gkrellm_read_theme_event), NULL);
+		}
+
 	g_signal_connect(G_OBJECT(gtree.window), "delete_event",
 				G_CALLBACK(cb_delete_event), NULL);
 	g_signal_connect(G_OBJECT(gtree.window), "destroy",
