@@ -511,7 +511,8 @@ draw_timer(GkrellmPanel *p, gint seconds, gint force)
 	prev_minute = minutes;
 	prev_hour = hours;
 
-	gkrellm_draw_decal_text(p, time_decal, buf, force ? -1 : minutes);
+	gkrellm_draw_decal_text(p, time_decal, buf,
+				force ?  -1 : (hours * 60 + minutes));
 
 	if (gkrellm_is_decal_visible(seconds_decal))
 		gkrellm_draw_decal_text(p, seconds_decal, buf_sec,
@@ -898,6 +899,9 @@ timed_net_visibility(void)
 		gkrellm_chart_show(net->chart, TRUE);	/* Make sure panel is shown */
 	else if (!net->force_up && !net_is_up && chart_is_visible)
 		gkrellm_chart_hide(net->chart, FALSE);  /* Don't hide the panel */
+
+	if (!net_is_up && chart_is_visible)
+		save_net_data();
 	}
 
 static gint
@@ -1829,6 +1833,12 @@ update_net(void)
 			net->up_event = net->down_event = FALSE;
 			}
 		}
+	if (GK.hour_tick)
+		{
+		tm = gkrellm_get_current_time();
+		if ((tm->tm_hour % 6) == 0)
+			save_net_data();
+		}
 	}
 
   /* A timed interface has its chart locked and is forced enabled.
