@@ -785,7 +785,7 @@ install_plugin(gchar *plugin_name)
 		return NULL;
 		}
 	if (!g_module_symbol(module, "gkrellm_init_plugin",
-				(gpointer *) &init_plugin))
+				(gpointer) &init_plugin))
 		{
 		snprintf(buf, sizeof(buf), _("\tError: %s\n"), g_module_error());
 		plugin_log(buf, NULL);
