@@ -390,7 +390,7 @@ static void
 cb_command_process(GkrellmAlert *alert, gchar *src, gchar *buf, gint size,
 			Sensor *sensor)
 	{
-	gchar		c, *s;
+	gchar		c, *s, *fmt;
 	gint		len;
 
 	if (!buf || size < 1)
@@ -409,9 +409,12 @@ cb_command_process(GkrellmAlert *alert, gchar *src, gchar *buf, gint size,
 			else if (c == 's')
 				{
 				if (sensor->type == SENSOR_FAN)
-					len = snprintf(buf, size, "%.0f", sensor->value);
+					fmt = "%.0f";
+				else if (sensor->type == SENSOR_TEMPERATURE)
+					fmt = "%.1f";
 				else
-					len = snprintf(buf, size, "%.2f", sensor->value);
+					fmt = "%.2f";
+				len = snprintf(buf, size, fmt, sensor->value);
 				}
 			else if (c == 'l')
 				len = snprintf(buf, size, "%s", sensor->name_locale);
@@ -631,7 +634,12 @@ static gint		minus_width;		/* If will be drawing neg voltages */
   /* If drawing '-' sign, grub a pixel or two to tighten the layout */
 static gint		pixel_grub;
 
-
+  /* Avoid writing decimal values into the config to avoid possible
+  |  locale changing decimal point breakage (decimal point can be '.' or ',')
+  */
+#define	SENSOR_FLOAT_FACTOR		10000.0
+static gfloat		sensor_float_factor = 1.0,
+					gkrellm_float_factor = 1.0;
 
 static void
 add_sensor_monitor(Sensor *sr, GkrellmPanel **p, GList **smon_list)
@@ -1369,9 +1377,21 @@ init_sensor_monitor(void)
 	}
 
 /* =================================================================== */
-/* Config for sensors monitor	*/
+/* Config for sensors monitor */
 
-#define	SENSOR_CONFIG_KEYWORD	"sensor"
+  /* Don't use the user-config.  Save into sensors-config and only if there
+  |  is a sensor_list.  This preserves configs across a possible sensors
+  |  modules load screw up.
+  */
+  /* 2.1.15 scales sensor factor/offset values by SENSOR_FLOAT_FACTOR to avoid
+  |  writting decimal points in the config.  This is not backwards compatible
+  |  with the pre 2.1.15 sensor_config format hence the config file name
+  |  change to sensor-config.  But sensor_config is forward compatible
+  |  since the float factor defaults to 1.0.
+  */
+#define	SENSOR_CONFIG_KEYWORD		"sensor"
+#define	SENSOR_CONFIG_FILE			"sensor-config"
+#define	SENSOR_2_1_14_CONFIG_FILE	"sensors_config"
 
 static void
 create_sensor_alert(Sensor *s)
@@ -1398,11 +1418,6 @@ create_sensor_alert(Sensor *s)
 	gkrellm_alert_config_connect(s->alert, cb_alert_config, s);
 	}
 
-
-  /* Don't use the user_config.  Save into sensor_config and only if there
-  |  is a sensor_list.  This preserves configs across a possible sensors
-  |  modules load screw up.
-  */
 static void
 save_sensors_config(FILE *f_not_used)
 	{
@@ -1413,23 +1428,29 @@ save_sensors_config(FILE *f_not_used)
 
 	if (!sensor_list)
 		return;
-	sprintf(buf, "%s/%s", GKRELLM_DIR, "sensors_config");
+	sprintf(buf, "%s/%s", GKRELLM_DIR, SENSOR_CONFIG_FILE);
 	config = gkrellm_make_config_file_name(gkrellm_homedir(), buf);
 	f = fopen(config, "w");
 	g_free(config);
 	if (!f)
 		return;
 
+	fprintf(f, "%s sensor_float_factor %.0f\n",
+				SENSOR_CONFIG_KEYWORD, SENSOR_FLOAT_FACTOR);
+	fprintf(f, "%s gkrellm_float_factor %.0f\n",
+				SENSOR_CONFIG_KEYWORD, GKRELLM_FLOAT_FACTOR);
 	for (list = sensor_list; list; list = list->next)
 		{
 		s = (Sensor *) list->data;
 		if (s->name && *(s->name))
 			{
 			snprintf(quoted_name, sizeof(quoted_name), "\"%s\"", s->basename);
-			fprintf(f, "%s \"%s\" %s %.4f %.4f %d %d\n",
+			fprintf(f, "%s \"%s\" %s %.0f %.0f %d %d\n",
 				SENSOR_CONFIG_KEYWORD,
 				s->name, quoted_name,
-				s->factor, s->offset, s->enabled, s->location);
+				s->factor * SENSOR_FLOAT_FACTOR,
+				s->offset * SENSOR_FLOAT_FACTOR,
+				s->enabled, s->location);
 			if (s->alert)
 				gkrellm_save_alertconfig(f, s->alert,
 						SENSOR_CONFIG_KEYWORD, quoted_name);
@@ -1444,7 +1465,6 @@ save_sensors_config(FILE *f_not_used)
 		}
 	fprintf(f, "%s units_fahrenheit %d\n", SENSOR_CONFIG_KEYWORD,
 				units_fahrenheit);
-//	fprintf(f, "%s use_threads %d\n", SENSOR_CONFIG_KEYWORD, use_threads);
 	fprintf(f, "%s volt_display_mode %d\n", SENSOR_CONFIG_KEYWORD,
 				display_mode);
 	fclose(f);
@@ -1461,43 +1481,53 @@ load_sensors_config(gchar *arg)
 				o	= 0.0;
 	gint		e	= 0,
 				location = 0;
+	gfloat		save_factor;
 
-	n = sscanf(arg, "%32s %[^\n]", config, item);
+	n = sscanf(arg, "%31s %[^\n]", config, item);
 	if (n != 2)
 		return;
 	if (_GK.debug_level & DEBUG_SENSORS)
 		printf("load_sensors_config: <%s> <%s>\n", config, item);
 	if (!strcmp(config, "units_fahrenheit"))
 		sscanf(item, "%d", &units_fahrenheit);
-//	else if (!strcmp(config, "use_threads"))
-//		sscanf(item, "%d", &use_threads);
 	else if (!strcmp(config, "volt_display_mode"))
 		sscanf(item, "%d", &display_mode);
+	else if (!strcmp(config, "sensor_float_factor"))
+		sscanf(item, "%f", &sensor_float_factor);
+	else if (!strcmp(config, "gkrellm_float_factor"))
+		sscanf(item, "%f", &gkrellm_float_factor);
 	else if (!strcmp(config, "vref"))
 		{
-		if (   sscanf(item, "\"%64[^\"]\" \"%64[^\"]\"", name, item1) == 2
+		if (   sscanf(item, "\"%63[^\"]\" \"%64[^\"]\"", name, item1) == 2
 			&& (s = lookup_sensor_from_basename(name)) != NULL
 		   )
 			s->vref = lookup_sensor_from_basename(item1);
 		}
 	else if (!strcmp(config, GKRELLM_ALERTCONFIG_KEYWORD))
 		{
-		if (   sscanf(item, "\"%64[^\"]\" %[^\n]", name, item1) == 2
+		if (   sscanf(item, "\"%63[^\"]\" %[^\n]", name, item1) == 2
 			&& (s = lookup_sensor_from_basename(name)) != NULL
 		   )
 			{
+			/* Since config files may be copied around, make sure to use the
+			|  gkrellm float factor in effect when the sensors config was
+			|  created.
+			*/
+			save_factor = _GK.float_factor;
+			_GK.float_factor = gkrellm_float_factor;
 			if (!s->alert)
 				create_sensor_alert(s);
 			gkrellm_load_alertconfig(&s->alert, item1);
+			_GK.float_factor = save_factor;
 			}
 		}
-	else if (   sscanf(arg, "\"%64[^\"]\" \"%[^\"]\" %f %f %d %d",
+	else if (   sscanf(arg, "\"%63[^\"]\" \"%[^\"]\" %f %f %d %d",
 						name, basename, &f, &o, &e, &location) > 1
 			 && (s = map_sensor_label(name, basename)) != NULL
 			)
 		{
-		s->factor = f;
-		s->offset = o;
+		s->factor = f / sensor_float_factor;
+		s->offset = o / sensor_float_factor;
 		s->enabled = e;
 		s->location = (s->type == SENSOR_VOLTAGE) ? 0 : location;
 		if (!using_new_config && s->type != SENSOR_VOLTAGE)
@@ -1513,19 +1543,29 @@ read_sensors_config(void)
 	FILE	*f;
 	Sensor	*sr;
 	GList	*list;
-	gchar	*config, buf[CFG_BUFSIZE];
+	gchar	*config, *config_2_1_14 = NULL;
+	gchar	buf[CFG_BUFSIZE];
 
-	sprintf(buf, "%s/%s", GKRELLM_DIR, "sensors_config");
+	sprintf(buf, "%s/%s", GKRELLM_DIR, SENSOR_CONFIG_FILE);
 	config = gkrellm_make_config_file_name(gkrellm_homedir(), buf);
 	f = fopen(config, "r");
-	g_free(config);
+	if (!f)
+		{
+		sprintf(buf, "%s/%s", GKRELLM_DIR, SENSOR_2_1_14_CONFIG_FILE);
+		config_2_1_14 = gkrellm_make_config_file_name(gkrellm_homedir(), buf);
+		f = fopen(config_2_1_14, "r");
+		}
 	if (f)
 		{
 		using_new_config = TRUE;
 		while (fgets(buf, sizeof(buf), f))
 			load_sensors_config(buf + strlen(SENSOR_CONFIG_KEYWORD) + 1);
 		fclose(f);
+//		if (config_2_1_14)
+//			rename(config_2_1_14, config);
 		}
+	g_free(config);
+	g_free(config_2_1_14);
 
 	/* In case not all sensors are in sensor_config (user edited?)
 	*/
@@ -2150,6 +2190,10 @@ N_("Enter a zero factor and a blank label to restore default values.\n"),
 N_("Drag and drop sensor rows to change the displayed order.\n"),
 "\n",
 N_("Temperature offset values must be in centigrade units.\n"),
+"\n",
+N_("Substitution variables may be used in alert commands.\n"),
+N_("\t$s    current sensor value.\n"),
+N_("\t$l    sensor label.\n"),
 	};
 
 static void
