@@ -29,7 +29,7 @@ char	errbuf[_POSIX2_LINE_MAX];
 
 // extern gboolean force_meminfo_update(void);
 #if defined(__i386__)
-extern void scan_for_sensors();
+static void scan_for_sensors();
 #endif
 
 
@@ -46,10 +46,7 @@ gkrellm_sys_main_init(void)
 		exit(1);
 		}
 #if defined(__i386__)
-#if 0
-	/ XXX: not provided it yet */
 	scan_for_sensors();
-#endif
 #endif
 	if (setuid(getuid()) != 0)
 		{
@@ -1120,10 +1117,17 @@ gkrellm_sys_battery_read_data(void)
 	struct apm_info	info;
 	gboolean	available, on_line, charging;
 	gint		percent, time_left;
+	gint		batt_num = 0;
 
 	if (!first_time_done)
 		{
 		first_time_done = TRUE;
+#ifdef ACPI_SUPPORTS_MULTIPLE_BATTERIES
+		/*
+		 * XXX: Disable getting battery information via ACPI
+		 * to support multiple batteries via APM sim until
+		 * ACPI sysctls support multiple batteries.
+		 */
 		for (i = 0; name[i] != NULL; ++i)
 			{
 			if (gk_sysctlnametomib(name[i], oid[i],
@@ -1132,7 +1136,9 @@ gkrellm_sys_battery_read_data(void)
 			}
 		if (name[i] == NULL)
 			acpi_enabled = TRUE;
+#endif
 		}
+
 	if (acpi_enabled)
 		{
 			for (i = 0; name[i] != NULL; ++i)
@@ -1147,28 +1153,70 @@ gkrellm_sys_battery_read_data(void)
 			charging = (acpi_info[ACPI_BATT_STATE] == ACPI_BATT_STAT_CHARGING);
 			percent = acpi_info[ACPI_BATT_LIFE];
 			time_left = acpi_info[ACPI_BATT_TIME];
-			gkrellm_battery_assign_data(0, available, on_line, charging,
-						percent, time_left);
+			gkrellm_battery_assign_data(
+				GKRELLM_BATTERY_COMPOSITE_ID, available,
+				on_line, charging, percent, time_left);
 			return;
 		}
+
 	if ((f = open(APMDEV, O_RDONLY)) == -1)
 		return;
-	r = ioctl(f, APMIO_GETINFO, &info);
-	close(f);
-	if (r == -1)
+	if ((r = ioctl(f, APMIO_GETINFO, &info)) == -1) {
+		close(f);
 		return;
-	available = (info.ai_batt_stat != L_UNKNOWN);
+	}
+
+	available = (info.ai_batt_stat != L_UNKNOWN ||
+		     info.ai_acline == L_ON_LINE);
 	on_line = (info.ai_acline == L_ON_LINE) ? TRUE : FALSE;
 	charging = (info.ai_batt_stat == L_CHARGING) ? TRUE : FALSE;
 	percent = info.ai_batt_life;
 #if defined(APM_GETCAPABILITIES)
-	time_left = info.ai_batt_time / 60;
+	if (info.ai_batt_time == -1)
+		time_left = -1;
+	else
+		time_left = info.ai_batt_time / 60;
 #else
 	time_left = -1;
 #endif
+	gkrellm_battery_assign_data(GKRELLM_BATTERY_COMPOSITE_ID, available,
+				    on_line, charging, percent, time_left);
 
-	gkrellm_battery_assign_data(0, available, on_line, charging,
-					percent, time_left);
+#if defined(APMIO_GETPWSTATUS)
+	if (info.ai_infoversion >= 1 && info.ai_batteries != (u_int) -1 &&
+	    info.ai_batteries > 1)
+		{
+		gint i;
+		struct apm_pwstatus aps;
+
+		for (i = 0; i < info.ai_batteries; ++i)
+			{
+			bzero(&aps, sizeof(aps));
+			aps.ap_device = PMDV_BATT0 + i;
+			if (ioctl(f, APMIO_GETPWSTATUS, &aps) == -1 ||
+			    (aps.ap_batt_flag != 255 &&
+			     (aps.ap_batt_flag & APM_BATT_NOT_PRESENT)))
+				continue;
+			available = (aps.ap_batt_stat != L_UNKNOWN ||
+				     aps.ap_acline == L_ON_LINE);
+			on_line = (aps.ap_acline == L_ON_LINE) ? TRUE : FALSE;
+			charging = (aps.ap_batt_stat == L_CHARGING) ?
+				TRUE : FALSE;
+			percent = aps.ap_batt_life;
+			if (aps.ap_batt_time == 0 &&
+			    (aps.ap_batt_stat == L_UNKNOWN ||
+			     aps.ap_batt_stat == L_CHARGING))
+				time_left = -1;
+			else
+				time_left = aps.ap_batt_time / 60;
+			gkrellm_battery_assign_data(batt_num++, available,
+						    on_line, charging, percent,
+						    time_left);
+			}
+		}
+#endif
+
+	close(f);
 	}
 
 gboolean
@@ -1348,21 +1396,29 @@ gkrellm_sys_sensors_get_voltage(gchar *path, gint id,
 	return FALSE;
 	}
 
+struct freebsd_sensor {
+	gint type;
+	gchar *basename;
+	gint id;
+	gint iodev;
+	gint inter;
+	gfloat factor;
+	gfloat offset;
+	gchar *default_label;
+};
+
+static GList	*freebsd_sensor_list;
+
 gboolean
 gkrellm_sys_sensors_init(void)
 	{
-	gchar		*chip_dir = SENSORS_DIR;
-#if __FreeBSD_version >= 300000
-	DIR		*dir;
-	struct dirent	*dentry;
-#endif
-	gchar		temp_file[256], base_name[8], label[8];
-	gint		iodev, interface, id;
-	gint		fandiv[3];
-	u_char		byte;
+	gchar		temp_file[256], label[8];
+	gint		interface, id;
 	int		oid_acpi_temp[CTL_MAXNAME + 2];
 	size_t		oid_acpi_temp_len = sizeof(oid_acpi_temp);
 	gboolean	found_sensors = FALSE, found_acpi_sensors = FALSE;
+	GList		*list;
+	struct freebsd_sensor *sensor;
 
 	for (id = 0;;)
 	{
@@ -1382,6 +1438,63 @@ gkrellm_sys_sensors_init(void)
 		found_acpi_sensors = TRUE;
 	}
 
+	if (freebsd_sensor_list)
+		{
+		for (list = freebsd_sensor_list; list; list = list->next)
+			{
+			sensor = (struct freebsd_sensor *)list->data;
+			gkrellm_sensors_add_sensor(sensor->type, SENSORS_DIR,
+					sensor->basename, sensor->id,
+					sensor->iodev, sensor->inter,
+					sensor->factor, sensor->offset,
+					NULL, sensor->default_label);
+			}
+		found_sensors = TRUE;
+		}
+
+	return (found_sensors || found_acpi_sensors);
+	}
+
+static gboolean
+sensors_add_sensor(gint type, gchar *basename, gint id, gint iodev, gint inter,
+		   gfloat factor, gfloat offset, gchar *default_label)
+{
+	struct freebsd_sensor *sensor;
+
+	if ((sensor = g_new0(struct freebsd_sensor, 1)) == NULL)
+		return FALSE;
+	sensor->type = type;
+	sensor->basename = g_strdup(basename);
+	sensor->id = id;
+	sensor->iodev = iodev;
+	sensor->inter = inter;
+	sensor->factor = factor;
+	sensor->offset = offset;
+	sensor->default_label = default_label ? g_strdup(default_label) : NULL;
+	if (g_list_append(freebsd_sensor_list, sensor) == NULL) {
+		g_free(sensor->basename);
+		if (sensor->default_label)
+			g_free(sensor->default_label);
+		g_free(sensor);
+		return FALSE;
+	}
+	return TRUE;
+}
+
+static void
+scan_for_sensors(void)
+	{
+	gchar		*chip_dir = SENSORS_DIR;
+#if __FreeBSD_version >= 300000
+	DIR		*dir;
+	struct dirent	*dentry;
+#endif
+	gchar		temp_file[256], base_name[8];
+	gint		iodev = -1, interface = -1, id;
+	gint		fandiv[3];
+	u_char		byte;
+	gboolean	found_sensors = FALSE;
+
 #if __FreeBSD_version >= 300000
 	if ((dir = opendir(chip_dir)) != NULL)
 		{
@@ -1403,9 +1516,10 @@ gkrellm_sys_sensors_init(void)
 				close(iodev);
 				continue;
 				}
-			gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE, SENSORS_DIR,
-					base_name, 0, iodev, interface, 1.0, 0.0, NULL, NULL);
+			sensors_add_sensor(SENSOR_TEMPERATURE, base_name, 0,
+					iodev, interface, 1.0, 0.0, NULL);
 			found_sensors = TRUE;
+			break;
 			}
 		closedir(dir);
 		}
@@ -1415,16 +1529,16 @@ gkrellm_sys_sensors_init(void)
 		snprintf(temp_file, sizeof(temp_file), "%s/%s",
 			 chip_dir, "io");
 		if ((iodev = open(temp_file, O_RDWR)) == -1)
-			return found_acpi_sensors;
+			return;
 		interface = INTERFACE_IO;
 		if (!gkrellm_sys_sensors_get_temperature(NULL, 0, iodev,
 				interface, NULL))
 			{
 			close(iodev);
-			return found_acpi_sensors;
+			return;
 			}
-		gkrellm_sensors_add_sensor(SENSOR_TEMPERATURE, SENSORS_DIR, "temp0",
-				0, iodev, interface, 1.0, 0.0, NULL, NULL);
+		sensors_add_sensor(SENSOR_TEMPERATURE, "temp0", 0, iodev,
+				interface, 1.0, 0.0, NULL);
 		found_sensors = TRUE;
 		}
 	if (found_sensors)
@@ -1440,9 +1554,10 @@ gkrellm_sys_sensors_init(void)
 						interface, NULL))
 					continue;
 				sprintf(base_name, "%s%d", "fan", id);
-				gkrellm_sensors_add_sensor(SENSOR_FAN, SENSORS_DIR, base_name,
-						id, iodev, interface, 1.0 / (gfloat) fandiv[id],
-						0.0, NULL, NULL);
+				sensors_add_sensor(SENSOR_FAN, base_name, id,
+						iodev, interface,
+						1.0 / (gfloat) fandiv[id], 0.0,
+						NULL);
 				}
 			}
 		for (id = 0; id < 7; ++id)
@@ -1451,12 +1566,12 @@ gkrellm_sys_sensors_init(void)
 					interface, NULL))
 				continue;
 			sprintf(base_name, "%s%d", "in", id);
-			gkrellm_sensors_add_sensor(SENSOR_VOLTAGE, SENSORS_DIR, base_name,
-					id, iodev, interface, voltdefault0[id].factor,
-					0.0, NULL, voltdefault0[id].name);
+			sensors_add_sensor(SENSOR_VOLTAGE, base_name, id,
+					iodev, interface,
+					voltdefault0[id].factor, 0.0,
+					voltdefault0[id].name);
 			}
 		}
-	return (found_sensors | found_acpi_sensors);
 	}
 
 #else
