@@ -24,14 +24,62 @@
 #include <sys/types.h>
 #include <dirent.h>
 
+
 static gboolean		need_locale_fix,
+					have_diskstats,
 					have_partition_stats,
-					have_sysfs_stats;
+					have_sysfs,
+					have_sysfs_stats,
+					have_sysfs_sensors;
 static gchar		locale_decimal_point;
 
+
+static gboolean		kernel_2_4,
+					kernel_2_6;
+
+static gint			os_major,
+					os_minor,
+					os_rev;
+
+static gboolean
+os_release(gint major, gint minor, gint rev)
+	{
+	if (   os_major > major
+		|| (os_major == major && os_minor > minor)
+		|| (os_major == os_major && os_minor == minor && os_rev >= rev)
+	   )
+		return TRUE;
+	return FALSE;
+	}
+
+
 void
 gkrellm_sys_main_init(void)
 	{
+	FILE	*f;
+	gchar	buf[1024];
+
+	if ((f = fopen("/proc/sys/kernel/osrelease", "r")) != NULL)
+		{
+		if (fgets(buf, sizeof(buf), f) != NULL)
+			sscanf(buf, "%d.%d.%d", &os_major, &os_minor, &os_rev);
+		fclose(f);
+		kernel_2_4 = os_release(2, 4, 0);
+		kernel_2_6 = os_release(2, 6, 0);
+		}
+
+	/* Various stats are in sysfs since 2.5.47, but it may not be mounted.
+	*/
+	if ((f = fopen("/proc/mounts", "r")) != NULL)
+		{
+		while (fgets(buf, sizeof(buf), f) != NULL)
+			if (strstr(buf, "sysfs"))
+				{
+				have_sysfs = TRUE;
+				break;
+				}
+		fclose(f);
+		}
 	}
 
 void
@@ -39,6 +87,7 @@ gkrellm_sys_main_cleanup(void)
 	{
 	}
 
+
   /* Linux /proc always reports floats with '.' decimal points, but sscanf()
   |  for some locales needs commas in place of periods.  So, if current
   |  locale doesn't use periods, must insert the correct decimal point char.
@@ -55,8 +104,8 @@ locale_fix(gchar *buf)
 
 
 /* ===================================================================== */
-/* CPU, disk, and swap monitor interfaces all get data from /proc/stat
-|  so they will share reading of /proc/stat.
+/* CPU, disk, and swap monitor interfaces might all get data from /proc/stat
+|  (depending on kernel version) so they will share reading of /proc/stat.
 */
 
 #define	PROC_STAT_FILE	"/proc/stat"
@@ -79,31 +128,28 @@ static gulong	swapin,
 static void
 linux_read_proc_stat(void)
 	{
-	FILE		*f;
+	static FILE	*f;
 	gint		n, i, ncpu;
-	gchar		*item, *arg;
-	gchar		buf[1024];
+	gchar		*s, buf[1024];
 	gulong		rblk[4], wblk[4], user, nice, sys, idle, iowait;
 	static gint	data_read_tick	= -1;
 
-	if (   data_read_tick == gkrellm_get_timer_ticks()	/* One read per tick */
-		|| (f = fopen(PROC_STAT_FILE, "r")) == NULL
-	   )
+	n = gkrellm_get_timer_ticks();
+	if (data_read_tick == n)
+		return;		/* Just one read per tick (multiple monitors call this) */
+
+	if (!f && (f = fopen(PROC_STAT_FILE, "r")) == NULL)
 		return;
-	data_read_tick = gkrellm_get_timer_ticks();
+
+	data_read_tick = n;
 	gkrellm_disk_reset_composite();
 
 	ncpu = 0;
 	while ((fgets(buf, sizeof(buf), f)) != NULL)
 		{
-		item = strtok(buf, " \t\n");
-		arg = strtok(NULL, "\n");
-		if (item == NULL || arg == NULL)
-			continue;
-
-		if (item[0] == 'c' && item[1] == 'p')
+		if (buf[0] == 'c' && buf[1] == 'p')
 			{
-			n = sscanf(arg, "%lu %lu %lu %lu %lu",
+			n = sscanf(buf, "%*s %lu %lu %lu %lu %lu",
 						&user, &nice, &sys, &idle, &iowait);
 
 			if (n == 5)			/* iowait is new in kernel 2.5	*/
@@ -121,35 +167,41 @@ linux_read_proc_stat(void)
 			else if (ncpu == 0)
 				gkrellm_cpu_assign_data(0, user, nice, sys, idle);
 			++ncpu;
+			continue;
 			}
-		else if (strcmp("disk_rblk", item) == 0)	/* Pre kernel 2.4 format */
+		if (!kernel_2_6 && !strncmp("swap", buf, 4))
 			{
+			sscanf(buf + 5, "%lu %lu", &swapin, &swapout);
+			continue;
+			}
+		if (buf[0] != 'd')
+			continue;
+
+		if (!strncmp("disk_rblk", buf, 9))
+			{
+			s = buf + 10;
 			for (i = 0; i < 4; ++i)
-				rblk[i] = strtoul(arg, &arg, 0);
+				rblk[i] = strtoul(s, &s, 0);
 			}
-		else if (strcmp("disk_wblk", item) == 0)	/* Pre kernel 2.4 format */
+		else if (!strncmp("disk_wblk", buf, 9))
 			{
+			s = buf + 10;
 			for (i = 0; i < 4; ++ i)
 				{
-				wblk[i] = strtoul (arg, &arg, 0);
+				wblk[i] = strtoul(s, &s, 0);
 				gkrellm_disk_assign_data_nth(i, 512 * rblk[i], 512 * wblk[i]);
 				}
 			}
-		/* Read swap data for the swap monitor
-		*/
-		else if (strcmp("swap", item) == 0)
-			sscanf(arg, "%lu %lu", &swapin, &swapout);
-
-		else if (strcmp("disk_io:", item) == 0)	/* Kernel 2.4 only format */
+		else if (!strncmp("disk_io:", buf, 8))		/* Kernel 2.4 only	*/
 			{
 			gint	major, i_disk;
 			gulong	rblk, wblk, rb1, rb2, wb1, wb2;
 
-			item = strtok(arg, " \t\n");
-			while (item)
+			s = strtok(buf + 9, " \t\n");
+			while (s)
 				{
 				/* disk_io lines in 2.4.x kernels have had 2 formats */
-				n = sscanf(item, "(%d,%d):(%*d,%lu,%lu,%lu,%lu)",
+				n = sscanf(s, "(%d,%d):(%*d,%lu,%lu,%lu,%lu)",
 						&major, &i_disk, &rb1, &rb2, &wb1, &wb2);
 				if (n == 6)	/* patched as of 2.4.0-test1-ac9 */
 					{		/* (major,disk):(total_io,rio,rblk,wio,wblk) */
@@ -179,12 +231,11 @@ linux_read_proc_stat(void)
 				else
 					gkrellm_disk_assign_data_by_device(major, i_disk,
 								512 * rblk, 512 * wblk);
-				item = strtok(NULL, " \t\n");
+				s = strtok(NULL, " \t\n");
 				}
-			break;
 			}
 		}
-	fclose(f);
+	rewind(f);
 	}
 
 
@@ -230,6 +281,7 @@ gkrellm_sys_cpu_init(void)
 /* Disk monitor interface */
 
 #define	PROC_PARTITIONS_FILE	"/proc/partitions"
+#define	PROC_DISKSTATS_FILE		"/proc/diskstats"
 
 #include <linux/major.h>
 #if ! defined (SCSI_DISK0_MAJOR)
@@ -269,6 +321,9 @@ gkrellm_sys_cpu_init(void)
 #if !defined(LVM_BLK_MAJOR)
 #define LVM_BLK_MAJOR 58
 #endif
+#if !defined(NBD_MAJOR)
+#define NBD_MAJOR 43
+#endif
 
 
 struct _disk_name_map
@@ -297,6 +352,7 @@ static struct _disk_name_map
 	{"sd",	SCSI_DISK0_MAJOR,			16,	'a' },	/* 8:  sda-sdh */
 	{"sg",	SCSI_GENERIC_MAJOR,			0,	'0' },	/* 21: sg0-sg16 */
 	{"scd",	SCSI_CDROM_MAJOR,			0,	'0' },	/* 11: scd0-scd16 */
+	{"sr",	SCSI_CDROM_MAJOR,			0,	'0' },	/* 11: sr0-sr16 */
 	{"md",	MD_MAJOR,					0,	'0' },	/* 9:  md0-md3 */
 
 	{"c0d",	DAC960_MAJOR,				0,	'0' },	/* 48:  c0d0-c0d31 */
@@ -329,6 +385,19 @@ static struct _disk_name_map
 	{"fd",	FLOPPY_MAJOR,				0,	'0' }	/* 2:  fd0-fd3  */
 	};
 
+static gboolean
+disk_major_ok(gint major)
+	{
+	gint		i;
+
+	for (i = 0; i < sizeof(disk_name_map) / sizeof(struct _disk_name_map); ++i)
+		{
+		if (major == disk_name_map[i].major)
+			return TRUE;
+		}
+	return FALSE;
+	}
+
 gchar *
 gkrellm_sys_disk_name_from_device(gint device_number, gint unit_number,
 			gint *order)
@@ -368,9 +437,9 @@ gkrellm_sys_disk_order_from_name(gchar *name)
 		{
 		dm = &disk_name_map[i];
 		len = strlen(dm->name);
-		if (strncmp(dm->name, name, strlen(dm->name)))
+		if (strncmp(dm->name, name, len))
 			continue;
-		suffix = name[len];
+		suffix = name[len];			/* So far looked at only for "hd" series */
 		if (i < table_size - 1)
 			{
 			dm_next = &disk_name_map[i + 1];
@@ -386,14 +455,14 @@ gkrellm_sys_disk_order_from_name(gchar *name)
 	return i;
 	}
 
-  /* Given a /proc/partitions line disk name in "partition", make "disk" have
-  |  the whole disk name (eg hda) and "partition" have the partition (eg hda1)
-  |  or NULL if not a partition.  For simple names, "disk" is expected to
-  |  initially have the whole disk name from the previous call (or NULL if
-  |  this is the first call per /proc/partitions parse).
+  /* Given a /proc/partitions or /proc/diskstats line disk name in "partition",
+  |  make "disk" have the whole disk name (eg hda) and "partition" have the
+  |  partition (eg hda1) or NULL if not a partition.  For simple names,
+  |  "disk" is expected to initially have the whole disk name from the
+  |  previous call (or NULL if this is the first call per /proc file parse).
   */
 static gboolean
-get_proc_partition_name(gint major, gint minor, gchar *disk, gchar *partition)
+disk_get_device_name(gint major, gint minor, gchar *disk, gchar *partition)
 	{
 	struct _disk_name_map	*dm	= NULL;
 	gint					i, unit = 0;
@@ -403,7 +472,7 @@ get_proc_partition_name(gint major, gint minor, gchar *disk, gchar *partition)
 		if (*p == '/')
 			break;
 	if (!*p)
-		{	/* Have a simple /proc/partition name like hda, hda1, sda, ... */
+		{	/* Have a simple name like hda, hda1, sda, ... */
 		d = disk;
 		p = partition;
 		while (*d && *p && *d++ == *p++)
@@ -439,25 +508,86 @@ get_proc_partition_name(gint major, gint minor, gchar *disk, gchar *partition)
 	return FALSE;
 	}
 
-  /* see linux/drivers/block/genhd.c
+  /* Kernels >= 2.5.69 have /proc/diskstats which can be more efficient to
+  |  read than getting stats from sysfs.  See:
+  |      /usr/src/linux/Documentation/iostats.txt
+  |  But gkrellm calls this only for 2.6+ kernels since there were some
+  |  format incompatible /proc/diskstats patches for 2.4.
   */
-void
-gkrellm_sys_disk_read_data(void)
+static void
+linux_read_proc_diskstats(void)
+	{
+	static FILE		*f;
+	gchar			buf[1024], part[64], disk[64];
+	gint			major, minor, n;
+	gulong			rd, wr, rd1, wr1;
+	gboolean		inactivity_override;
+	static gboolean	initial_read = TRUE;
+
+	if (!f && (f = fopen(PROC_DISKSTATS_FILE, "r")) == NULL)
+		return;
+
+	disk[0] = '\0';
+	part[0] = '\0';
+		
+	while ((fgets(buf, sizeof(buf), f)) != NULL)
+		{
+		/* major minor name rio rmerge rsect ruse wio wmerge wsect wuse
+		|               running use aveq
+		|  --or for a partition--
+		|  major minor name rio rsect wio wsect
+		*/
+		n = sscanf(buf, "%d %d %64s %*d %lu %lu %lu %*d %*d %lu",
+					&major, &minor, part, &rd, &rd1, &wr, &wr1);
+		if (n == 7)
+			{
+			rd = rd1;
+			wr = wr1;
+			}
+
+		/* Make sure all real disks get reported (so they will be added to the
+		|  disk monitor in order) the first time this function is called.
+		|  Use disk_major_ok() instead of simply initial_read until I'm sure
+		|  I'm testing for all the right "major" exclusions.
+		|  Note: disk_get_device_name() assumes "part[]" retains results from
+		|  previous calls and that disk/subdisk parsing will be in order
+		|  (ie hda will be encountered before hda1).
+		*/
+		inactivity_override = initial_read ? disk_major_ok(major) : FALSE;
+
+		if (   (n != 7 && n != 6)
+			|| (rd == 0 && wr == 0 && !inactivity_override)
+			|| major == LVM_BLK_MAJOR || major == NBD_MAJOR
+			|| major == RAMDISK_MAJOR || major == LOOP_MAJOR
+			|| !disk_get_device_name(major, minor, disk, part)
+		   )
+			continue;
+		if (part[0] == '\0')
+			gkrellm_disk_assign_data_by_name(disk, 512 * rd, 512 * wr);
+		else
+			gkrellm_disk_subdisk_assign_data_by_name(part, disk,
+						512 * rd, 512 * wr);
+		}
+	rewind(f);
+	initial_read = FALSE;
+	}
+
+  /* /proc/partitions can have diskstats in 2.4 kernels or in 2.5+ it's just
+  |  a list of disk names which can be used to get names to look for in sysfs.
+  |  But, with gkrellm 2.1.15 and for 2.6+kernels /proc/diskstats is
+  |  used instead of sysfs.
+  */
+static void
+linux_read_proc_partitions_or_sysfs(void)
 	{
 	FILE		*f, *sf;
 	gchar		buf[1024], part[64], disk[64], sysfspath[128];
 	gint		major, minor, n;
 	gulong		sectors, rd, wr;
 
-	if (!have_sysfs_stats)
-		linux_read_proc_stat();		/* handles floppys and CDroms */
-
-	if (!have_partition_stats && !have_sysfs_stats)
-		return;
-
 	if ((f = fopen(PROC_PARTITIONS_FILE, "r")) != NULL)
 		{
-		fgets(buf, sizeof(buf), f); /* nuke header */
+		fgets(buf, sizeof(buf), f); /* header */
 		fgets(buf, sizeof(buf), f); 
 		disk[0] = '\0';
 		part[0] = '\0';
@@ -472,7 +602,7 @@ gkrellm_sys_disk_read_data(void)
 				n = sscanf(buf, "%d %d %lu %64s %*d %*d %lu %*d %*d %*d %lu",
 							&major, &minor, &sectors, part, &rd, &wr);
 				if (   n < 6 || sectors <= 1 || major == LVM_BLK_MAJOR
-					|| !get_proc_partition_name(major, minor, disk, part)
+					|| !disk_get_device_name(major, minor, disk, part)
 				   )
 					continue;
 				}
@@ -481,7 +611,7 @@ gkrellm_sys_disk_read_data(void)
 				n = sscanf(buf, "%d %d %lu %64s",
 							&major, &minor, &sectors, part);
 				if (   n < 4 || sectors <= 1 || major == LVM_BLK_MAJOR
-					|| !get_proc_partition_name(major, minor, disk, part)
+					|| !disk_get_device_name(major, minor, disk, part)
 				   )
 					continue;
 				if (part[0] == '\0')
@@ -506,18 +636,36 @@ gkrellm_sys_disk_read_data(void)
 				gkrellm_disk_subdisk_assign_data_by_name(part, disk,
 							512 * rd, 512 * wr);
 			}
-			fclose(f);
+		fclose(f);
 		}
-		return;
+	}
+
+void
+gkrellm_sys_disk_read_data(void)
+	{
+	/* If have_partition_stats, still need to get floppy and CDrom data
+	|  from /proc/stat
+	*/
+	if (!have_sysfs_stats && !have_diskstats)
+		linux_read_proc_stat();	
+
+	if (have_diskstats)
+		linux_read_proc_diskstats();
+	else if (have_partition_stats || have_sysfs_stats)
+		linux_read_proc_partitions_or_sysfs();
 	}
 
 gboolean
 gkrellm_sys_disk_init(void)
 	{
-	FILE	*f;
+	FILE	*f = NULL;
 	gchar	buf[1024];
 
-	if ((f = fopen(PROC_PARTITIONS_FILE, "r")) != NULL)
+	/* There were some incompatible /proc/diskstats patches for 2.4
+	*/
+	if (os_release(2,6,0) && (f = fopen(PROC_DISKSTATS_FILE, "r")) != NULL)
+		have_diskstats = TRUE;
+	else if ((f = fopen(PROC_PARTITIONS_FILE, "r")) != NULL)
 		{
 		if (fgets(buf, sizeof(buf), f))
 			{
@@ -525,20 +673,17 @@ gkrellm_sys_disk_init(void)
 				have_partition_stats = TRUE;
 			else 
 				{
-				/* stats are in sysfs since 2.5.47, but it may not be mounted
-				*/
-				fclose(f);
-				if ((f = fopen("/proc/mounts", "r")) != NULL)
-					while ((fgets(buf, sizeof(buf), f)) != NULL)
-						if (strstr(buf, "sysfs"))
-							{
-							have_sysfs_stats = TRUE;
-							break;
-							}
+				if (have_sysfs)
+					have_sysfs_stats = TRUE;
 				}
 			}
-		fclose(f);
 		}
+	if (f)
+		fclose(f);
+	if (_GK.debug_level & DEBUG_SYSDEP)
+		printf("diskstats=%d partition_stats=%d sysfs_stats=%d\n",
+			have_diskstats, have_partition_stats, have_sysfs_stats);
+
 	return TRUE;
 	}
 
@@ -630,7 +775,7 @@ gkrellm_sys_inet_read_tcp_data(void)
 
 	if ((f = fopen(PROC_NET_TCP_FILE, "r")) != NULL)
 		{
-		fgets(buf, sizeof(buf), f);		/* Waste first line */
+		fgets(buf, sizeof(buf), f);		/* header */
 
 		while (fgets(buf, sizeof(buf), f))
 			{
@@ -699,17 +844,17 @@ static gint			rx_bytes_index,
 void
 gkrellm_sys_net_check_routes(void)
 	{
-	FILE		*f;
-	GList		*list;
-	NetUp		*net;
-	gchar		*s;
-	gchar		buf[512];
+	static FILE		*f;
+	GList			*list;
+	NetUp			*net;
+	gchar			*s;
+	gchar			buf[512];
 
 
 	for (list = net_routed_list; list; list = list->next)
 		((NetUp *) list->data)->cur_up = FALSE;
 
-	if ((f = fopen(PROC_NET_ROUTE_FILE, "r")) != NULL)
+	if (f || (f = fopen(PROC_NET_ROUTE_FILE, "r")) != NULL)
 		{
 		fgets(buf, sizeof(buf), f);		/* Waste the first line */
 		while (fgets(buf, sizeof(buf), f))
@@ -736,7 +881,7 @@ gkrellm_sys_net_check_routes(void)
 				net->cur_up = TRUE;
 				}
 			}
-		fclose(f);
+		rewind(f);
 		}
 	for (list = net_routed_list; list; list = list->next)
 		{
@@ -755,20 +900,18 @@ gkrellm_sys_net_check_routes(void)
 void
 gkrellm_sys_net_read_data(void)
 	{
-	FILE	*f;
-	gchar	buf[512];
-	gchar	*name, *s, *s1;
-	gint	i;
-	gulong	rx, tx;
-	gulong	rx_packets	= 0,
-			tx_packets	= 0;
-#if defined(G_HAVE_GINT64)
-	guint64	ll;
-#endif
+	static FILE	*f;
+	gchar		buf[512];
+	gchar		*name, *s, *s1;
+	gint		i;
+	gulong		rx, tx;
+	gulong		rx_packets	= 0,
+				tx_packets	= 0;
+	guint64		ll;
 
-	if ((f = fopen(PROC_NET_DEV_FILE, "r")) == NULL)
+	if (!f && (f = fopen(PROC_NET_DEV_FILE, "r")) == NULL)
 		return;
-	fgets(buf, sizeof(buf), f);		/* Waste first 2 lines */
+	fgets(buf, sizeof(buf), f);		/* 2 line header */
 	fgets(buf, sizeof(buf), f);
 	while (fgets(buf, sizeof(buf), f))
 		{
@@ -789,6 +932,7 @@ gkrellm_sys_net_read_data(void)
 		if ((name = strtok(buf, " \t\n")) == NULL)	/* Get name of interface */
 			{
 			fclose(f);
+			f = NULL;
 			return;
 			}
 		if (!strncmp(name, "dummy", 5))
@@ -799,7 +943,7 @@ gkrellm_sys_net_read_data(void)
 			if (i == rx_bytes_index)
 				{
 				rx = strtoul(s, NULL, 0);
-#if defined(G_HAVE_GINT64)
+
 				/* Can have 32 bit library / 64 bit kernel mismatch.
 				|  If so, just using the 32 low bits of a long long is OK.
 				*/
@@ -807,17 +951,14 @@ gkrellm_sys_net_read_data(void)
 					&& sscanf(s, "%Ld", &ll) == 1
 				   )
 					rx = (gulong) ll;
-#endif
 				}
 			else if (i == tx_bytes_index)
 				{
 				tx = strtoul(s, NULL, 0);
-#if defined(G_HAVE_GINT64)
 				if (   tx == ULONG_MAX && errno == ERANGE
 					&& sscanf(s, "%Ld", &ll) == 1
 				   )
 					tx = (gulong) ll;
-#endif
 				}
 			else if (i == rx_packets_index)
 				rx_packets = strtoul(s, NULL, 0);
@@ -833,7 +974,7 @@ gkrellm_sys_net_read_data(void)
 			}
 		gkrellm_net_assign_data(name, rx, tx);
 		}
-	fclose(f);
+	rewind(f);
 	}
 
 gboolean
@@ -942,6 +1083,7 @@ gkrellm_sys_net_init(void)
 /* Memory/Swap monitor interface */
 
 #define	PROC_MEMINFO_FILE	"/proc/meminfo"
+#define	PROC_VMSTAT_FILE	"/proc/vmstat"
 
 static guint64	swap_total, swap_used;
 
@@ -950,7 +1092,7 @@ static guint64	swap_total, swap_used;
 static void
 tagged_format_meminfo(gchar *buf, guint64 *mint)
 	{
-	sscanf(buf,"%*s %Lu", mint);
+	sscanf(buf,"%Lu", mint);
 	*mint *= 1024;
 	}
 
@@ -969,35 +1111,35 @@ gkrellm_sys_mem_read_data(void)
 		if (buf[0] == 'M')
 			{
 			if (!strncmp(buf, "Mem:", 4))
-				sscanf(buf,"Mem: %Lu %Lu %Lu %Lu %Lu %Lu",
+				sscanf(buf + 5, "%Lu %Lu %Lu %Lu %Lu %Lu",
 						&total, &x_used, &free,
 						&shared, &buffers, &cached);
 			else if (!strncmp(buf, "MemTotal:", 9))
 				{
-				tagged_format_meminfo(buf, &total);
+				tagged_format_meminfo(buf + 10, &total);
 				using_tagged = TRUE;
 				}
 			else if (!strncmp(buf, "MemFree:", 8))
-				tagged_format_meminfo(buf, &free);
+				tagged_format_meminfo(buf + 9, &free);
 			else if (!strncmp(buf, "MemShared:", 10))
-				tagged_format_meminfo(buf, &shared);
+				tagged_format_meminfo(buf + 11, &shared);
 			}
 		else if (buf[0] == 'S')
 			{
 			if (!strncmp(buf, "Swap:", 5))
 				{
-				sscanf(buf,"Swap: %Lu %Lu", &swap_total, &swap_used);
+				sscanf(buf + 6,"%Lu %Lu", &swap_total, &swap_used);
 				break;
 				}
 			else if (!strncmp(buf, "SwapTotal:", 10))
-				tagged_format_meminfo(buf, &swap_total);
+				tagged_format_meminfo(buf + 11, &swap_total);
 			else if (!strncmp(buf, "SwapFree:", 9))
-				tagged_format_meminfo(buf, &swap_used);
+				tagged_format_meminfo(buf + 10, &swap_used);
 			}
 		else if (buf[0] == 'B' && !strncmp(buf, "Buffers:", 8))
-			tagged_format_meminfo(buf, &buffers);
+			tagged_format_meminfo(buf + 9, &buffers);
 		else if (buf[0] == 'C' && !strncmp(buf, "Cached:", 7))
-			tagged_format_meminfo(buf, &cached);
+			tagged_format_meminfo(buf + 8, &cached);
 		}
 	fclose(f);
 	if (using_tagged)
@@ -1009,13 +1151,32 @@ gkrellm_sys_mem_read_data(void)
 	gkrellm_mem_assign_data(total, used, free, shared, buffers, cached);
 	}
 
+  /* Kernel >= 2.6 swap page in/out is in /proc/vmstat.
+  |  Kernel <= 2.4 swap page in/out is in /proc/stat read in read_proc_stat().
+  |  Swap total/used for all kernels is read in mem_read_data() above.
+  */
 void
 gkrellm_sys_swap_read_data(void)
 	{
-	/* swap page in/out data is in /proc/stat and this is read in
-	|  read_proc_stat() where page_in/page_out are assigned.  Swap
-	|  total/used is assigned above in mem_read_data.
-	*/
+	static FILE		*f;
+	gchar			buf[128];
+
+	if (!f && kernel_2_6)
+		f = fopen(PROC_VMSTAT_FILE, "r");
+
+	if (f)
+		{
+		while (fgets(buf, sizeof(buf), f) != NULL)
+			{
+			if (buf[0] != 'p' || buf[1] != 's')
+				continue;
+			sscanf(buf, "pswpin %lu", &swapin);
+			if (fgets(buf, sizeof(buf), f) == NULL)
+				break;
+			sscanf(buf, "pswpout %lu", &swapout);
+			}
+		rewind(f);
+		}
 	gkrellm_swap_assign_data(swap_total, swap_used, swapin, swapout);
 	}
 
@@ -1121,6 +1282,7 @@ gkrellm_sys_fs_get_fstab_list(void)
 			|| !strcmp(type, "proc")
 			|| !strcmp(type, "sysfs")
 			|| !strcmp(type, "usbdevfs")
+			|| !strcmp(type, "usbfs")
 			|| !strcmp(type, "ignore")
 		   )
 			continue;
@@ -1162,6 +1324,7 @@ gkrellm_sys_fs_get_mounts_list(void)
 		if (   !strcmp(type, "devpts")
 			|| !strcmp(type, "proc")
 			|| !strcmp(type, "usbdevfs")
+			|| !strcmp(type, "usbfs")
 			|| !strcmp(type, "sysfs")
 		   )
 			continue;
@@ -1570,7 +1733,7 @@ apm_battery_data(void)
 	gint		percent = 0, time_left = 0;
 	gint		ac_line_status, battery_status, battery_flag;
 	gint		bios_major, bios_minor;
-	gint		id, n_batteries;
+	gint		id, n_batteries = 1;
 
 	if ((f = fopen(PROC_APM_FILE, "r")) == NULL)
 		return FALSE;
@@ -1581,12 +1744,19 @@ apm_battery_data(void)
 				&ac_line_status, &battery_status, &battery_flag,
 				&percent, &time_left, units);
 
-	if (   fgets(buf, sizeof(buf), f)
-		&& sscanf(buf, "%d\n", &n_batteries) == 1
-	   )
+	/* If have APM dual battery patch, next line will be number of batteries.
+	*/
+	if (fgets(buf, sizeof(buf), f))
+		sscanf(buf, "%d\n", &n_batteries);
+
+	if (n_batteries < 2)
+		apm_battery_assign(0, bios_major, bios_minor, ac_line_status,
+					battery_status, battery_flag, percent, time_left, units);
+	else
 		{
-		/* Handle dual APM battery patch.
-		*/
+		apm_battery_assign(GKRELLM_BATTERY_COMPOSITE_ID,
+					bios_major, bios_minor, ac_line_status,
+					battery_status, battery_flag, percent, time_left, units);
 		while (n_batteries-- > 0 && fgets(buf, sizeof(buf), f))
 			{
 			sscanf(buf, "%d %x %x %d%% %d %32s\n", &id,
@@ -1596,9 +1766,6 @@ apm_battery_data(void)
 					battery_status, battery_flag, percent, time_left, units);
 			}
 		}
-	else
-		apm_battery_assign(0, bios_major, bios_minor, ac_line_status,
-				battery_status, battery_flag, percent, time_left, units);
 
 	fclose(f);
 	return TRUE;
@@ -1658,6 +1825,7 @@ gkrellm_sys_uptime_init(void)
 /* ------- Linux ------------------------------------------------------- */
 
 #define	SENSORS_DIR	"/proc/sys/dev/sensors"
+#define SYSFS_I2C_DIR	"/sys/bus/i2c/devices"
 
 typedef struct
 	{
@@ -1784,6 +1952,76 @@ static VoltDefault	voltdefault9[] =
 	{ "+3.3V",	1.0,    0, 		NULL }
 	};
 
+static gboolean
+sysfs_get_sensor(gchar *device_name, gint id, gint type, gfloat *val)
+	{
+	gchar		*path, *p, *name = NULL;
+	FILE		*f;
+	gfloat		V = 0.0;
+	gint		t, n;
+	gboolean	ret = FALSE;
+
+	/* get file name back */
+	path = g_strdup(device_name);
+	if (!path)
+		goto out;
+	p = strrchr(path, '/');
+	if (!p)
+		goto free_path;
+	*p = '\0';
+	p = strrchr(path, '/');
+	if (!p)
+		goto free_path;
+	*p = '\0';
+
+	switch (type)
+		{
+		case SENSOR_VOLTAGE:
+			name = g_strdup_printf("%s/%s%d", path, "in_input", id); 
+			break;
+		case SENSOR_FAN:
+			name = g_strdup_printf("%s/%s%d", path, "fan_input", id); 
+			break;
+		case SENSOR_TEMPERATURE:
+			name = g_strdup_printf("%s/%s%d", path, "temp_input", id); 
+			break;
+		}
+	if (!name)
+		goto free_path;
+
+	if ((f = fopen(name, "r")) == NULL)
+		goto free_name;
+	n = fscanf(f, "%d", &t);
+	fclose(f);
+
+	if (n < 1)
+		goto free_name;
+
+	switch (type)
+		{
+		case SENSOR_VOLTAGE:
+			V = (gfloat) t / 1000.0;
+			break;
+		case SENSOR_TEMPERATURE:
+			V = (gfloat) t / 100.0;
+			break;
+		case SENSOR_FAN:
+			V = (gfloat) t;
+			break;
+		}
+
+	if (val)
+		*val = V;
+	ret = TRUE;
+
+free_name:
+	g_free(name);
+free_path:
+	g_free(path);
+out:
+	return ret;
+}
+
 gboolean
 gkrellm_sys_sensors_get_temperature(gchar *device_name, gint id,
 		gint iodev, gint interface, gfloat *temp)
@@ -1793,6 +2031,9 @@ gkrellm_sys_sensors_get_temperature(gchar *device_name, gint id,
 	gint	n;
 	gfloat	T, t[5];
 
+	if (have_sysfs_sensors)
+		return sysfs_get_sensor(device_name, id, SENSOR_TEMPERATURE, temp);
+	
 	if ((f = fopen(device_name, "r")) == NULL)
 		return FALSE;
 	fgets(buf, sizeof(buf), f);
@@ -1821,6 +2062,9 @@ gkrellm_sys_sensors_get_fan(gchar *device_name, gint id,
 	gint	n;
 	gfloat	t[4];
 
+	if (have_sysfs_sensors)
+		return sysfs_get_sensor(device_name, id, SENSOR_FAN, fan);
+
 	if ((f = fopen(device_name, "r")) == NULL)
 		return FALSE;
 	fgets(buf, sizeof(buf), f);
@@ -1846,6 +2090,9 @@ gkrellm_sys_sensors_get_voltage(gchar *device_name, gint id,
 	gfloat	V, t[3];
 	gint	n;
 
+	if (have_sysfs_sensors)
+		return sysfs_get_sensor(device_name, id, SENSOR_VOLTAGE, volt);
+
 	if ((f = fopen(device_name, "r")) == NULL)
 		return FALSE;
 	fgets(buf, sizeof(buf), f);
@@ -1926,18 +2173,157 @@ get_volt_default(gchar *chip_name, VoltDefault **vdf, gint *vdfsize)
 	}
 
 
+static gchar *
+sysfs_get_chip_name(gchar *dir)
+	{
+	gchar	*name, buf[256], *p, *chip;
+	FILE 	*f;
+
+	name = g_strdup_printf("%s/%s/%s", SYSFS_I2C_DIR, dir, "name");
+	if (!name)
+		return NULL;
+	f = fopen(name, "r");
+	g_free(name);
+	if (!f)
+		return NULL;
+
+	if (fgets(buf, sizeof(buf), f) == NULL)
+		{
+		fclose(f);
+		return NULL;
+		}
+	fclose(f);
+	
+	if (buf[strlen(buf) - 1] == '\n')
+		buf[strlen(buf) - 1] = '\0';
+
+	p = strchr(buf, ' ');
+	if (!p)
+		return NULL;
+
+	*p++ = '\0';
+	/* FIXME anybody has better idea how to detect sensor chip? */
+	if (strcmp(p, "chip") && strcmp(p, "subclient"))
+		return NULL;
+
+	chip = g_strdup(buf);
+	for (p = chip; *p; p++)
+		*p = tolower(*p);
+	return chip;
+	}
+
+static gboolean
+sysfs_sensors_init(void)
+	{
+	DIR				*dir, *chip_dir;
+	struct dirent	*dentry, *chip_dentry;
+	VoltDefault		*voltdefault;
+	gint			id = 0, bus;
+	guint			addr;
+	gint			type, voltdefaultsize, n_volt_sensors;
+	gfloat			factor, offset;
+	gchar			*name, *default_label, *vref, *compat,
+					*base_name,  *chip_name;
+	gchar			path[256];
+
+	if (!have_sysfs)
+		return TRUE;
+
+	if ((dir = opendir(SYSFS_I2C_DIR)) == NULL)
+		return TRUE;
+	n_volt_sensors = 0;
+	while ((dentry = readdir(dir)) != NULL)
+		{
+		if (dentry->d_name[0] == '.' || dentry->d_ino <= 0)
+			continue;
+		snprintf(path, sizeof(path), "%s/%s", SYSFS_I2C_DIR, dentry->d_name);
+		if ((chip_dir = opendir(path)) == NULL)
+			continue;
+		if ((chip_name = sysfs_get_chip_name(dentry->d_name)) == NULL)
+			{
+			closedir(chip_dir);
+			continue;
+			}
+		have_sysfs_sensors = TRUE;
+		get_volt_default(chip_name, &voltdefault, &voltdefaultsize);
+		while ((chip_dentry = readdir(chip_dir)) != NULL)
+			{
+			name = chip_dentry->d_name;
+			if (   !strncmp(name, "temp_input", 10)
+				&& isdigit(name[10]) && !name[11]
+			   )
+				{
+				type = SENSOR_TEMPERATURE;
+				id = name[10] - '0';
+				compat = "temp";
+				}
+			else if (   !strncmp(name, "fan_input", 9)
+					 && isdigit(name[9]) && !name[10]
+				    )
+				{
+				type = SENSOR_FAN;
+				id = name[9] - '0';
+				compat = "fan";
+				}
+			else if (   !strncmp(name, "in_input", 8)
+					 && isdigit(name[8]) && !name[9]
+					)
+				{
+				type = SENSOR_VOLTAGE;
+				id = name[8] - '0';
+				compat = "in";
+				}
+			else
+				continue;
+
+			factor = 1.0;
+			offset = 0;
+			default_label = vref = NULL;
+
+			if (type == SENSOR_VOLTAGE)
+				{
+				if (id < voltdefaultsize)
+					{
+					default_label = voltdefault[id].name;
+					factor = voltdefault[id].factor;
+					offset = voltdefault[id].offset;
+					vref = voltdefault[id].vref;
+					}
+				else
+					default_label =  compat;
+				}
+			/* the following is kludge to make config compatible with /proc
+			*/
+			sscanf(dentry->d_name, "%d-%x", &bus, &addr);
+			base_name = g_strdup_printf("%s-i2c-%d-%x/%s%d",
+						chip_name, bus, addr,
+						compat, id);
+			gkrellm_sensors_add_sensor(type, path, base_name,
+						id, 0, 0,
+						factor, offset, vref, default_label);
+			g_free(base_name);
+
+			if (_GK.debug_level & DEBUG_SENSORS)
+				printf("sys_sensors_init: %s/%s %d\n", path, name, type);
+			}
+		g_free(chip_name);
+		closedir(chip_dir);
+		}
+	closedir(dir);
+	return TRUE;
+	}
+
 gboolean
 gkrellm_sys_sensors_init(void)
 	{
 	DIR				*dir, *chip_dir;
 	struct dirent	*dentry, *chip_dentry;
-	FILE			*f, *f1;
 	VoltDefault		*voltdefault;
 	gint			id = 0;
-	gint			type, divisor, voltdefaultsize, n_volt_sensors;
+	gint			type, voltdefaultsize, n_volt_sensors;
 	gfloat			factor, offset;
 	gchar			*name, *default_label, *vref;
-	gchar			path[256], base_name[128], fan_div[256];
+	gchar			path[256], base_name[128];
 	struct lconv	*lc;
 
 	lc = localeconv();
@@ -1946,7 +2332,10 @@ gkrellm_sys_sensors_init(void)
 		need_locale_fix = TRUE;
 
 	if ((dir = opendir(SENSORS_DIR)) == NULL)
+		{
+		sysfs_sensors_init();
 		return TRUE;
+		}
 	n_volt_sensors = 0;
 	while ((dentry = readdir(dir)) != NULL)
 		{
@@ -1956,8 +2345,6 @@ gkrellm_sys_sensors_init(void)
 		if ((chip_dir = opendir(path)) == NULL)
 			continue;
 		get_volt_default(dentry->d_name, &voltdefault, &voltdefaultsize);
-		snprintf(fan_div, sizeof(fan_div), "%s/%s", path, "fan_div");
-		f = fopen(fan_div, "r");
 		while ((chip_dentry = readdir(chip_dir)) != NULL)
 			{
 			name = chip_dentry->d_name;
@@ -1989,22 +2376,6 @@ gkrellm_sys_sensors_init(void)
 			offset = 0;
 			default_label = vref = NULL;
 
-			if (type == SENSOR_FAN)
-				{
-				divisor = 0;
-				if (!f || fscanf(f, "%d", &divisor) != 1)
-					{
-					snprintf(fan_div, sizeof(fan_div), "%s/%s_div",
-							path, name);
-					if ((f1 = fopen(fan_div, "r")) != NULL)
-						{
-						fscanf(f1, "%d", &divisor);
-						fclose(f1);
-						}
-					}
-				if (divisor > 0)
-					factor /= (gfloat) divisor;
-				}
 			if (type == SENSOR_VOLTAGE)
 				{
 				if (id < voltdefaultsize)
@@ -2027,8 +2398,6 @@ gkrellm_sys_sensors_init(void)
 				printf("sys_sensors_init: %s/%s %d\n", path, name, type);
 			}
 		closedir(chip_dir);
-		if (f)
-			fclose(f);
 		}
 	closedir(dir);
 	return TRUE;
