@@ -126,7 +126,7 @@ gkrellm_sys_proc_read_data(void)
 void
 gkrellm_sys_proc_read_users(void)
 	{
-	gint	n_user;
+	gint	n_users;
    static time_t utmp_mtime;
    struct utmp utmp;
    struct stat s;
@@ -191,6 +191,7 @@ static VoltDefault	voltdefault0[] =
  *
  * Perform sensor reading
  */
+#include <sys/ioctl.h>
 
 gboolean
 gkrellm_sys_sensors_get_temperature(gchar *path, gint id,
@@ -217,7 +218,7 @@ gkrellm_sys_sensors_get_fan(gchar *path, gint id,
 {
    envsys_tre_data_t data;	/* sensor data */
 
-   data.sensor = sensor->interface;
+   data.sensor = interface;
    if (ioctl(sensor->iodev, ENVSYS_GTREDATA, &data) < 0) return FALSE;
    if (!(data.validflags & (ENVSYS_FVALID|ENVSYS_FCURVALID))) return FALSE;
 
@@ -235,7 +236,7 @@ gkrellm_sys_sensors_get_voltage(gchar *path, gint id,
 {
    envsys_tre_data_t data;	/* sensor data */
 
-   data.sensor = sensor->interface;
+   data.sensor = interface;
    if (ioctl(sensor->iodev, ENVSYS_GTREDATA, &data) < 0) return FALSE;
    if (!(data.validflags & (ENVSYS_FVALID|ENVSYS_FCURVALID))) return FALSE;
 
