@@ -204,6 +204,11 @@ void
 gkrellm_winop_state_above(gboolean state)
 	{
 	net_wm_state("_NET_WM_STATE_ABOVE", state);
+	/* Apparently KDE 3.1.0 and possibly below does not implement
+	 * _NET_WM_STATE_ABOVE but _NET_WM_STATE_STAYS_ON_TOP that implies
+	 * approximately the same thing
+	 */
+	net_wm_state("_NET_WM_STATE_STAYS_ON_TOP", state);
 	}
 
 void
@@ -245,10 +250,13 @@ gkrellm_winop_options(gint argc, gchar **argv)
 	{
 	Display		*display;
 	Window		window;
-	Atom		atoms[3];
+	Atom		atoms[4];
 	gint		n = 0;
 
-	if (!_GK.allow_multiple_instances_real && !_gkrellm_get_lock())
+	if (   !_GK.allow_multiple_instances_real
+		&& !_GK.force_host_config
+		&& !_gkrellm_get_lock()
+	   )
 		{
 		printf("gkrellm: %s\n",
 			_("Exiting because multiple instances option is off.\n"));
@@ -285,8 +293,9 @@ gkrellm_winop_options(gint argc, gchar **argv)
 		}
 	if (_GK.state_above)
 		{
-		atoms[n] = XInternAtom(display, "_NET_WM_STATE_ABOVE", False);
-		++n;
+		atoms[n++] = XInternAtom(display, "_NET_WM_STATE_ABOVE", False);
+		/* see gkrellm_winop_state_above() */
+		atoms[n++] = XInternAtom(display, "_NET_WM_STATE_STAYS_ON_TOP", False);
 		_GK.state_below = FALSE;
 		}
 	if (_GK.state_below)
